#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "generate_cubed_sphere.h"
#include "grid_unstruct.h"
#include "geometry.h"

// This routine is based on Mathlab code provided by Mike Hobson and written by
// Mike Rezny (both from MetOffice)
void generate_cubed_sphere_grid_information(
  unsigned n, unsigned * num_cells, unsigned * num_vertices,
  double ** x_vertices, double ** y_vertices, double ** z_vertices,
  unsigned ** cell_to_vertex, unsigned ** face_id) {

  if (n < 1) {
    fputs("invalid number of linear subdivisions", stderr);
    exit(EXIT_FAILURE);
  }

  *num_cells = n * n * 6;
  *num_vertices = *num_cells + 2;

  // allocation of output variables
  *x_vertices = malloc(*num_vertices * sizeof(**x_vertices));
  *y_vertices = malloc(*num_vertices * sizeof(**y_vertices));
  *z_vertices = malloc(*num_vertices * sizeof(**z_vertices));

  *cell_to_vertex = malloc(*num_cells * 4 * sizeof(**cell_to_vertex));

  *face_id = malloc(*num_cells * sizeof(**face_id));

  // allocation of temporary coordinate variables
  unsigned num_edge_coords = n - 1;
  double * cube_edge_x_vertices = malloc(12 * num_edge_coords * sizeof(*cube_edge_x_vertices));
  double * cube_edge_y_vertices = malloc(12 * num_edge_coords * sizeof(*cube_edge_y_vertices));
  double * cube_edge_z_vertices = malloc(12 * num_edge_coords * sizeof(*cube_edge_z_vertices));

  unsigned num_inner_coords = num_edge_coords * num_edge_coords;
  double * cube_inner_x_vertices = malloc(num_inner_coords * sizeof(*cube_inner_x_vertices));
  double * cube_inner_y_vertices = malloc(num_inner_coords * sizeof(*cube_inner_y_vertices));
  double * cube_inner_z_vertices = malloc(num_inner_coords * sizeof(*cube_inner_z_vertices));

  double * temp_x_vertices = malloc((n + 1) * (n + 1) * sizeof(*temp_x_vertices));
  double * temp_y_vertices = malloc((n + 1) * (n + 1) * sizeof(*temp_y_vertices));
  double * temp_z_vertices = malloc((n + 1) * (n + 1) * sizeof(*temp_z_vertices));

  {
    double * theta = malloc((n + 1) * sizeof(*theta));

    {
      double d = (M_PI_2 / (double)n);
      for (unsigned i = 0; i < n + 1; ++i)
        theta[i] = tan(- M_PI_4 + d * (double)i);
    }

    for (unsigned i = 0; i < n + 1; ++i) {
      for (unsigned j = 0; j < n + 1; ++j) {

        double scale =
          sqrt(1.0 / (theta[i] * theta[i] + theta[j] * theta[j] + 1.0));

        temp_x_vertices[i * (n + 1) + j] = theta[j] * scale;
        temp_y_vertices[i * (n + 1) + j] = theta[i] * scale;
        temp_z_vertices[i * (n + 1) + j] = - scale;
      }
    }
    free(theta);
  }

  // Store the coordinates for the 4 vertices on face 1
  (*x_vertices)[0] = temp_x_vertices[ 0 * (n + 1) + 0];
  (*y_vertices)[0] = temp_y_vertices[ 0 * (n + 1) + 0];
  (*z_vertices)[0] = temp_z_vertices[ 0 * (n + 1) + 0];
  (*x_vertices)[1] = temp_x_vertices[ n * (n + 1) + 0];
  (*y_vertices)[1] = temp_y_vertices[ n * (n + 1) + 0];
  (*z_vertices)[1] = temp_z_vertices[ n * (n + 1) + 0];
  (*x_vertices)[2] = temp_x_vertices[ 0 * (n + 1) + n];
  (*y_vertices)[2] = temp_y_vertices[ 0 * (n + 1) + n];
  (*z_vertices)[2] = temp_z_vertices[ 0 * (n + 1) + n];
  (*x_vertices)[3] = temp_x_vertices[ n * (n + 1) + n];
  (*y_vertices)[3] = temp_y_vertices[ n * (n + 1) + n];
  (*z_vertices)[3] = temp_z_vertices[ n * (n + 1) + n];
  // Store the coordinates for the 4 vertices on face 2
  (*x_vertices)[4] =  temp_x_vertices[ 0 * (n + 1) + 0];
  (*y_vertices)[4] =  temp_y_vertices[ 0 * (n + 1) + 0];
  (*z_vertices)[4] = -temp_z_vertices[ 0 * (n + 1) + 0];
  (*x_vertices)[5] =  temp_x_vertices[ n * (n + 1) + 0];
  (*y_vertices)[5] =  temp_y_vertices[ n * (n + 1) + 0];
  (*z_vertices)[5] = -temp_z_vertices[ n * (n + 1) + 0];
  (*x_vertices)[6] =  temp_x_vertices[ 0 * (n + 1) + n];
  (*y_vertices)[6] =  temp_y_vertices[ 0 * (n + 1) + n];
  (*z_vertices)[6] = -temp_z_vertices[ 0 * (n + 1) + n];
  (*x_vertices)[7] =  temp_x_vertices[ n * (n + 1) + n];
  (*y_vertices)[7] =  temp_y_vertices[ n * (n + 1) + n];
  (*z_vertices)[7] = -temp_z_vertices[ n * (n + 1) + n];

  // Store the coordinates for the edges
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    cube_edge_x_vertices[ 0 * num_edge_coords + i] = temp_x_vertices[(1 + i) * (n + 1) + 0];
    cube_edge_y_vertices[ 0 * num_edge_coords + i] = temp_y_vertices[(1 + i) * (n + 1) + 0];
    cube_edge_z_vertices[ 0 * num_edge_coords + i] = temp_z_vertices[(1 + i) * (n + 1) + 0];
    cube_edge_x_vertices[ 1 * num_edge_coords + i] = temp_x_vertices[0 * (n + 1) + (1 + i)];
    cube_edge_y_vertices[ 1 * num_edge_coords + i] = temp_y_vertices[0 * (n + 1) + (1 + i)];
    cube_edge_z_vertices[ 1 * num_edge_coords + i] = temp_z_vertices[0 * (n + 1) + (1 + i)];
    cube_edge_x_vertices[ 2 * num_edge_coords + i] = temp_x_vertices[n * (n + 1) + (1 + i)];
    cube_edge_y_vertices[ 2 * num_edge_coords + i] = temp_y_vertices[n * (n + 1) + (1 + i)];
    cube_edge_z_vertices[ 2 * num_edge_coords + i] = temp_z_vertices[n * (n + 1) + (1 + i)];
    cube_edge_x_vertices[ 3 * num_edge_coords + i] = temp_x_vertices[(1 + i) * (n + 1) + n];
    cube_edge_y_vertices[ 3 * num_edge_coords + i] = temp_y_vertices[(1 + i) * (n + 1) + n];
    cube_edge_z_vertices[ 3 * num_edge_coords + i] = temp_z_vertices[(1 + i) * (n + 1) + n];
    cube_edge_x_vertices[ 4 * num_edge_coords + i] =  temp_x_vertices[(1 + i) * (n + 1) + 0];
    cube_edge_y_vertices[ 4 * num_edge_coords + i] =  temp_y_vertices[(1 + i) * (n + 1) + 0];
    cube_edge_z_vertices[ 4 * num_edge_coords + i] = -temp_z_vertices[(1 + i) * (n + 1) + 0];
    cube_edge_x_vertices[ 5 * num_edge_coords + i] =  temp_x_vertices[0 * (n + 1) + (1 + i)];
    cube_edge_y_vertices[ 5 * num_edge_coords + i] =  temp_y_vertices[0 * (n + 1) + (1 + i)];
    cube_edge_z_vertices[ 5 * num_edge_coords + i] = -temp_z_vertices[0 * (n + 1) + (1 + i)];
    cube_edge_x_vertices[ 6 * num_edge_coords + i] =  temp_x_vertices[n * (n + 1) + (1 + i)];
    cube_edge_y_vertices[ 6 * num_edge_coords + i] =  temp_y_vertices[n * (n + 1) + (1 + i)];
    cube_edge_z_vertices[ 6 * num_edge_coords + i] = -temp_z_vertices[n * (n + 1) + (1 + i)];
    cube_edge_x_vertices[ 7 * num_edge_coords + i] =  temp_x_vertices[(1 + i) * (n + 1) + n];
    cube_edge_y_vertices[ 7 * num_edge_coords + i] =  temp_y_vertices[(1 + i) * (n + 1) + n];
    cube_edge_z_vertices[ 7 * num_edge_coords + i] = -temp_z_vertices[(1 + i) * (n + 1) + n];
    cube_edge_x_vertices[ 8 * num_edge_coords + i] =  temp_z_vertices[0 * (n + 1) + (1 + i)];
    cube_edge_y_vertices[ 8 * num_edge_coords + i] =  temp_y_vertices[0 * (n + 1) + (1 + i)];
    cube_edge_z_vertices[ 8 * num_edge_coords + i] =  temp_x_vertices[0 * (n + 1) + (1 + i)];
    cube_edge_x_vertices[ 9 * num_edge_coords + i] =  temp_z_vertices[n * (n + 1) + (1 + i)];
    cube_edge_y_vertices[ 9 * num_edge_coords + i] =  temp_y_vertices[n * (n + 1) + (1 + i)];
    cube_edge_z_vertices[ 9 * num_edge_coords + i] =  temp_x_vertices[n * (n + 1) + (1 + i)];
    cube_edge_x_vertices[10 * num_edge_coords + i] = -temp_z_vertices[0 * (n + 1) + (1 + i)];
    cube_edge_y_vertices[10 * num_edge_coords + i] =  temp_y_vertices[0 * (n + 1) + (1 + i)];
    cube_edge_z_vertices[10 * num_edge_coords + i] =  temp_x_vertices[0 * (n + 1) + (1 + i)];
    cube_edge_x_vertices[11 * num_edge_coords + i] = -temp_z_vertices[n * (n + 1) + (1 + i)];
    cube_edge_y_vertices[11 * num_edge_coords + i] =  temp_y_vertices[n * (n + 1) + (1 + i)];
    cube_edge_z_vertices[11 * num_edge_coords + i] =  temp_x_vertices[n * (n + 1) + (1 + i)];
  }
  
  // Move the 12 edges to the final vertices array
  unsigned Estart = 9 - 1;
  unsigned Eend = Estart + 12 * num_edge_coords;
  for (unsigned i = 0; i < 12 * num_edge_coords; ++i) {
    (*x_vertices)[i + Estart] = cube_edge_x_vertices[i];
    (*y_vertices)[i + Estart] = cube_edge_y_vertices[i];
    (*z_vertices)[i + Estart] = cube_edge_z_vertices[i];
  }

  free(cube_edge_x_vertices);
  free(cube_edge_y_vertices);
  free(cube_edge_z_vertices);

  // store the internal vertices for face 1
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    for (unsigned j = 0; j < num_edge_coords; ++j) {

      cube_inner_x_vertices[i * num_edge_coords + j] = temp_x_vertices[(1 + i) * (n + 1) + (1 + j)];
      cube_inner_y_vertices[i * num_edge_coords + j] = temp_y_vertices[(1 + i) * (n + 1) + (1 + j)];
      cube_inner_z_vertices[i * num_edge_coords + j] = temp_z_vertices[(1 + i) * (n + 1) + (1 + j)];
    }
  }

  // Move face 1 to final Vertices array
  unsigned Fstart  = Eend;
  unsigned Fend    = Fstart + num_inner_coords;
  for (unsigned i = 0; i < num_inner_coords; ++i) {
    (*x_vertices)[i + Fstart] = cube_inner_x_vertices[i];
    (*y_vertices)[i + Fstart] = cube_inner_y_vertices[i];
    (*z_vertices)[i + Fstart] = cube_inner_z_vertices[i];
  }

  // store the internal vertices for face 2
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    for (unsigned j = 0; j < num_edge_coords; ++j) {

      cube_inner_x_vertices[i * num_edge_coords + j] =  temp_x_vertices[(1 + i) * (n + 1) + (1 + j)];
      cube_inner_y_vertices[i * num_edge_coords + j] =  temp_y_vertices[(1 + i) * (n + 1) + (1 + j)];
      cube_inner_z_vertices[i * num_edge_coords + j] = -temp_z_vertices[(1 + i) * (n + 1) + (1 + j)];
    }
  }

  // Move face 2 to final Vertices array
  Fstart += num_inner_coords;
  Fend   += num_inner_coords;
  for (unsigned i = 0; i < num_inner_coords; ++i) {
    (*x_vertices)[i + Fstart] = cube_inner_x_vertices[i];
    (*y_vertices)[i + Fstart] = cube_inner_y_vertices[i];
    (*z_vertices)[i + Fstart] = cube_inner_z_vertices[i];
  }

  // store the internal vertices for face 3
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    for (unsigned j = 0; j < num_edge_coords; ++j) {

      cube_inner_x_vertices[i * num_edge_coords + j] = temp_z_vertices[(1 + i) * (n + 1) + (1 + j)];
      cube_inner_y_vertices[i * num_edge_coords + j] = temp_y_vertices[(1 + i) * (n + 1) + (1 + j)];
      cube_inner_z_vertices[i * num_edge_coords + j] = temp_x_vertices[(1 + i) * (n + 1) + (1 + j)];
    }
  }

  // Move face 3 to final Vertices array
  Fstart += num_inner_coords;
  Fend   += num_inner_coords;
  for (unsigned i = 0; i < num_inner_coords; ++i) {
    (*x_vertices)[i + Fstart] = cube_inner_x_vertices[i];
    (*y_vertices)[i + Fstart] = cube_inner_y_vertices[i];
    (*z_vertices)[i + Fstart] = cube_inner_z_vertices[i];
  }

  // store the internal vertices for face 4
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    for (unsigned j = 0; j < num_edge_coords; ++j) {

      cube_inner_x_vertices[i * num_edge_coords + j] = -temp_z_vertices[(1 + i) * (n + 1) + (1 + j)];
      cube_inner_y_vertices[i * num_edge_coords + j] =  temp_y_vertices[(1 + i) * (n + 1) + (1 + j)];
      cube_inner_z_vertices[i * num_edge_coords + j] =  temp_x_vertices[(1 + i) * (n + 1) + (1 + j)];
    }
  }

  // Move face 4 to final Vertices array
  Fstart += num_inner_coords;
  Fend   += num_inner_coords;
  for (unsigned i = 0; i < num_inner_coords; ++i) {
    (*x_vertices)[i + Fstart] = cube_inner_x_vertices[i];
    (*y_vertices)[i + Fstart] = cube_inner_y_vertices[i];
    (*z_vertices)[i + Fstart] = cube_inner_z_vertices[i];
  }

  // store the internal vertices for face 5
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    for (unsigned j = 0; j < num_edge_coords; ++j) {

      cube_inner_x_vertices[i * num_edge_coords + j] = temp_x_vertices[(1 + i) * (n + 1) + (1 + j)];
      cube_inner_y_vertices[i * num_edge_coords + j] = temp_z_vertices[(1 + i) * (n + 1) + (1 + j)];
      cube_inner_z_vertices[i * num_edge_coords + j] = temp_y_vertices[(1 + i) * (n + 1) + (1 + j)];
    }
  }

  // Move face 5 to final Vertices array
  Fstart += num_inner_coords;
  Fend   += num_inner_coords;
  for (unsigned i = 0; i < num_inner_coords; ++i) {
    (*x_vertices)[i + Fstart] = cube_inner_x_vertices[i];
    (*y_vertices)[i + Fstart] = cube_inner_y_vertices[i];
    (*z_vertices)[i + Fstart] = cube_inner_z_vertices[i];
  }

  // store the internal vertices for face 6
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    for (unsigned j = 0; j < num_edge_coords; ++j) {

      cube_inner_x_vertices[i * num_edge_coords + j] =  temp_x_vertices[(1 + i) * (n + 1) + (1 + j)];
      cube_inner_y_vertices[i * num_edge_coords + j] = -temp_z_vertices[(1 + i) * (n + 1) + (1 + j)];
      cube_inner_z_vertices[i * num_edge_coords + j] =  temp_y_vertices[(1 + i) * (n + 1) + (1 + j)];
    }
  }

  // Move face 6 to final Vertices array
  Fstart += num_inner_coords;
  Fend   += num_inner_coords;
  for (unsigned i = 0; i < num_inner_coords; ++i) {
    (*x_vertices)[i + Fstart] = cube_inner_x_vertices[i];
    (*y_vertices)[i + Fstart] = cube_inner_y_vertices[i];
    (*z_vertices)[i + Fstart] = cube_inner_z_vertices[i];
  }

  free(temp_x_vertices);
  free(temp_y_vertices);
  free(temp_z_vertices);
  free(cube_inner_x_vertices);
  free(cube_inner_y_vertices);
  free(cube_inner_z_vertices);

  for (unsigned i = 0; i < 6; ++i)
    for (unsigned j = 0; j < n * n; ++j)
      (*face_id)[i * n * n + j] = i + 1;

  unsigned * edge_vertices = malloc(num_edge_coords * 12 * sizeof(*edge_vertices));

  Fstart = Estart + 12 * num_edge_coords;

  for (unsigned i = 0; i < num_edge_coords * 12; ++i) edge_vertices[i] = i + Estart;

  unsigned * F = malloc((n + 1) * (n + 1) * sizeof(*F));
  unsigned cell_to_vertex_offset = 0;

// Calculate the indices for all the vertices on face 1
//        v2-----e3-----v4
//        |              |
//        e1     f1     e4
//        |              |
//        v1-----e2-----v3

  F[0] = 0; F[n] = 2; F[n * (n + 1) + 0] = 1; F[n * (n + 1) + n] = 3;
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    F[i + 1] =                 edge_vertices[1 * num_edge_coords + i];
    F[(i + 1) * (n + 1)] =     edge_vertices[0 * num_edge_coords + i];
    F[(i + 1) * (n + 1) + n] = edge_vertices[3 * num_edge_coords + i];
    F[n * (n + 1) + i + 1] =   edge_vertices[2 * num_edge_coords + i];
  }

  for (unsigned i = 0; i < n-1; ++i)
    for (unsigned j = 0; j < n-1; ++j)
      F[(i + 1) * (n + 1) + (j + 1)] = Fstart + i * (n - 1) + j;

  for (unsigned i = 0; i < n; ++i) {
    for (unsigned j = 0; j < n; ++j) {
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 0) * (n + 1) + (j + 0)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 1) * (n + 1) + (j + 0)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 1) * (n + 1) + (j + 1)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 0) * (n + 1) + (j + 1)];
    }
  }

// Calculate the indices for all the vertices on face 2
//        v6-----e7-----v8
//        |              |
//        e5     f2     e8
//        |              |
//        v5-----e6-----v7

  F[0] = 4; F[n] = 6; F[n * (n + 1) + 0] = 5; F[n * (n + 1) + n] = 7;
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    F[i + 1] =                 edge_vertices[5 * num_edge_coords + i];
    F[(i + 1) * (n + 1)] =     edge_vertices[4 * num_edge_coords + i];
    F[(i + 1) * (n + 1) + n] = edge_vertices[7 * num_edge_coords + i];
    F[n * (n + 1) + i + 1] =   edge_vertices[6 * num_edge_coords + i];
  }

  for (unsigned i = 0; i < n-1; ++i)
    for (unsigned j = 0; j < n-1; ++j)
      F[(i + 1) * (n + 1) + (j + 1)] += (n - 1) * (n - 1);

  for (unsigned i = 0; i < n; ++i) {
    for (unsigned j = 0; j < n; ++j) {
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 0) * (n + 1) + (j + 0)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 1) * (n + 1) + (j + 0)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 1) * (n + 1) + (j + 1)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 0) * (n + 1) + (j + 1)];
    }
  }

// Calculate the indices for all the vertices on face 3
//        v2-----e10----v6
//        |              |
//        e1     f3     e5
//        |              |
//        v1-----e9-----v5

  F[0] = 0; F[n] = 4; F[n * (n + 1) + 0] = 1; F[n * (n + 1) + n] = 5;
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    F[i + 1] =                 edge_vertices[8 * num_edge_coords + i];
    F[(i + 1) * (n + 1)] =     edge_vertices[0 * num_edge_coords + i];
    F[(i + 1) * (n + 1) + n] = edge_vertices[4 * num_edge_coords + i];
    F[n * (n + 1) + i + 1] =   edge_vertices[9 * num_edge_coords + i];
  }

  for (unsigned i = 0; i < n-1; ++i)
    for (unsigned j = 0; j < n-1; ++j)
      F[(i + 1) * (n + 1) + (j + 1)] += (n - 1) * (n - 1);

  for (unsigned i = 0; i < n; ++i) {
    for (unsigned j = 0; j < n; ++j) {
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 0) * (n + 1) + (j + 0)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 1) * (n + 1) + (j + 0)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 1) * (n + 1) + (j + 1)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 0) * (n + 1) + (j + 1)];
    }
  }

// Calculate the indices for all the vertices on face 4
//        v4-----e12----v8
//        |              |
//        e4     f4     e8
//        |              |
//        v3-----e11----v7

  F[0] = 2; F[n] = 6; F[n * (n + 1) + 0] = 3; F[n * (n + 1) + n] = 7;
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    F[i + 1] =                 edge_vertices[10 * num_edge_coords + i];
    F[(i + 1) * (n + 1)] =     edge_vertices[ 3 * num_edge_coords + i];
    F[(i + 1) * (n + 1) + n] = edge_vertices[ 7 * num_edge_coords + i];
    F[n * (n + 1) + i + 1] =   edge_vertices[11 * num_edge_coords + i];
  }

  for (unsigned i = 0; i < n-1; ++i)
    for (unsigned j = 0; j < n-1; ++j)
      F[(i + 1) * (n + 1) + (j + 1)] += (n - 1) * (n - 1);

  for (unsigned i = 0; i < n; ++i) {
    for (unsigned j = 0; j < n; ++j) {
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 0) * (n + 1) + (j + 0)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 1) * (n + 1) + (j + 0)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 1) * (n + 1) + (j + 1)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 0) * (n + 1) + (j + 1)];
    }
  }

// Calculate the indices for all the vertices on face 5
//        v5-----e6-----v7
//        |              |
//        e9     f5     e11
//        |              |
//        v1-----e2-----v3

  F[0] = 0; F[n] = 2; F[n * (n + 1) + 0] = 4; F[n * (n + 1) + n] = 6;
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    F[i + 1] =                 edge_vertices[ 1 * num_edge_coords + i];
    F[(i + 1) * (n + 1)] =     edge_vertices[ 8 * num_edge_coords + i];
    F[(i + 1) * (n + 1) + n] = edge_vertices[10 * num_edge_coords + i];
    F[n * (n + 1) + i + 1] =   edge_vertices[ 5 * num_edge_coords + i];
  }

  for (unsigned i = 0; i < n-1; ++i)
    for (unsigned j = 0; j < n-1; ++j)
      F[(i + 1) * (n + 1) + (j + 1)] += (n - 1) * (n - 1);

  for (unsigned i = 0; i < n; ++i) {
    for (unsigned j = 0; j < n; ++j) {
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 0) * (n + 1) + (j + 0)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 1) * (n + 1) + (j + 0)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 1) * (n + 1) + (j + 1)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 0) * (n + 1) + (j + 1)];
    }
  }

// Calculate the indices for all the vertices on face 6
//        v6-----e7-----v8
//        |              |
//       e10     f6     e12
//        |              |
//        v2-----e3-----v4

  F[0] = 1; F[n] = 3; F[n * (n + 1) + 0] = 5; F[n * (n + 1) + n] = 7;
  for (unsigned i = 0; i < num_edge_coords; ++i) {
    F[i + 1] =                 edge_vertices[ 2 * num_edge_coords + i];
    F[(i + 1) * (n + 1)] =     edge_vertices[ 9 * num_edge_coords + i];
    F[(i + 1) * (n + 1) + n] = edge_vertices[11 * num_edge_coords + i];
    F[n * (n + 1) + i + 1] =   edge_vertices[ 6 * num_edge_coords + i];
  }

  for (unsigned i = 0; i < n-1; ++i)
    for (unsigned j = 0; j < n-1; ++j)
      F[(i + 1) * (n + 1) + (j + 1)] += (n - 1) * (n - 1);

  for (unsigned i = 0; i < n; ++i) {
    for (unsigned j = 0; j < n; ++j) {
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 0) * (n + 1) + (j + 0)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 1) * (n + 1) + (j + 0)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 1) * (n + 1) + (j + 1)];
      (*cell_to_vertex)[cell_to_vertex_offset++] = F[(i + 0) * (n + 1) + (j + 1)];
    }
  }

  free(edge_vertices);
  free(F);
}

struct grid * generate_cubed_sphere_grid(unsigned n) {

  unsigned num_cells, num_vertices;
  double * x_vertices, * y_vertices, * z_vertices;
  unsigned * cell_to_vertex;
  unsigned * face_id;

  generate_cubed_sphere_grid_information(
    n, &num_cells, &num_vertices, &x_vertices, &y_vertices, &z_vertices,
    &cell_to_vertex, &face_id);

  free(face_id);

  struct dep_list cell_to_vertex_dep_list;
  unsigned * num_vertices_per_cell =
    malloc(num_cells * sizeof(*num_vertices_per_cell));

  for (unsigned i = 0; i < num_cells; ++i)
    num_vertices_per_cell[i] = 4;

  double * lon_vertices = malloc(num_vertices * sizeof(*lon_vertices));
  double * lat_vertices = malloc(num_vertices * sizeof(*lat_vertices));

  for (unsigned i = 0; i < num_vertices; ++i)
    XYZtoLL((double[]){x_vertices[i], y_vertices[i], z_vertices[i]},
            lon_vertices + i, lat_vertices + i);

  free(x_vertices);
  free(y_vertices);
  free(z_vertices);

  yac_set_dependencies(&cell_to_vertex_dep_list, num_cells,
                       (unsigned*)num_vertices_per_cell,
                       (unsigned*)cell_to_vertex);

  return yac_unstruct_grid_new(lon_vertices, lat_vertices, num_vertices,
                               cell_to_vertex_dep_list);

}