#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <stdio.h>

#include "grid.h"
#include "geometry.h"
#include "points.h"
#include "grid_unstruct.h"
#include "dep_list.h"

static FILE * open_cube_csv_file    ( const char *filename);

static void close_cube_csv_file   ( FILE * file );

static void check_status(int status, const char * filename, int ref, int line) {

  if (status == ref) return;

  fprintf(stderr, "error while reading file %s (%s:%d)\n", filename, __FILE__,
          line);
  exit(EXIT_FAILURE);
}
 
void read_cube_csv_grid_information(const char * filename, int * nbr_vertices,
                                    int * nbr_cells, int ** cell_to_vertex,
                                    double ** x_vertices, double ** y_vertices) {

  // open file

  FILE * file = open_cube_csv_file ( filename );

  // read number of vertices and cells

  check_status(fscanf(file, "%d,%d\n", nbr_vertices, nbr_cells), filename, 2,
               __LINE__);

  // read coordinates of vertices

  *x_vertices = malloc (*nbr_vertices * sizeof(**x_vertices));
  *y_vertices = malloc (*nbr_vertices * sizeof(**y_vertices));

  for (int i = 0, dummy; i < *nbr_vertices; ++i)
    check_status(fscanf(file, "%d,%lf,%lf\n", &dummy, *x_vertices+i,
                        *y_vertices+i), filename, 3, __LINE__);

  // read indices of cell vertices

  *cell_to_vertex = malloc(4 * *nbr_cells * sizeof(**cell_to_vertex));

  for (int i = 0, dummy; i < *nbr_cells; ++i) {
    check_status(fscanf(file, "%d,%d,%d,%d,%d\n", &dummy, *cell_to_vertex+4*i+0,
                        *cell_to_vertex+4*i+1, *cell_to_vertex+4*i+2,
                        *cell_to_vertex+4*i+3), filename, 5, __LINE__);

    for (unsigned j = 0; j < 4; ++j)
      (*cell_to_vertex+4*i)[j]--;
  }

  close_cube_csv_file ( file );
}

struct grid * read_cube_csv_grid(char * filename) {

   int nbr_vertices;
   int nbr_cells;
   int * cell_to_vertex;

   double * x_vertices;
   double * y_vertices;

   read_cube_csv_grid_information(filename, &nbr_vertices, &nbr_cells,
                                  &cell_to_vertex, &x_vertices, &y_vertices);

   unsigned * num_vertices_per_cell =
    malloc(nbr_cells * sizeof(*num_vertices_per_cell));

   for (int i = 0; i < nbr_cells; ++i)
     num_vertices_per_cell[i] = 4;

   struct dep_list cell_to_vertex_dep_list;

   yac_set_dependencies(&cell_to_vertex_dep_list, nbr_cells,
                        num_vertices_per_cell, (unsigned*)cell_to_vertex);

   return yac_unstruct_grid_new(x_vertices, y_vertices, nbr_vertices,
                                cell_to_vertex_dep_list);
}

static FILE * open_cube_csv_file (const char *filename) {

  FILE * file = fopen(filename, "r");

  if (file == NULL) {
    perror("Error while opening the file.\n");
    exit(EXIT_FAILURE);
  }

  return file;
}

/* ---------------------------------------------------------------- */

static void close_cube_csv_file (FILE * file) {

   fclose(file);
}
