#include "grid.h"

/** \example test_read_mpiom.c
 * This contains examples for read_mpiom_grid.
 */

/**
 * reads in an mpiom grid netcdf file and generates a struct %grid from it
 * @param[in] filename name of the mpiom grid netcdf file
 * @returns %grid structure that contains the mpiom grid
 */
struct grid * read_mpiom_grid(char * filename);

/**
 * reads in an mpiom grid netcdf file and return the grid information in
 * a format that is supported by the YAC user interface.
 * @param[in]  filename              name of the mpiom grid netcdf file
 * @param[out] nbr_vertices          number of vertices in the grid
 * @param[out] nbr_cells             number of cells in the grid
 * @param[out] num_vertices_per_cell number of vertices per cell
 * @param[out] cell_to_vertex        vertex indices for each cell
 * @param[out] x_vertices            longitudes of vertices
 * @param[out] y_vertices            latitudes of vertices
 * @param[out] x_cells               longitudes of cell center
 * @param[out] y_cells               latitudes of cell center
 * @param[out] cellmask              mask value for cells
 *
 */
void read_mpiom_grid_information(const char * filename, int * nbr_vertices,
                                 int * nbr_cells, int ** num_vertices_per_cell,
                                 int ** cell_to_vertex, double ** x_vertices,
                                 double ** y_vertices, double ** x_cells,
                                 double ** y_cells, int ** cellmask);
