#! /bin/bash
#SBATCH --account=mh0287
#SBATCH --job-name=ICONtoy
#SBATCH --partition=compute
#SBATCH --output=ICONtoy-%j.out
#SBATCH --error=ICONtoy-%j.err
#SBATCH --nodes=32
#SBATCH --ntasks-per-node=24
#SBATCH --exclusive
#SBATCH --time=00:20:00
#SBATCH --mail-type=FAIL
#SBATCH --mail-user=rene.redler@mpimet.mpg.de

source /sw/rhel6-x64/etc/profile.mistral

module load intel/15.0.3
module load mxm/3.3.3002 fca/2.5.2379 bullxmpi_mlx/bullxmpi_mlx-1.2.8.3

export OMPI_MCA_pml=cm
export OMPI_MCA_mtl=mxm
export OMPI_MCA_mtl_mxm_np=0
export OMPI_MCA_coll=^ghc

export MXM_RDMA_PORTS=mlx5_0:1
export MXM_LOG_LEVEL=ERROR

module load intelmpi/5.1.0.038
export I_MPI_PMI_LIBRARY=/usr/lib64/libpmi.so

ulimit -l unlimited
ulimit -c unlimited
ulimit -s unlimited

cd /home/zmaw/m300083/YAC/examples

count=1
max_count=5

# no hyperthreading -> SLURM_NNODES / 2

(( ncpu = SLURM_CPUS_ON_NODE * SLURM_NNODES / 2 ))
(( atm_ncpu = ncpu / 2 ))
(( ocn_ncpu = ncpu / 2 ))
(( ocn_last_cpu = ocn_ncpu - 1 ))
(( atm_last_cpu = ncpu - 1 ))

# Create MPMD configuration file
cat > mpmd.conf <<EOF
0-${ocn_last_cpu} ./toy_mpiom_ocn.x
${ocn_ncpu}-${atm_last_cpu} ./toy_icon_atm.x
EOF

while [[ $count -le $max_count ]]
do

   echo start run $count at `date`

   srun -l --cpu_bind=cores --distribution=block:cyclic --multi-prog mpmd.conf

   echo end run at `date`
   echo

   (( count++ ))

   rm -f *.vtk

done

rm mpmd.conf
