/**
 * @file config_xml_example.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Teresa Holfeld <teresa.holfeld@mpimet.mpg.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Teresa Holfeld <teresa.holfeld@mpimet.mpg.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "config_xml.h"


/**
 * Executes all given interface functions.
 *
 * Examples that are commented out are there to show that the correct error is
 * thrown. To test an error, uncomment such a line.
 */


// macros for indices
#define FIRST          0
#define SECOND         1
#define THIRD          2
#define FOURTH         3
#define THIRTEENTH    12

// macros for IDs
#define FIRST_ID       1
#define SECOND_ID      2
#define THIRD_ID       3
#define FOURTH_ID      4
#define SIXTH_ID       6
#define THIRTEENTH_ID 13

// constants for component names
static const char FIRST_NAME[] = "ICON-ocean";
//static const char EMPTY[] = "";   // enable for testing for errors
static const char NON_EXISTENT[] = "Non-existent";
static const char TRANSIENT_STANDARD_NAME_EXAMPLE[] = "sea_surface_temperature";



int main(void) {
#ifdef USE_XML2

   //* initialization *********************************************************/

   char * xml_filename = "coupling.xml"; // file ok
// char * xml_filename = "../src/xml/couplin.xml";  // file corrupt
// char * xml_filename = "couplig.xml";             // file not found
   char * schema_filename = "coupling.xsd";
   yac_xml_init_config_xml(xml_filename, schema_filename);



   //* components *************************************************************/


   printf("\nTesting components:\n\n");

   int a = yac_xml_get_number_of_components();
   printf("Number of components: %d\n", a);

   // expected: exists == 1
   int exists = yac_xml_component_name_exists_in_xml(FIRST_NAME);
   if (exists) {
      printf("Component with name '%s' exists. \n", FIRST_NAME);
   }

   // expected: exists == 0
   exists = yac_xml_component_name_exists_in_xml(NON_EXISTENT);
   if (!exists) {
      printf("Component with name '%s' does not exist. \n", NON_EXISTENT);
   }

   // expected: exists == 1
   exists = yac_xml_component_id_exists_in_xml(FIRST_ID);
   if (exists) {
      printf("Component with ID '%d' exists. \n", FIRST_ID);
   }

   // expected: exists == 0
   exists = yac_xml_component_id_exists_in_xml(THIRTEENTH_ID);
   if (!exists) {
      printf("Component with ID '%d' does not exist. \n", THIRTEENTH_ID);
   }

   int b = yac_xml_get_component_id(FIRST);
   printf("ID of component %d: %d\n", FIRST, b);

   // expected error: index out of bounds
   //   int c_error = get_component_id(FOURTH);

   int c = yac_xml_get_component_id_by_component_name(FIRST_NAME);
   printf("ID of component '%s': %d \n", FIRST_NAME, c);

   // expected error: string NULL
   //   int c_error1 = get_component_id_by_component_name(NULL);
   // expected error: string empty
   //   int c_error2 = get_component_id_by_component_name(EMPTY);
   // expected error: none, but return value of component id == -1
   //   int c_error3 = get_component_id_by_component_name(NON_EXISTENT);
   //   printf("ID of component '%s': %d \n", NON_EXISTENT, c_error3);

   yac_xml_component * a_component = malloc(sizeof(yac_xml_component));
   yac_xml_get_component(FIRST, a_component);
   printf("Get component %d: name %s and id %d\n", FIRST, a_component->name,
         a_component->global_id);
   yac_xml_free_xml_component(a_component);

   // expected error: index out of bounds
   //   component * a_error_component = malloc(sizeof(component));
   //   get_component(FOURTH, a_error_component);
   // expected error: pointer is NULL
   //   component * error_component = NULL;
   //   get_component(FIRST, error_component);

   yac_xml_component * b_component = malloc(sizeof(yac_xml_component));
   yac_xml_get_component_by_component_id(SECOND_ID, b_component);
   printf("Get component by ID %d: name %s and id %d\n", SECOND_ID,
         b_component->name, b_component->global_id);
   yac_xml_free_xml_component(b_component);

   // expected error: no such ID
   //   component * error_component = malloc(sizeof(component));
   //   get_component_by_component_id(FOURTH_ID, error_component);
   // expected error: pointer is NULL
   //   component * error_component = NULL;
   //   get_component_by_component_id(SECOND_ID, error_component);

   yac_xml_component * c_component = malloc(sizeof(yac_xml_component));
   yac_xml_get_component_by_component_name(FIRST_NAME, c_component);
   printf("Component by name w/ name %s and id %d\n", c_component->name,
         c_component->global_id);
   yac_xml_free_xml_component(c_component);

   //   component * c_error_component = malloc(sizeof(component));
   // expected error: string NULL
   //   get_component_by_component_name(NULL, c_error_component);
   // expected error: string empty
   //   get_component_by_component_name(EMPTY, c_error_component);
   // expected error: no such ID
   //   get_component_by_component_name(NON_EXISTENT, c_error_component);

   // expected error: pointer is NULL
   //   component * error_component = NULL;
   //   get_component_by_component_name(FIRST_NAME, error_component);

   const char * a_name = yac_xml_get_component_name(FIRST);
   printf("Get component name of %d: %s \n", FIRST, a_name);

   // expected error: index out of bounds
   //   char * a_error_name = get_component_name(FOURTH);

   const char * b_name = yac_xml_get_component_name_by_component_id(FIRST_ID);
   printf("Get component name by component ID %d: %s \n", FIRST_ID, b_name);

   // expected error: no such ID
   //   char * b_error_name = get_component_name_by_component_id(FOURTH_ID);

   const char * a_model = yac_xml_get_component_model(FIRST);
   printf("Get component model of %d: %s \n", FIRST, a_model);

   // expected error: index out of bounds
   //   char * a_error_model = get_component_model(FOURTH);

   const char * b_model = yac_xml_get_component_model_by_component_id(FIRST_ID);
   printf("Get component model by component ID %d: %s \n", FIRST_ID, b_model);

   // expected error: no such ID
   //   char * b_error_model = get_component_model_by_component_id(FOURTH_ID);

   const char * a_simulated = yac_xml_get_component_simulated(FIRST);
   printf("Get component simulated of %d: %s \n", FIRST, a_simulated);

   // expected error: index out of bounds
   //   char * a_error_simulated = get_component_simulated(FOURTH);

   const char * b_simulated = yac_xml_get_component_simulated_by_component_id(FIRST_ID);
   printf("Get component simulated by ID %d: %s \n", FIRST_ID, b_simulated);

   // expected error: no such ID
   //   char * b_error_simulated = get_component_simulated_by_component_id(FOURTH_ID);

   int a_no_t = yac_xml_get_number_of_transient_grid_refs(FIRST);
   printf("Get number of transients in component %d: %d \n", FIRST, a_no_t);

   // expected error: index out of bounds
   //   int a_error_no_t = get_number_of_transients_in_component(FOURTH);

   int b_no_t = yac_xml_get_number_of_transient_grid_refs_by_component_id(FIRST_ID);
   printf("Get number of transients in component by component ID %d: %d \n",
         FIRST_ID, b_no_t);

   // expected error: no such ID
   //      int b_error_no_t = get_number_of_transient_grid_refs_by_component_id(
   //            FOURTH_ID);

   int transient_grid_ref_id = yac_xml_get_transient_grid_ref_id(FIRST, THIRD);
   printf("Get transient grid ref id of transient_grid_ref no. %d of component no. %d: %d \n",
         THIRD, FIRST, transient_grid_ref_id);

   // expected error: index out of bounds
   //   int error_transient_grid_ref_id = get_transient_grid_ref_id(THIRTEENTH, FIRST);
   //   int error_transient_grid_ref_id = get_transient_grid_ref_id(FIRST, THIRTEENTH);

   int transient_grid_ref_id_by_component_id =
         yac_xml_get_transient_grid_ref_id_by_component_id(FIRST_ID, THIRD);
   printf("Get transient grid ref id of transient_grid_ref no. %d of component with ID %d: %d \n",
            THIRD, FIRST_ID, transient_grid_ref_id_by_component_id);

   // expected error: no such ID
   //   int error_transient_grid_ref_id_by_component_id =
   //            get_transient_grid_ref_id_by_component_id(THIRTEENTH_ID, THIRD);
   // expected error: index out of bounds
   //   int error_transient_grid_ref_id_by_component_id =
   //            get_transient_grid_ref_id_by_component_id(FIRST_ID, THIRTEENTH);

   int transient_ref = yac_xml_get_transient_id_by_transient_grid_ref_id(FIRST,
         SECOND_ID);
   printf("Get transient id by transient_grid_ref with ID %d in component with index %d: %d \n",
               SECOND_ID, FIRST, transient_ref);

   // expected error: no such ID
   //   int error_transient_ref = get_transient_id_by_transient_grid_ref_id(FIRST,
   //         THIRTEENTH_ID);
   // expected error: index out of bounds
   //   int error_transient_ref =
   //         get_transient_id_by_transient_grid_ref_id(THIRTEENTH, SECOND_ID);

   int transient_ref_by_component_id =
         yac_xml_get_transient_id_by_transient_grid_ref_id_by_component_id(FIRST_ID,
               SECOND_ID);
   printf("Get transient id by transient_grid_ref with ID %d in component with ID %d: %d \n",
                  SECOND_ID, FIRST_ID, transient_ref_by_component_id);

   // expected error: no such ID
   //   int error_transient_ref_by_component_id =
   //            get_transient_id_by_transient_grid_ref_id_by_component_id(
   //                  THIRTEENTH_ID, SECOND_ID);
   //   int error_transient_ref_by_component_id =
   //               get_transient_id_by_transient_grid_ref_id_by_component_id(
   //                     FIRST_ID, THIRTEENTH_ID);

   int grid_ref = yac_xml_get_grid_id_by_transient_grid_ref_id(SECOND, FIRST_ID);
   printf("Get grid id by transient_grid_ref with ID %d in component with index %d: %d \n",
                  FIRST_ID, SECOND, grid_ref);

   // expected error: index out of bounds
   //   int error_grid_ref = get_grid_id_by_transient_grid_ref_id(THIRTEENTH,
   //         FIRST_ID);
   // expected error: no such ID
   //   int error_grid_ref = get_grid_id_by_transient_grid_ref_id(FIRST,
   //         THIRTEENTH_ID);

   int grid_ref_by_component_id =
         yac_xml_get_grid_id_by_transient_grid_ref_id_by_component_id(SECOND_ID,
               FIRST_ID);
   printf("Get grid id by transient_grid_ref with ID %d in component with ID %d: %d \n",
                     FIRST_ID, SECOND_ID, grid_ref_by_component_id);

   // expected error: no such ID
   //   int error_grid_ref_by_component_id =
   //            get_grid_id_by_transient_grid_ref_id_by_component_id(THIRTEENTH_ID,
   //                  FIRST_ID);
   //   int error_grid_ref_by_component_id =
   //               get_grid_id_by_transient_grid_ref_id_by_component_id(SECOND_ID,
   //                     THIRTEENTH_ID);

   //* transients *************************************************************/

   printf("\nTesting transients:\n\n");

   int no_t_total = yac_xml_get_number_of_transients();
   printf("Number of transients: %d \n", no_t_total);

   int t_id = yac_xml_get_transient_id_by_transient_standard_name(
         TRANSIENT_STANDARD_NAME_EXAMPLE);
   printf("Get transient ID by transient standard name '%s': %d \n",
         TRANSIENT_STANDARD_NAME_EXAMPLE, t_id);

   // expected id: -1 (= error)
   //   int error_t_id = get_transient_id_by_transient_standard_name(NON_EXISTENT);
   //   printf("Transient '%s' has id %d \n", NON_EXISTENT, error_t_id);

   // expected: exists == 1
   exists = yac_xml_transient_id_exists_in_xml(FIRST_ID);
   if (exists) {
      printf("Transient with ID '%d' exists. \n", FIRST_ID);
   }

   // expected: exists == 0
   exists = yac_xml_transient_id_exists_in_xml(THIRTEENTH_ID);
   if (!exists) {
      printf("Transient with ID '%d' does not exist. \n", THIRTEENTH_ID);
   }

   const char * a_standard_name = yac_xml_get_transient_standard_name_by_transient_id(
         FIRST_ID);
   printf("Get transient standard name by transient ID %d: %s \n", FIRST_ID,
         a_standard_name);
   yac_xml_free_xml_string(a_standard_name);

   // expected error: no such ID
   //   char * a_error_standard_name = get_transient_standard_name_by_transient_id(
   //         THIRTEENTH_ID);

   // expected: exists == 1
   exists = yac_xml_transient_standard_name_exists_in_xml(
         TRANSIENT_STANDARD_NAME_EXAMPLE);
   if (exists) {
      printf("Transient with standard name '%s' exists. \n",
            TRANSIENT_STANDARD_NAME_EXAMPLE);
   }

   // expected: exists == 0
   exists = yac_xml_transient_standard_name_exists_in_xml(NON_EXISTENT);
   if (!exists) {
      printf("Transient with standard name '%s' does not exist. \n",
            NON_EXISTENT);
   }

   yac_xml_transient * a_t = malloc(sizeof(yac_xml_transient));
   yac_xml_get_transient(FIRST, a_t);
   printf("Get transient %d: name '%s', ID %d \n", FIRST, a_t->name,
         a_t->global_id);
   yac_xml_free_xml_transient(a_t);

   // expected error: index out of bounds
   //   transient * a_error_t = malloc(sizeof(transient));
   //   get_transient(THIRTEENTH, a_error_t);
   // expected error: pointer is NULL
   //   transient * error_transient = NULL;
   //   get_transient(FIRST, error_transient);

   yac_xml_transient * t_by_id = malloc(sizeof(yac_xml_transient));
   yac_xml_get_transient_by_id(FIRST_ID, t_by_id);
   printf("Get transient by ID %d: name '%s', ID %d \n", FIRST_ID,
         t_by_id->name, t_by_id->global_id);
   yac_xml_free_xml_transient(t_by_id);

   // expected error: no such ID
   //   transient * a_error_t = malloc(sizeof(transient));
   //   get_transient_by_id(THIRTEENTH_ID, a_error_t);
   // expected error: pointer is NULL
   //   transient * error_transient = NULL;
   //   get_transient_by_id(FIRST_ID, error_transient);

   yac_xml_transient * transients = malloc(no_t_total * sizeof(yac_xml_transient));
   yac_xml_get_all_transients(transients);
   printf("Get all transients: \n");
   int i;
   for (i = 0; i < no_t_total; i++) {
      printf("   Standard name: '%s', ID: %d \n", transients[i].name,
            transients[i].global_id);
   }
   yac_xml_free_xml_transients(transients, no_t_total);

   // expected error: pointer is NULL
   //   transient * error_transients = NULL;
   //   get_all_transients(error_transients);

   //* grids ******************************************************************/

   printf("\nTesting grids:\n\n");

   int a_no_grids = yac_xml_get_number_of_grids();
   printf("Number of grids: %d \n", a_no_grids);

   // expected: exists == 1
   exists = yac_xml_grid_id_exists_in_xml(FIRST_ID);
   if (exists) {
      printf("Grid with ID '%d' exists. \n", FIRST_ID);
   }

   // expected: exists == 0
   exists = yac_xml_grid_id_exists_in_xml(THIRTEENTH_ID);
   if (!exists) {
      printf("Grid with ID '%d' does not exist. \n", THIRTEENTH_ID);
   }

   const char * a_alias_name = yac_xml_get_grid_alias_name_by_grid_id(FIRST_ID);
   printf("Get grid alias name by grid ID %d: %s \n", FIRST_ID, a_alias_name);
   yac_xml_free_xml_string(a_alias_name);

   // expected error: no such ID
   //   const char * a_error_name = get_grid_alias_name_by_grid_id(THIRTEENTH_ID);

   yac_xml_grid * g = malloc(sizeof(yac_xml_grid));
   yac_xml_get_grid(FIRST, g);
   printf("Get grid %d: ID %d, alias name '%s' \n", FIRST, g->global_id,
         g->name);
   yac_xml_free_xml_grid(g);

   // expected error: index out of bounds
   //   g = malloc(sizeof(yac_xml_grid));
   //   get_grid(THIRTEENTH, g);
   // expected error: pointer is NULL
   //   yac_xml_grid * error_grid = NULL;
   //   get_grid(FIRST_ID, error_grid);

   yac_xml_grid * g_by_id = malloc(sizeof(yac_xml_grid));
   yac_xml_get_grid_by_id(SECOND_ID, g_by_id);
   printf("Get grid by ID %d: ID %d, alias name '%s' \n", SECOND_ID,
         g_by_id->global_id, g_by_id->name);
   yac_xml_free_xml_grid(g_by_id);

   // expected error: no such ID
   //   g_by_id = malloc(sizeof(yac_xml_grid));
   //   get_grid_by_id(THIRTEENTH_ID, g_by_id);
   // expected error: pointer is NULL
   //   yac_xml_grid * error_grid = NULL;
   //   get_grid_by_id(SECOND_ID, error_grid);

   yac_xml_grid * grids = malloc(a_no_grids * sizeof(yac_xml_grid));
   yac_xml_get_all_grids(grids);
   for (i = 0; i < a_no_grids; i++) {
      printf("   Alias name: '%s', ID: %d \n", grids[i].name,
            grids[i].global_id);
   }
   yac_xml_free_xml_grids(grids, a_no_grids);

   // expected error: pointer is NULL
   //   yac_xml_grid * error_grids = NULL;
   //   get_all_grids(error_grids);

   //* dates and timestep *****************************************************/

   printf("\nTesting dates and timestep:\n\n");

   const char * start_date = yac_xml_get_start_date();
   printf("Start date: %s \n", start_date);

   const char * end_date = yac_xml_get_end_date();
   printf("End date: %s \n", end_date);

   yac_xml_timestep_unit unit = yac_xml_get_timestep_unit();
   switch (unit) {
   case T_SECOND:
      printf("Timestep unit: T_SECOND \n");
      break;
   case T_MINUTE:
      printf("Timestep unit: T_MINUTE \n");
      break;
   case T_HOUR:
      printf("Timestep unit: T_HOUR \n");
      break;
   case T_DAY:
      printf("Timestep unit: T_DAY \n");
      break;
   case T_MONTH:
      printf("Timestep unit: T_MONTH \n");
      break;
   case T_YEAR:
      printf("Timestep unit: T_YEAR \n");
      break;
   case T_MILLISECOND:
      printf("Timestep unit: T_MILLISECOND \n");
      break;
   default:
      printf("Error: timestep unit undefined. \n");
      break;
   }

   //* couples ****************************************************************/

   printf("\nTesting couples:\n\n");

   int couples = yac_xml_get_number_of_couples();
   printf("Number of couples: %d\n", couples);

   int first_component_id = yac_xml_get_first_component_id(FIRST);
   printf("First component ID of couple %d: %d \n", FIRST, first_component_id);

   // expected error: index out of bounds
   //   int error_first_component_id = get_first_component_id(FOURTH);

   int second_component_id = yac_xml_get_second_component_id(FIRST);
   printf("Second component ID of couple %d: %d \n", FIRST,
         second_component_id);

   // expected error: index out of bounds
   //   int error_second_component_id = get_second_component_id(FOURTH);

   int no_t_c = yac_xml_get_number_of_transient_couples(FIRST);
   printf("Number of transient couples of couple %d: %d \n", FIRST, no_t_c);

   // expected error: index out of bounds
   //   int error_no_t_c = get_number_of_transient_couples(FOURTH);

   int t_c_id = yac_xml_get_transient_couple_transient_id(FIRST, FIRST);
   printf(
         "Get transient couple transient ID (of couple %d, transient couple %d): %d \n",
         FIRST, FIRST, t_c_id);

   // expected error: index out of bounds
   //   int error_t_c_id = get_transient_couple_transient_id(FOURTH, FIRST);
   //   int error_t_c_id = get_transient_couple_transient_id(FIRST, FOURTH);

   yac_xml_transient * t_c_t = malloc(sizeof(yac_xml_transient));
   yac_xml_get_transient_couple_transient(FIRST, FIRST, t_c_t);
   printf(
         "Get transient couple transient (od couple %d, transient %d): standard name '%s', ID %d \n",
         FIRST, FIRST, t_c_t->name, t_c_t->global_id);
   yac_xml_free_xml_transient(t_c_t);

   // expected error: index out of bounds
   //   transient * error_t_c_t = malloc(sizeof(transient));
   //   get_transient_couple_transient(FOURTH, FIRST, error_t_c_t);
   //   get_transient_couple_transient(FIRST, FOURTH, error_t_c_t);
   // expected error: pointer is NULL
   //   transient * error_transient = NULL;
   //   get_transient_couple_transient(FIRST, FIRST, error_transient);

   int k = yac_xml_get_source_component_id(FIRST, SECOND);
   printf("Get source component ID (of couple %d, transient couple %d): %d \n",
         FIRST, SECOND, k);

   // expected error: index out of bounds
   //   int error1_k = get_source_component_id(FOURTH, SECOND);
   //   int error2_k = get_source_component_id(FIRST, FOURTH);

   int l = yac_xml_get_target_component_id(FIRST, SECOND);
   printf("Get target component ID (of couple %d, transient couple %d): %d \n",
         FIRST, SECOND, l);

   // expected error: index out of bounds
   //   int error1_l = get_target_component_id(FOURTH, SECOND);
   //   int error2_l = get_target_component_id(FIRST, FOURTH);

   int source_transient_grid_ref_id = yac_xml_get_source_transient_grid_ref_id(FIRST,
         SECOND);
   printf(
         "Get source transient_grid_ref ID (of couple %d, transient couple %d): %d \n",
         SECOND, FIRST, source_transient_grid_ref_id);

   // expected error: index out of bounds
   //   int error_source_transient_grid_ref_id = get_source_transient_grid_ref_id(
   //         THIRTEENTH, SECOND);
   //   int error_source_transient_grid_ref_id = get_source_transient_grid_ref_id(
   //         FIRST, THIRTEENTH);

   int target_transient_grid_ref_id = yac_xml_get_target_transient_grid_ref_id(FIRST,
         SECOND);
   printf(
         "Get target transient_grid_ref ID (of couple %d, transient couple %d): %d \n",
         SECOND, FIRST, target_transient_grid_ref_id);

   // expected error: index out of bounds
   //   int error_target_transient_grid_ref_id = get_target_transient_grid_ref_id(
   //         THIRTEENTH, SECOND);
   //   int error_target_transient_grid_ref_id = get_target_transient_grid_ref_id(
   //         FIRST, THIRTEENTH);

   int source_timestep = yac_xml_get_source_timestep(FIRST, FIRST);
   printf("Get source timestep (of couple %d, transient couple %d): %d \n",
         FIRST, FIRST, source_timestep);

   // expected error: index out of bounds
   //   int error_source_timestep = get_source_timestep(FOURTH, FIRST);
   //   int error_source_timestep = get_source_timestep(FIRST, FOURTH);

   int target_timestep = yac_xml_get_target_timestep(FIRST, FIRST);
   printf("Get target timestep (of couple %d, transient couple %d): %d \n",
            FIRST, FIRST, target_timestep);

   // expected error: index out of bounds
   //   int error_target_timestep = get_target_timestep(FOURTH, FIRST);
   //   int error_target_timestep = get_target_timestep(FIRST, FOURTH);

   int coupling_period = yac_xml_get_coupling_period(FIRST, SECOND);
   printf("Get coupling period (of couple %d, transient couple %d): %d \n",
         FIRST, SECOND, coupling_period);

   // expected error: index out of bounds
   //   int error_coupling_period = get_coupling_period(FOURTH, SECOND);
   //   int error_coupling_period = get_coupling_period(FIRST, FOURTH);

   yac_xml_operation o = yac_xml_get_coupling_period_operation(FIRST, SECOND);
   if (o == T_ACCUMULATE) {
      printf(
            "Get coupling period operation (of couple %d, transient couple %d): accumulate \n",
            FIRST, SECOND);
   } else if (o == T_AVERAGE) {
      printf(
            "Get coupling period operation (of couple %d, transient couple %d): average \n",
            FIRST, SECOND);
   } else if (o == T_NONE) {
      printf(
            "Get coupling period operation (of couple %d, transient couple %d): none \n",
            FIRST, SECOND);
   }

   // expected error: index out of bounds
   //   yac_xml_operation error_o = get_coupling_period_operation(FOURTH, SECOND);
   //   yac_xml_operation error_o = get_coupling_period_operation(FIRST, FOURTH);

   int lag = yac_xml_get_source_timelag(FIRST, SECOND);
   printf("Get source time lag (of couple %d, transient couple %d): %d \n", FIRST,
         SECOND, lag);

   // expected error: index out of bounds
   //   int error_lag = get_source_timelag(FOURTH, SECOND);
   //   int error_lag = get_source_timelag(FIRST, FOURTH);

   int no_interpolations = yac_xml_get_number_of_interpolations(FIRST, SECOND);
   printf("Number of interpolations (of couple %d, transient couple %d): %d \n",
         FIRST, SECOND, no_interpolations);

   // expected error: index out of bounds
   //   int error1_no_i = get_number_of_interpolations(FOURTH, SECOND);
   //   int error2_no_i = get_number_of_interpolations(FIRST, FOURTH);

   // expected: equals = 1
   int equals = yac_xml_compare_interpolation_lists(FIRST, FIRST, FIRST, FIRST);
   printf("Interpolation lists are equal (0:0, 0:0): %d \n", equals);

   // expected: equals = 0
   int equals_not = yac_xml_compare_interpolation_lists(FIRST, FIRST, FIRST, SECOND);
   printf("Interpolation lists are equal (0:0, 0:1): %d \n", equals_not);

   // expected error: index out of bounds
   //   int error_equals = compare_interpolation_lists(FOURTH, FIRST, FIRST, FIRST);
   //   int error_equals = compare_interpolation_lists(FIRST, FOURTH, FIRST, FIRST);
   //   int error_equals = compare_interpolation_lists(FIRST, FIRST, FOURTH, FIRST);
   //   int error_equals = compare_interpolation_lists(FIRST, FIRST, FIRST, FOURTH);

   printf("Interpolations of couple %d, transient couple %d: \n", FIRST,
         SECOND);
   for (i = 0; i < no_interpolations; i++) {
      enum yac_interpolation_list interpolation = yac_xml_get_interpolation_method(FIRST,
            SECOND, i);
      char * interpolation_name = yac_interpolation_enum_to_string(interpolation);
      printf("   Interpolation #%d: %s \n", i, interpolation_name);
   }

   // expected error: index out of bounds
   //   enum interpolation_list error1_interpolation = get_interpolation(FOURTH,
   //         SECOND, FIRST);
   //   enum interpolation_list error2_interpolation = get_interpolation(FIRST,
   //         FOURTH, FIRST);
   //   enum interpolation_list error3_interpolation = get_interpolation(FIRST,
   //         SECOND, FOURTH);

   // expected: has_parameters = 1
   int has_parameters = yac_xml_interpolation_has_parameters(FIRST, SECOND, THIRD);
   if (has_parameters) {
      printf(
            "Interpolation %d in transient couple %d in couple %d has parameters. \n",
            THIRD, SECOND, FIRST);
   }

   // expected: has_parameters = 0
   has_parameters = yac_xml_interpolation_has_parameters(FIRST, FIRST, FIRST);
   if (!has_parameters) {
      printf(
            "Interpolation %d in transient couple %d in couple %d does not have parameters. \n",
            FIRST, FIRST, FIRST);
   }

   // expected error: index out of bounds
   //   int error_has_parameters = interpolation_has_parameters(FOURTH, FIRST,
   //         FIRST);
   //   int error_has_parameters = interpolation_has_parameters(FIRST, FOURTH,
   //         FIRST);
   //   int error_has_parameters = interpolation_has_parameters(FIRST, FIRST,
   //         FOURTH);

   // expected: has_parameter = 1
   char * parameter = "n";
   int has_parameter = yac_xml_interpolation_has_parameter(FIRST, SECOND, SECOND,
         parameter);
   if (has_parameter) {
      printf(
            "Interpolation %d in transient couple %d in couple %d has parameter '%s'. \n",
            SECOND, SECOND, FIRST, parameter);
   }

   // expected: has_parameter = 0
   has_parameter = yac_xml_interpolation_has_parameter(FIRST, SECOND, SECOND,
         NON_EXISTENT);
   if (!has_parameter) {
      printf(
            "Interpolation %d in transient couple %d in couple %d does not have parameter '%s'. \n",
            SECOND, SECOND, FIRST, NON_EXISTENT);
   }

   // expected error: index out of bounds
   //   int error_has_parameter = interpolation_has_parameter(FOURTH, FIRST, FIRST,
   //         parameter);
   //   int error_has_parameter = interpolation_has_parameter(FIRST, FOURTH, FIRST,
   //         parameter);
   //   int error_has_parameter = interpolation_has_parameter(FIRST, FIRST, FOURTH,
   //         parameter);

   int number_parameters = yac_xml_get_interpolation_number_of_parameters(FIRST, SECOND,
         THIRD);
   printf(
         "Interpolation %d in transient couple %d in couple %d has %d parameters. \n",
         THIRD, SECOND, FIRST, number_parameters);

   // expected error: index out of bounds
   //   int error_number_parameters = get_interpolation_number_of_parameters(FOURTH,
   //         FIRST, FIRST);
   //   int error_number_parameters = get_interpolation_number_of_parameters(FIRST,
   //         FOURTH, FIRST);
   //   int error_number_parameters = get_interpolation_number_of_parameters(FIRST,
   //         FIRST, FOURTH);

   const char * parameter_name = yac_xml_get_interpolation_parameter_name(FIRST, SECOND, THIRD, FIRST);
   printf("Parameter name of parameter #%d (couple %d, transient couple %d, interpolation %d): '%s' \n",
         FIRST, FIRST, SECOND, THIRD, parameter_name);

   // expected error: index out of bounds
   //   const char * error_parameter_name = get_interpolation_parameter_name(FOURTH, SECOND, THIRD, FIRST);
   //   const char * error_parameter_name = get_interpolation_parameter_name(FIRST, FOURTH, THIRD, FIRST);
   //   const char * error_parameter_name = get_interpolation_parameter_name(FIRST, SECOND, FOURTH, FIRST);
   //   const char * error_parameter_name = get_interpolation_parameter_name(FIRST, SECOND, THIRD, FOURTH);

   const char * parameter_value = yac_xml_get_interpolation_parameter_value_string(
         FIRST, SECOND, SECOND, parameter);
   printf(
         "Parameter value of attribute '%s' (couple %d, transient couple %d, interpolation %d): '%s' \n",
         parameter, FIRST, SECOND, SECOND, parameter_value);
   yac_xml_free_xml_string(parameter_value);

   // expected error: index out of bounds
   //   const char * error_parameter_value =
   //         get_interpolation_parameter_value_string(FOURTH, SECOND, SECOND,
   //               parameter);
   //   const char * error_parameter_value =
   //         get_interpolation_parameter_value_string(FIRST, FOURTH, SECOND,
   //               parameter);
   //   const char * error_parameter_value =
   //         get_interpolation_parameter_value_string(FIRST, SECOND, FOURTH,
   //               parameter);
   //   const char * error_parameter_value =
   //         get_interpolation_parameter_value_string(FIRST, SECOND, SECOND,
   //               NON_EXISTENT);

   const int parameter_value_int = yac_xml_get_interpolation_parameter_value_int(FIRST,
         SECOND, SECOND, parameter);
   printf(
         "Parameter value (int) of attribute '%s' (couple %d, transient couple %d, interpolation %d): %d \n",
         parameter, FIRST, SECOND, SECOND, parameter_value_int);

   // expected error: index out of bounds
   //   const int error_parameter_value_int = get_interpolation_parameter_value_int(
   //         FOURTH, SECOND, SECOND, parameter);
   //   const int error_parameter_value_int = get_interpolation_parameter_value_int(
   //         FIRST, FOURTH, SECOND, parameter);
   //   const int error_parameter_value_int = get_interpolation_parameter_value_int(
   //         FIRST, SECOND, FOURTH, parameter);
   //   const int error_parameter_value_int = get_interpolation_parameter_value_int(
   //         FIRST, SECOND, SECOND, NON_EXISTENT);
   // expected error: no numeric value
   //   const int error_parameter_value_int = get_interpolation_parameter_value_int(
   //         FIRST, SECOND, FIRST, "mask_handling");

   const double parameter_value_double =
         yac_xml_get_interpolation_parameter_value_double(FIRST, SECOND, SECOND,
               parameter);
   printf(
         "Parameter value (double) of attribute '%s' (couple %d, transient couple %d, interpolation %d): %f \n",
         parameter, FIRST, SECOND, SECOND, parameter_value_double);

   // expected error: index out of bounds
   //   const double error_parameter_value_double =
   //         get_interpolation_parameter_value_double(FOURTH, SECOND, SECOND,
   //               parameter);
   //   const double error_parameter_value_double =
   //         get_interpolation_parameter_value_double(FIRST, FOURTH, SECOND,
   //               parameter);
   //   const double error_parameter_value_double =
   //         get_interpolation_parameter_value_double(FIRST, SECOND, FOURTH,
   //               parameter);
   //   const double error_parameter_value_double =
   //         get_interpolation_parameter_value_double(FIRST, SECOND, SECOND,
   //               NON_EXISTENT);
   // expected error: no numeric value
   //   const double error_parameter_value_double =
   //         get_interpolation_parameter_value_double(FIRST, SECOND, FIRST,
   //               "mask_handling");

   // expected error: index out of bounds
   //   double error_mask_value = get_mask_value(FIRST, FOURTH);
   //   double error_mask_value = get_mask_value(FOURTH, FIRST);

   int at_source_before = yac_xml_debug_mode_at_source_before_interpolation(FIRST,
         SECOND);
   printf(
         "Debug mode at source before interpolation? (couple %d, transient couple %d): %d \n",
         FIRST, SECOND, at_source_before);

   // expected error: index out of bounds
   //   int error_at_source_before = debug_mode_at_source_before_interpolation(FIRST,
   //         THIRTEENTH);
   //   int error_at_source_before = debug_mode_at_source_before_interpolation(
   //         THIRTEENTH, SECOND);

   int at_source_after = yac_xml_debug_mode_at_source_after_interpolation(FIRST,
         SECOND);
   printf(
         "Debug mode at source after interpolation? (couple %d, transient couple %d): %d \n",
         FIRST, SECOND, at_source_after);

   // expected error: index out of bounds
   //   int error_at_source_after = debug_mode_at_source_after_interpolation(FIRST,
   //         THIRTEENTH);
   //   int error_at_source_after = debug_mode_at_source_after_interpolation(
   //         THIRTEENTH, SECOND);

   int at_target = yac_xml_debug_mode_at_target(FIRST, SECOND);
   printf(
         "Debug mode at target? (couple %d, transient couple %d): %d \n",
         FIRST, SECOND, at_target);

   // expected error: index out of bounds
   //   int error_at_target = debug_mode_at_target(FIRST, THIRTEENTH);
   //   int error_at_target = debug_mode_at_target(THIRTEENTH, SECOND);

   int write_restart = yac_xml_enforce_write_restart(FIRST, SECOND);
   printf("Enforce write restart? (couple %d, transient couple %d): %d \n",
         FIRST, SECOND, write_restart);

   printf("tests successfully executed.\n");

   //* destroy ****************************************************************/

   yac_xml_destroy_config_xml();
   return EXIT_SUCCESS;
   
#else
   printf("Example test does not work. Compile with --with-xml2-include and --with-xml-lib.\n");
#endif

}
