#ifdef USE_MPI
#include <mpi.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "yac_interface.h"
#include "points.h"

#define NBR_CELLS 2
#define NBR_VERTICES 4
#define NBR_SUBDOMAIN_IDS 1

int main () {

  char * xml_filename = "coupling_aquaplanet.xml";
  char * xsd_filename = "coupling.xsd";

  char * comp_name = "dummy_io";

  int ierror;
  int comp_id;
  int no_fields = 0;
  int field_id[1] = {0};

#ifdef USE_MPI
  MPI_Comm local_comm;
#else
  int local_comm;
#endif

  int size, rank;

#ifdef USE_MPI
  MPI_Init (0, NULL);
#endif

  // Initialise the coupler
  yac_cinit ( xml_filename, xsd_filename );

  // Inform the coupler about what we are
  yac_cdef_comp ( comp_name, &comp_id );

  yac_cget_localcomm ( &local_comm, comp_id );

#ifdef USE_MPI
  MPI_Comm_rank ( local_comm, &rank );
  MPI_Comm_size ( local_comm, &size );
#else
  rank = 0;
  size = 1;
#endif

  printf (" %s rank %d : local size %d \n", comp_name, rank, size );

  // An empty search call to mark the end of the definition phase

  yac_csearch ( 1, &comp_id, no_fields, field_id, &ierror );

  yac_cfinalize ();

#ifdef USE_MPI
  MPI_Finalize ();
#endif

  return 0;
}
