package de.mpg.mpimet.yacgui.junit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertSame;
import static org.junit.Assert.assertTrue;

import java.io.File;

import org.junit.Ignore;
import org.junit.Test;

import de.mpg.mpimet.yacgui.controller.GuiController;
import de.mpg.mpimet.yacgui.model.Model.Align;
import de.mpg.mpimet.yacgui.ui.CouplingGui;

/**
 * JUnit tests for {@link GuiController}.
 * 
 * <p>
 * Some of the tests below will be ignored, because they require user input. To 
 * test them anyway, comment out the line saying '@Ignore'. In the test then a 
 * Dialog is displayed. Click the button as the instruction of the '@Ignore' 
 * line says.
 * 
 * @author teresa
 *
 */
public class GuiControllerTest extends GuiController {
	
	private final static String FILE_COMPONENT1 = "res/xml/component1.xml";
	private final static String FILE_COMPONENT2 = "res/xml/component2.xml";
	private final static String FILE_COUPLING = "res/xml/coupling.xml";
	private final static String FILE_COUPLING_TEST = "res/xml/coupling_test.xml";

	// creation and initialization
	
	@Test
	public void testGetInstance() {
		GuiController controller = GuiController.getInstance();
		GuiController controllerCopy = GuiController.getInstance();
		assertTrue(controller != null);
		assertTrue(controllerCopy != null);
		assertTrue(controller.equals(controllerCopy));
		assertTrue(controller == controllerCopy);
		
		CouplingGui gui = new CouplingGui();
		gui.createAndShowGui();
		controller.registerCouplingGui(gui);
		GuiController otherCopy = GuiController.getInstance();
		assertTrue(controller.equals(otherCopy));
		assertTrue(controller == otherCopy);
		assertTrue(controllerCopy.equals(controller));
		assertTrue(controllerCopy == controller);
	}
	
	@Test
	public void testRegisterCouplingGui() {
		CouplingGui gui = new CouplingGui();
		gui.createAndShowGui();
		registerCouplingGui(gui);
		assertEquals(this.gui, gui);
	}

	// finite state machine
	
	@Test
	public void testState_stateEmpty() {
		createShowRegisterGui();
		assertSame(state, GuiState.EMPTY);
	}
	
	@Test
	public void testState_stateLeftLoaded() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertSame(state, GuiState.LEFT_LOADED);
	}
	
	@Test
	public void testState_stateRightLoaded() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertSame(state, GuiState.RIGHT_LOADED);
	}
	
	@Test
	public void testState_stateFullyLoaded() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	public void testState_stateFullyLoaded_LeftRight() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	public void testState_stateFullyLoaded_RightLeft() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testState_stateLeftLoaded_overwrite() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		assertSame(state, GuiState.LEFT_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite'")
	public void testState_stateLeftLoaded_overwriteCoupling() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openCoupling(new File(FILE_COUPLING));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testState_stateRightLoaded_overwrite() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertSame(state, GuiState.RIGHT_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite'")
	public void testState_stateRightLoaded_overwriteCoupling() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openCoupling(new File(FILE_COUPLING));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testState_stateFullyLoaded_overwrite() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		openCoupling(new File(FILE_COUPLING));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testState_stateFullyLoaded_overwriteLeft() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testState_stateFullyLoaded_overwriteRight() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testState_stateFullyLoaded_LeftRight_overwrite() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		openCoupling(new File(FILE_COUPLING));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testState_stateFullyLoaded_LeftRight_overwriteLeft() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testState_stateFullyLoaded_LeftRight_overwriteRight() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testState_stateFullyLoaded_RightLeft_overwrite() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		openCoupling(new File(FILE_COUPLING));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testState_stateFullyLoaded_RightLeft_overwriteLeft() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testState_stateFullyLoaded_RightLeft_overwriteRight() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertSame(state, GuiState.FULLY_LOADED);
	}
	
	// GUI
	
	@Test
	public void testGui_stateEmpty() {
		createShowRegisterGui();
		assertTrue(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	public void testGui_openComponentLeft() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT).areAllCheckBoxesDisabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	public void testGui_openComponentRight() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesDisabled());
		assertTrue(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	public void testGui_openCoupling() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT).areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT).areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	public void testGui_openComponentLeftRight() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	public void testGui_openComponentRightLeft() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));		
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testGui_openComponentLeft_overwrite() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesDisabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite'")
	public void testGui_openComponentLeft_overwriteCoupling() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openCoupling(new File(FILE_COUPLING));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testGui_openComponentRight_overwrite() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesDisabled());
		assertTrue(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite'")
	public void testGui_openComponentRight_overwriteCoupling() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openCoupling(new File(FILE_COUPLING));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testGui_openCoupling_overwrite() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		openCoupling(new File(FILE_COUPLING));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testGui_openCoupling_overwriteLeft() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testGui_openCoupling_overwriteRight() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testGui_openComponentLeftRight_overwrite() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		openCoupling(new File(FILE_COUPLING));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testGui_openComponentLeftRight_overwriteLeft() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testGui_openComponentLeftRight_overwriteRight() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testGui_openComponentRightLeft_overwrite() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));		
		openCoupling(new File(FILE_COUPLING));
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testGui_openComponentRightLeft_overwriteLeft() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));		
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));		
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testGui_openComponentRightLeft_overwriteRight() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));		
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));		
		assertFalse(gui.getComponentPanel(Align.LEFT).isEmpty());
		assertFalse(gui.getComponentPanel(Align.RIGHT).isEmpty());
		assertTrue(gui.getComponentPanel(Align.LEFT)
				.areAllCheckBoxesEnabled());
		assertTrue(gui.getComponentPanel(Align.RIGHT)
				.areAllCheckBoxesEnabled());
		assertTrue(glassPane.isEmpty());
	}
	
	// model
	
	@Test
	public void testModel_stateEmpty() {
		createShowRegisterGui();
		assertNotNull(model);
		assertTrue(model.isEmpty());
	}
	
	@Test
	public void testModel_openComponentLeft() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertFalse(model.isEmpty(Align.LEFT));
		assertTrue(model.isEmpty(Align.RIGHT));
	}
	
	@Test
	public void testModel_openComponentRight() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertFalse(model.isEmpty(Align.RIGHT));
		assertTrue(model.isEmpty(Align.LEFT));
	}
	
	@Test
	public void testModel_openCoupling() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));
	}
	
	@Test
	public void testModel_openComponentLeftRight() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));
		assertSame(model.getComponentId(Align.LEFT), 1);
		assertSame(model.getComponentId(Align.RIGHT), 2);
	}
	
	@Test
	public void testModel_openComponentRightLeft() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));
		assertSame(model.getComponentId(Align.RIGHT), 1);
		assertSame(model.getComponentId(Align.LEFT), 2);
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testModel_openComponentLeft_overwrite() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertSame(model.getComponentId(Align.LEFT), 1);
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		assertSame(model.getComponentId(Align.LEFT), 2);
		assertFalse(model.isEmpty(Align.LEFT));
		assertTrue(model.isEmpty(Align.RIGHT));
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite'")
	public void testModel_openComponentLeft_overwriteCoupling() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertSame(model.getComponentId(Align.LEFT), 1);
		openCoupling(new File(FILE_COUPLING));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testModel_openComponentRight_overwrite() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertSame(model.getComponentId(Align.RIGHT), 1);
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertSame(model.getComponentId(Align.RIGHT), 2);
		assertFalse(model.isEmpty(Align.RIGHT));
		assertTrue(model.isEmpty(Align.LEFT));
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite'")
	public void testModel_openComponentRight_overwriteCoupling() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertSame(model.getComponentId(Align.RIGHT), 1);
		openCoupling(new File(FILE_COUPLING));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testModel_openCoupling_overwrite() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		openCoupling(new File(FILE_COUPLING));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testModel_openCoupling_overwriteLeft() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));
		assertSame(model.getComponentId(Align.LEFT), 1);
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testModel_openCoupling_overwriteRight() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));
		assertSame(model.getComponentId(Align.RIGHT), 1);
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testModel_openComponentLeftRight_overwrite() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertSame(model.getComponentId(Align.LEFT), 1);
		assertSame(model.getComponentId(Align.RIGHT), 2);
		openCoupling(new File(FILE_COUPLING));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));	
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testModel_openComponentLeftRight_overwriteLeft() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertSame(model.getComponentId(Align.LEFT), 1);
		assertSame(model.getComponentId(Align.RIGHT), 2);
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));
		assertSame(model.getComponentId(Align.LEFT), 2);
		assertSame(model.getComponentId(Align.RIGHT), 2);		
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testModel_openComponentLeftRight_overwriteRight() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertSame(model.getComponentId(Align.LEFT), 1);
		assertSame(model.getComponentId(Align.RIGHT), 2);
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));
		assertSame(model.getComponentId(Align.LEFT), 1);
		assertSame(model.getComponentId(Align.RIGHT), 1);		
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testModel_openComponentRightLeft_overwrite() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));	
		assertSame(model.getComponentId(Align.RIGHT), 1);
		assertSame(model.getComponentId(Align.LEFT), 2);		
		openCoupling(new File(FILE_COUPLING));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));	
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testModel_openComponentRightLeft_overwriteLeft() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));	
		assertSame(model.getComponentId(Align.RIGHT), 1);
		assertSame(model.getComponentId(Align.LEFT), 2);
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));
		assertSame(model.getComponentId(Align.LEFT), 1);
		assertSame(model.getComponentId(Align.RIGHT), 1);		
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testModel_openComponentRightLeft_overwriteRight() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		assertSame(model.getComponentId(Align.RIGHT), 1);
		assertSame(model.getComponentId(Align.LEFT), 2);
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertFalse(model.isEmpty());
		assertFalse(model.isEmpty(Align.LEFT));
		assertFalse(model.isEmpty(Align.RIGHT));
		assertSame(model.getComponentId(Align.LEFT), 2);
		assertSame(model.getComponentId(Align.RIGHT), 2);		
	}
	
	// FileStatus: saved or not, save menu enabled/disabled
	
	@Test
	public void testFileStatus_stateEmpty() {
		createShowRegisterGui();
		assertNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(gui.isMenuFileSaveEnabled());
		assertFalse(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	public void testFileStatus_openComponentLeft() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(gui.isMenuFileSaveEnabled());
		assertFalse(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	public void testFileStatus_openComponentRight() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(gui.isMenuFileSaveEnabled());
		assertFalse(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	public void testFileStatus_openCoupling() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		assertNotNull(fileStatus);
		assertNotNull(saveDestination);
		assertTrue(fileStatus.isSaved());
		assertFalse(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	public void testFileStatus_openCouplingEdit() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		fileStatus.edit();
		assertNotNull(fileStatus);
		assertNotNull(saveDestination);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	public void testFileStatus_openCouplingEditSave() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		fileStatus.edit();
		fileStatus.save();
		assertNotNull(fileStatus);
		assertNotNull(saveDestination);
		assertTrue(fileStatus.isSaved());
		assertFalse(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	public void testFileStatus_openComponentLeftRight() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertNotNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	public void testFileStatus_openComponentRightLeft() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));		
		assertNotNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testFileStatus_openComponentLeft_overwrite() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		assertNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(gui.isMenuFileSaveEnabled());
		assertFalse(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite'")
	public void testFileStatus_openComponentLeft_overwriteCoupling() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openCoupling(new File(FILE_COUPLING));
		assertNotNull(fileStatus);
		assertNotNull(saveDestination);
		assertTrue(fileStatus.isSaved());
		assertFalse(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testFileStatus_openComponentRight_overwrite() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(gui.isMenuFileSaveEnabled());
		assertFalse(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite'")
	public void testFileStatus_openComponentRight_overwriteCoupling() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openCoupling(new File(FILE_COUPLING));
		assertNotNull(fileStatus);
		assertNotNull(saveDestination);
		assertTrue(fileStatus.isSaved());
		assertFalse(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testFileStatus_openCoupling_overwrite() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		openCoupling(new File(FILE_COUPLING));
		assertNotNull(fileStatus);
		assertNotNull(saveDestination);
		assertTrue(fileStatus.isSaved());
		assertFalse(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testFileStatus_openCoupling_editOverwrite() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		fileStatus.edit();
		openCoupling(new File(FILE_COUPLING));
		assertNotNull(fileStatus);
		assertNotNull(saveDestination);
		assertTrue(fileStatus.isSaved());
		assertFalse(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testFileStatus_openCoupling_overwriteLeft() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertNotNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testFileStatus_openCoupling_editOverwriteLeft() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		fileStatus.edit();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertNotNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Yes'")
	public void testFileStatus_openCoupling_overwriteRight() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertNotNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testFileStatus_openCoupling_editOverwriteRight() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		fileStatus.edit();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertNotNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testFileStatus_openComponentLeftRight_overwrite() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		openCoupling(new File(FILE_COUPLING));
		assertNotNull(fileStatus);
		assertNotNull(saveDestination);
		assertTrue(fileStatus.isSaved());
		assertFalse(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testFileStatus_openComponentLeftRight_overwriteLeft() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		assertNotNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testFileStatus_openComponentLeftRight_overwriteRight() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		assertNotNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testFileStatus_openComponentRightLeft_overwrite() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));		
		openCoupling(new File(FILE_COUPLING));
		assertNotNull(fileStatus);
		assertNotNull(saveDestination);
		assertTrue(fileStatus.isSaved());
		assertFalse(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testFileStatus_openComponentRightLeft_overwriteLeft() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		assertNotNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: click 'Overwrite without saving'")
	public void testFileStatus_openComponentRightLeft_overwriteRight() {
		createShowRegisterGui();
		openComponent(Align.RIGHT, new File(FILE_COMPONENT1));
		openComponent(Align.LEFT, new File(FILE_COMPONENT2));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		assertNotNull(fileStatus);
		assertNull(saveDestination);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	// FileStatus with actual file operations (save, editing)
	
	@Test
	@Ignore("User input required: provide filename and location")
	public void testFileStatus_saveCoupling() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING_TEST));
		fileStatus.edit();
		saveCoupling(false);
		assertTrue(fileStatus.isSaved());
		assertFalse(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: provide filename and location")
	public void testFileStatus_saveCouplingAs() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		saveCoupling(true);
		assertTrue(fileStatus.isSaved());
		assertFalse(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: provide filename and location")
	public void testFileStatus_saveCouplingLeftRight() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		saveCoupling(false);
		assertTrue(fileStatus.isSaved());
		assertFalse(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	@Ignore("User input required: provide filename and location")
	public void testFileStatus_saveCouplingAsLeftRight() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		saveCoupling(true);
		assertTrue(fileStatus.isSaved());
		assertFalse(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	public void testFileStatus_editCoupling() {
		createShowRegisterGui();
		openCoupling(new File(FILE_COUPLING));
		onTransientCheckBoxCheck(Align.LEFT, 0);
		onTransientCheckBoxCheck(Align.RIGHT, 1);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	@Test
	public void testFileStatus_editCouplingLeftRight() {
		createShowRegisterGui();
		openComponent(Align.LEFT, new File(FILE_COMPONENT1));
		openComponent(Align.RIGHT, new File(FILE_COMPONENT2));
		onTransientCheckBoxCheck(Align.LEFT, 0);
		onTransientCheckBoxCheck(Align.RIGHT, 1);
		assertFalse(fileStatus.isSaved());
		assertTrue(gui.isMenuFileSaveEnabled());
		assertTrue(gui.isMenuFileSaveAsEnabled());
	}
	
	// TODO test checkboxes -> model delegation
	
	// helper methods
	
	private void createShowRegisterGui() {
		CouplingGui gui = new CouplingGui();
		gui.createAndShowGui();
		registerCouplingGui(gui);
	}
}
