package de.mpg.mpimet.yacgui.ui;

import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import de.mpg.mpimet.yacgui.utils.Logger;

/**
 * File chooser for opening XML files. Keeps track of last used directory.
 */
public class XmlFileChooser {

	private static final String FILTER_DESCRIPTION = "XML files";
	private static final String FILTER_FILE_EXTENSION = "xml";
	private static final String CANCEL_LOG = "File choosing action has been cancelled.";
	
	private static File lastUsedDirectory = null;
	private static String lastUsedFileName = "coupling.xml";

	/**
	 * Creates and shows JFileChooser for opening a File. Filters files with '.xml' extension.
	 * 
	 * @return File array containing user file selections,
	 * 			in case of not multiple selection - array with single element
	 * 			in case of dialog cancel - null
	 */
	public static File[] chooseFileOpen(JPanel parentPanel, boolean multipleSelection) {
		JFileChooser fileChooser = new JFileChooser(lastUsedDirectory);
		FileFilter filter = new FileNameExtensionFilter(FILTER_DESCRIPTION, FILTER_FILE_EXTENSION);
		fileChooser.setFileFilter(filter);
		fileChooser.setMultiSelectionEnabled(multipleSelection);
		int status = fileChooser.showOpenDialog(parentPanel);
		
		File[] files = null;
		if (status == JFileChooser.APPROVE_OPTION) {
			if (multipleSelection) {
				files = fileChooser.getSelectedFiles();
			} else {
				files = new File[] {fileChooser.getSelectedFile()};
			}
			if (files != null && files.length > 0 && files[0] != null) {
				lastUsedDirectory = files[0].getParentFile();
			}
		} else if (status == JFileChooser.CANCEL_OPTION) {
			Logger.logInfo(CANCEL_LOG);
		}
		return files;
	}
	
	/**
	 * Helper method to choose single file. 
	 * @see XmlFileChooser#chooseFileOpen(JPanel, boolean)
	 * 
	 * @param parentPanel
	 * @return
	 */
	public static File chooseFileOpen(JPanel parentPanel) {
		File[] files = chooseFileOpen(parentPanel, false); 
		
		return files != null ? files[0] : null;
	}

	/**
	 * Creates JFileChooser for saving a File.<br>
	 * 
	 * @return File chosen by user
	 */
	public static File chooseFileSave(JPanel parentPanel) {
		JFileChooser fileChooser = new JFileChooser(lastUsedDirectory);
		fileChooser.setSelectedFile(new File(lastUsedFileName));
		int status = fileChooser.showSaveDialog(parentPanel);
		
		File destinationFile = null;
		if (status == JFileChooser.APPROVE_OPTION) {
			destinationFile = fileChooser.getSelectedFile();
			if (destinationFile != null) {
				lastUsedDirectory = destinationFile.getParentFile();
				lastUsedFileName =  destinationFile.getName();
			}
		} else if (status == JFileChooser.CANCEL_OPTION) {
			Logger.logInfo(CANCEL_LOG);
		}
		return destinationFile;
	}
}
