package de.mpg.mpimet.yacgui.ui.dialog;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import de.mpg.mpimet.yacgui.controller.GuiController;
import de.mpg.mpimet.yacgui.ui.CouplingGui;
import de.mpg.mpimet.yacgui.ui.dialog.tabpanel.InterpolationsTab;
//import de.mpg.mpimet.yacgui.ui.dialog.tabpanel.MoreTab;
import de.mpg.mpimet.yacgui.ui.dialog.tabpanel.TimestepTab;
import de.mpg.mpimet.yacgui.xml.TimestepUnit;
import de.mpg.mpimet.yacgui.xml.TransientCouple;
import de.mpg.mpimet.yacgui.junit.XmlDomHandlerTest;

public class CouplingDialog extends JDialog {
	
	private static final long serialVersionUID = 5283512859321606370L;
	
	private static final String INTERPOLATION = "Interpolation";
	private static final String TIMESTEP = "Timestep";
	//private static final String MORE = "More";
	
	private static final String USE_AS_DEFAULT_LABEL = "Use these parameters as default";
	private static final String FORGET_DEFAULT_LABEL = "Forget default parameters";
	private static final String DEFAULT_TITLE = "Default parameters";
	private static final String USE_DEFAULT_LABEL = "Load default params";	
	
	private static final String LABEL_SAVE = "Save";
	private static final String LABEL_CLOSE = "Close";
	private static final int BUTTONS_VIEW_HEIGHT = 150;
	
	private InterpolationsTab interpolationsTab;
	private TimestepTab timestepTab;
	//private MoreTab moreTab;
	private TransientCouple transientCouple;
	private GuiController m_guiController;

	public CouplingDialog(String dialogCaption, String dialogDescription, TransientCouple transientCouple,
			TimestepUnit timestepUnit) {
		super((JDialog) null, true);
		
		this.transientCouple = transientCouple;
		
		setTitle(dialogCaption);
		setLayout(new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS));
		
		interpolationsTab = new InterpolationsTab(dialogDescription, transientCouple);
		timestepTab = new TimestepTab(dialogDescription, transientCouple, timestepUnit);
		//moreTab = new MoreTab(dialogDescription, transientCouple);
		
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.addTab(INTERPOLATION, interpolationsTab);
		tabbedPane.addTab(TIMESTEP, timestepTab);
		//tabbedPane.addTab(MORE, moreTab);
		
		// init model and controller
		m_guiController = GuiController.getInstance();
		//m_guiController.registerCouplingGui(couplingGui);
		
		add(tabbedPane);
		add(createDefaultButtonsPanel());
		add(createButtonsPanel());
		pack();
	}
	
	private JPanel createDefaultButtonsPanel() {
		JPanel panel = new JPanel(new FlowLayout());

		//default button
		final JButton saveDefaultBut = new JButton();
		final JButton loadDefaultBut = new JButton(USE_DEFAULT_LABEL);

		if (!m_guiController.hasDefaultCouplingParams()) {
			saveDefaultBut.setText(USE_AS_DEFAULT_LABEL);
			loadDefaultBut.setEnabled(false);
		} else {
			saveDefaultBut.setText(FORGET_DEFAULT_LABEL);
		}
		
		saveDefaultBut.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				
				if (!m_guiController.hasDefaultCouplingParams()) {
					collectAllParams();
					m_guiController.setDefaultCouplingParams(transientCouple);
					
					saveDefaultBut.setText(FORGET_DEFAULT_LABEL);
					loadDefaultBut.setEnabled(true);
				} else {
					m_guiController.setDefaultCouplingParams(null);
					saveDefaultBut.setText(USE_AS_DEFAULT_LABEL);
					loadDefaultBut.setEnabled(false);
				}
			}
		});
		
		loadDefaultBut.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TransientCouple defTransientCouple = m_guiController.getDefaultCouplingParams();
				
				if (defTransientCouple != null) {
					transientCouple.loadTransientCoupleData(defTransientCouple);
					
					interpolationsTab.loadParameters();
					timestepTab.loadParameters();
					//moreTab.loadParameters();
				}
			}
		});
		panel.add(saveDefaultBut);
		panel.add(loadDefaultBut);
		
		panel.setBorder(BorderFactory.createTitledBorder(DEFAULT_TITLE));
		panel.setMaximumSize(new Dimension(Integer.MAX_VALUE, BUTTONS_VIEW_HEIGHT));
		return panel;
	}
	
	protected JPanel createButtonsPanel() {
		JPanel bottom = new JPanel();
		final JButton saveButton = new JButton(LABEL_SAVE);
		saveButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
					collectAllParams();
					m_guiController.setTransientCoupleData(transientCouple);
					
					// added by Junpei Xu					
					JOptionPane.showMessageDialog(null, "transientCouple is saved");
			}
		});
		
		bottom.add(saveButton);
		final JButton closeButton = new JButton(LABEL_CLOSE);
		closeButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				CouplingDialog.this.dispose();
			}
		});
		bottom.add(closeButton);
		bottom.setMaximumSize(new Dimension(Integer.MAX_VALUE, BUTTONS_VIEW_HEIGHT));
		return bottom;
	}

	public void collectAllParams() {
		interpolationsTab.collectData();
		timestepTab.collectData();
		//moreTab.collectData();
	}
}
