package de.mpg.mpimet.yacgui.ui.dialog;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;

import de.mpg.mpimet.yacgui.model.Model;
import de.mpg.mpimet.yacgui.model.Model.Align;
import de.mpg.mpimet.yacgui.xml.Component;
import de.mpg.mpimet.yacgui.xml.Couple;

public class SelectComponentsDialog extends JDialog {

	private static final long serialVersionUID = 1311022694660512420L;
	
	private static final String[] columnNames = {"Coupling between", "Number of coupled fields"};

	private static final String TITLE = "Components";
	private static final String TITLE_LEFT_COMPONENT = "Left Component";
	private static final String TITLE_RIGHT_COMPONENT = "Right Component";
	private static final String TITLE_COUPLING_TABLE = "Existing couples";
	
	private static final String LABEL_APPLY = "Apply";
	private static final String LABEL_CLOSE = "Close";
	
	private static final int DIALOG_WIDTH = 420;
	private static final int DIALOG_HEIGHT = 300;

	private Map<Integer, Component> componentsMap;
	private Map<String, Couple> couplesMap;
	
	private JList<String> leftCompList;
	private JList<String> rightCompList;
	
	private JTable couplesTable;
	private CouplesTableModel couplesTableModel;

	private boolean ignoringCoupleSelectionEvents;
	private boolean ignoringComponentsSelectionEvents;
	
	public interface SelectComponentsDialogActionsListener {
		void onComponentsSelected(int leftCompId, int rightCompId);
		void onSelectComponentsDialogClosed();
	}
	
	private SelectComponentsDialogActionsListener dialogActionsListener;

	private Model model;

	private JButton applyButton;
	
	public SelectComponentsDialog(SelectComponentsDialogActionsListener listener, Model model) {
		super();
		
		this.dialogActionsListener = listener;
		this.model = model;
		
		setTitle(TITLE);
		setPreferredSize(new Dimension(DIALOG_WIDTH, DIALOG_HEIGHT));
		setContentPane(createContentPane());
		pack();
		
		updateInfoFromModel();
		
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				notifyListenerAboutClosing();
			}
		});
	}
	
	private JPanel createContentPane() {
		
		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new GridBagLayout());
		mainPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		
		leftCompList = new JList<String>();
		leftCompList.setLayoutOrientation(JList.VERTICAL);
		JScrollPane listScroller = new JScrollPane(leftCompList);
		listScroller.setBorder(BorderFactory.createTitledBorder(TITLE_LEFT_COMPONENT));
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0.5;
		c.weighty = 0.5;
		c.fill = GridBagConstraints.BOTH;
		mainPanel.add(listScroller, c);
		
		rightCompList = new JList<String>();
		rightCompList.setLayoutOrientation(JList.VERTICAL);
		listScroller = new JScrollPane(rightCompList);
		listScroller.setBorder(BorderFactory.createTitledBorder(TITLE_RIGHT_COMPONENT));
		c = new GridBagConstraints();
		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 0.5;
		c.weighty = 0.5;
		c.fill = GridBagConstraints.BOTH;
		mainPanel.add(listScroller, c);
		
		ListSelectionListener mutualExclusiveSelectionListener = new ListSelectionListener() {
			
			@Override
			public void valueChanged(ListSelectionEvent e) {
				if (ignoringComponentsSelectionEvents) {
					return;
				}
				
				String leftSel = leftCompList.getSelectedValue();
				String rightSel = rightCompList.getSelectedValue();
				
				applyButton.setEnabled(leftSel != null && rightSel != null);
				
				if (e.getSource() == leftCompList && leftSel != null && leftSel.equals(rightSel)) {
					rightCompList.clearSelection();
				} else if (e.getSource() == rightCompList && rightSel != null && rightSel.equals(leftSel)) {
					leftCompList.clearSelection();
				}
				
				int firstComponentId = getComponentIdByName(leftSel);
				int secondComponentId = getComponentIdByName(rightSel);
				
				String coupleMapId = firstComponentId + "-" + secondComponentId;
				
				boolean hasCorrespondingCouple = couplesMap.containsKey(coupleMapId);
				
				if (!hasCorrespondingCouple) {
					coupleMapId = secondComponentId + "-" + firstComponentId;
					hasCorrespondingCouple = couplesMap.containsKey(coupleMapId);
				}
				
				if (hasCorrespondingCouple) {
					setCouplesTableSelectionAt(couplesMap.get(coupleMapId));
				} else {
					setCouplesTableSelectionAt(null);
				}
			}
		};
		leftCompList.addListSelectionListener(mutualExclusiveSelectionListener);
		rightCompList.addListSelectionListener(mutualExclusiveSelectionListener);
		
		couplesTable = new JTable();
		couplesTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane scrollPane = new JScrollPane(couplesTable);
		couplesTable.setFillsViewportHeight(true);
		scrollPane.setBorder(BorderFactory.createTitledBorder(TITLE_COUPLING_TABLE));
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 1;
		c.gridheight = 1;
		c.gridwidth = 2;
		c.weighty = 0.5;
		c.fill = GridBagConstraints.BOTH;
		mainPanel.add(scrollPane, c);
		
		couplesTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			
			@Override
			public void valueChanged(ListSelectionEvent e) {
				if (ignoringCoupleSelectionEvents) {
					return;
				}
				
				int selectedRow = couplesTable.getSelectedRow();
				
				Couple selectedCouple = couplesTableModel.getCouple(selectedRow);
				
				String firstComponentName = componentsMap.get(selectedCouple.getComponent1Id()).getName();
				String secondComponentName = componentsMap.get(selectedCouple.getComponent2Id()).getName();
				
				
				ignoringComponentsSelectionEvents = true;
				leftCompList.setSelectedValue(firstComponentName, true);
				rightCompList.setSelectedValue(secondComponentName, true);
				ignoringComponentsSelectionEvents = false;
				
				applyButton.setEnabled(true);
			}
		});
		
		JPanel buttonsPanel = new JPanel();
		
		applyButton = new JButton(LABEL_APPLY);
		applyButton.setEnabled(false);
		applyButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				if (dialogActionsListener != null) {
					int leftCompId = getComponentIdByName(leftCompList.getSelectedValue());
					int rightCompId = getComponentIdByName(rightCompList.getSelectedValue());
					dialogActionsListener.onComponentsSelected(leftCompId, rightCompId);
				}
			}
		});
		buttonsPanel.add(applyButton);
		
		final JButton closeButton = new JButton(LABEL_CLOSE);
		closeButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				SelectComponentsDialog.this.dispose();
				notifyListenerAboutClosing();
			}
		});
		buttonsPanel.add(closeButton);
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 2;
		c.gridheight = 1;
		c.gridwidth = 2;
		
		mainPanel.add(buttonsPanel, c);
		
		return mainPanel;
	}
	
	protected void notifyListenerAboutClosing() {
		if (dialogActionsListener != null) {
			dialogActionsListener.onSelectComponentsDialogClosed();
		}
	}

	public void updateInfoFromModel() {
		
		componentsMap = model.getCoupling().getComponentsMap();
		couplesMap = model.getCoupling().getCouples();
		
		ignoringCoupleSelectionEvents = true;
		couplesTableModel = new CouplesTableModel();
		couplesTable.setModel(couplesTableModel);
		ignoringCoupleSelectionEvents = false;
		
		//set the first column more space
		couplesTable.getColumnModel().getColumn(0).setPreferredWidth((int) (DIALOG_WIDTH * 0.45));

		String[] componentNameArray = getComponentsNameArray();
		leftCompList.setListData(componentNameArray);
		rightCompList.setListData(componentNameArray);
		
		leftCompList.setSelectedValue(model.getComponentName(Align.LEFT), true);
		rightCompList.setSelectedValue(model.getComponentName(Align.RIGHT), true);
	}
	
	private void setCouplesTableSelectionAt(Couple couple) {
		ignoringCoupleSelectionEvents = true;
		
		if (couple == null) {
			couplesTable.clearSelection();
		} else {
			int index = couplesTableModel.getIndex(couple);
			couplesTable.setRowSelectionInterval(index, index);
		}
		
		ignoringCoupleSelectionEvents = false;
	}

	private String[] getComponentsNameArray() {
		String[] componentNameArray = new String[componentsMap.size()];
		
		int i = 0;
		for (Integer key : componentsMap.keySet()) {
			componentNameArray[i++] = componentsMap.get(key).getName();
		}
		return componentNameArray;
	}
	
	private int getComponentIdByName(String componentName) {
		for (Integer key : componentsMap.keySet()) {
			if (componentsMap.get(key).getName().equals(componentName)) {
				return key;
			}
		}
		return 0;
	}

	private class CouplesTableModel extends AbstractTableModel {
		
		private static final long serialVersionUID = -7790031145848659044L;
		
		private List<Couple> couplesList;
		
		public CouplesTableModel() {
			couplesList = new ArrayList<Couple>(couplesMap.values());
		}

		public int getIndex(Couple couple) {
			return couplesList.indexOf(couple);
		}
		
		public Couple getCouple(int index) {
			return couplesList.get(index);
		}
		
		@Override
		public Object getValueAt(int rowIndex, int columnIndex) {
			switch (columnIndex) {
			case 0:
				int firstCompId = couplesList.get(rowIndex).getComponent1Id();
				int secondCompId = couplesList.get(rowIndex).getComponent2Id();
				return  componentsMap.get(firstCompId).getName() + "<->" + componentsMap.get(secondCompId).getName();
			case 1:
				return couplesList.get(rowIndex).getTransientCouples().size();
			default:
				break;
			}
			return null;
		}
		
		@Override
		public int getRowCount() {
			return couplesList.size();
		}
		
		@Override
		public int getColumnCount() {
			return columnNames.length;
		}
		
		@Override
		public String getColumnName(int column) {
			return columnNames[column];
		}
	}
}
