package de.mpg.mpimet.yacgui.ui.dialog.tabpanel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JFormattedTextField.AbstractFormatter;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.text.InternationalFormatter;

import de.mpg.mpimet.yacgui.utils.Default;
import de.mpg.mpimet.yacgui.utils.Utils;
import de.mpg.mpimet.yacgui.xml.interpolation.AverageInterpolation;
import de.mpg.mpimet.yacgui.xml.interpolation.ConservativeInterpolation;
import de.mpg.mpimet.yacgui.xml.interpolation.ConservativeInterpolation.Normalisation;
import de.mpg.mpimet.yacgui.xml.interpolation.FixedValueInterpolation;
import de.mpg.mpimet.yacgui.xml.interpolation.Interpolation;
import de.mpg.mpimet.yacgui.xml.interpolation.InterpolationMethod;
import de.mpg.mpimet.yacgui.xml.interpolation.NNearestNeighborInterpolation;
import de.mpg.mpimet.yacgui.xml.interpolation.NNearestNeighborInterpolation.Weighted;
import de.mpg.mpimet.yacgui.xml.interpolation.PatchRecoveryInterpolation;
import de.mpg.mpimet.yacgui.xml.interpolation.SmoothedPatchRecoveryInterpolation;
import de.mpg.mpimet.yacgui.xml.interpolation.UserFileInterpolation;
import de.mpg.mpimet.yacgui.xml.interpolation.UserFileInterpolation.YacLocation;
import de.mpg.mpimet.yacgui.xml.interpolation.Source2TargetMapInterpolation;
 

/**
 * Panel which holds combobox with interpolation options and parameters panels in case if the 
 * selected interpolation has any.
 * 
 * @author Maxim Yastremsky maxim.yastremsky@mpimet.mpg.de
 * @author modified by Junpei Xu  
 */
public class InterpolationPanel extends JPanel {

	private static final long serialVersionUID = 4936278751187365696L;

	//default comboBoxValue
	private static final String NONE_SELECTED = "none selected";
	private static final String OPTION = "Option ";
	
	//for n-nearest neighbor
	private static final String LABEL_N = "n: ";
	private static final String LABEL_WEIGHTED = "Weighted: ";
	
	//for conservative and average
	private static final String LABEL_ENFORCED_CONSERVATION = "enforced_conservation";
	private static final String LABEL_PARTIAL_COVERAGE = "partial coverage";
	private static final String LABEL_NORMALISATION = "normalisation: ";
	
	//for patch-recovery
	private static final String LABEL_ALLOW_EXTRAPOLATION = "allow extrapolation";
	private static final String LABEL_EXTEND_SOURCE_PATCH = "extend source patch: ";
	private static final String LABEL_POLYNOMIAL_ORDER = "polynomial order: ";
	private static final String LABEL_GAUSS_ORDER = "gauss order: ";
	private static final String LABEL_ALLOW_CORNER_WEIGHT = "allow corner weight";
	private static final String LABEL_CORNER_WEIGHT = "corner weight: ";

	
	//for fixed value
	private static final String LABEL_USER_VALUE = "user value: ";
	
	//for user file
	private static final String LABEL_FILE = "file: ";
	private static final String LABEL_YAC_LOCATION = "YAC location: ";
	
	private static final int DEFAULT_SELECTED_INDEX = 0;

	private static String[] comboBoxValues;
	
	static {
		int size = InterpolationMethod.values().length;
		comboBoxValues = new String[size + 1]; //plus NONE_SELECTED
		
		int i = 0;
		comboBoxValues[0] = NONE_SELECTED;
		for (InterpolationMethod method : InterpolationMethod.values()) {
			comboBoxValues[++i] = method.getName();
		}
	}

	private int index;
	
	//interpolation of the current interpolation panel
	private Interpolation interpolation;
	
	private Map<String, JComponent> parameterComponents = new HashMap<String, JComponent>();

	private JComboBox<String> comboBox;

	public interface InterpolationUpdateListener {
		public void onInterpolationUpdate();
		public void onInterpolationRemoved(InterpolationPanel panel);
	}

	private InterpolationUpdateListener listener;

	private JPanel parametersPanel;

	private JLabel optionNumberLabel;

	private JButton clearButton;
	
	public InterpolationPanel(InterpolationUpdateListener listener, int index) {
		this(listener, index, null);
	}

	public InterpolationPanel(InterpolationUpdateListener listener, int index, Interpolation interpolation) {
		this.index = index;
		this.listener = listener;
		this.interpolation = interpolation;
		createLayout();
	}

	public Interpolation getInterpolation() {
		collectInterpolationParameters();
		return interpolation;
	}
	
	/**
	 * create layout of panel on the interpolation tab to show dynamic parameters panel
	 */
	private void createLayout() {
		// set up a vertical layout
		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

		JPanel comboBoxPanel = new JPanel();

		// label
		optionNumberLabel = new JLabel(OPTION + String.valueOf(index));
		comboBoxPanel.add(optionNumberLabel);

		// combo box for user selection
		comboBox = new JComboBox<String>(comboBoxValues);
		comboBox.setBorder(new EmptyBorder(5, 5, 5, 5));
		
		comboBoxPanel.add(comboBox);
		
		ImageIcon clearIcon = Utils.createImageIcon("/img/close-icon.png");
		clearButton = new JButton(clearIcon);
		clearButton.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				if (interpolation != null) {
					comboBox.setSelectedItem(NONE_SELECTED);
	
					interpolation = null;
	
					showParametersPanel(false);
				}

				listener.onInterpolationRemoved(InterpolationPanel.this);
			}
		});
		comboBoxPanel.add(clearButton);
		
		add(comboBoxPanel);

		if (interpolation != null) {
			comboBox.setSelectedItem(interpolation.getMethod().getName());
			
			JPanel interptnParametersPanel = createParametersPanel();
			if (interptnParametersPanel != null) {
				add(interptnParametersPanel);
				showInterpolationParameters();
			}
		} else {
			if (index < InterpolationsTab.DEFAULT_INTERPOLATIONS_NUMBER) {
				clearButton.setEnabled(false);
			}
		}
		
		comboBox.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int index = comboBox.getSelectedIndex();
				
				if (index == DEFAULT_SELECTED_INDEX) { //none_selected
					interpolation = null;
					showParametersPanel(false);
				} else {
					String method = comboBoxValues[index];
					
					interpolation = Default.defaultInterpolation(method);
					showParametersPanel(true);
					showInterpolationParameters();
				}
				
				clearButton.setEnabled(index != DEFAULT_SELECTED_INDEX);
			}
		});
	}
	
	/**
	 * function for updating indexes of all next panels when one before them is removed 
	 * @param index index of panel in the interpolation list
	 */
	public void setNewIndex(int index) {
		this.index = index;
		if (optionNumberLabel != null) {
			optionNumberLabel.setText(OPTION + String.valueOf(index));
		}
	}
	
	/**
	 * show panel when showPanel is true, otherwise clear the panel
	 * @param showPanel when true, panel is selected when false, panel is removed 
	 */
	protected void showParametersPanel(boolean showPanel) {
		if (parametersPanel != null) {
			remove(parametersPanel);
			listener.onInterpolationUpdate();
		}

		if (showPanel) {
			
			createParametersPanel();
			
			if (parametersPanel != null) {
				add(parametersPanel);
				listener.onInterpolationUpdate();
			}
		}
	}

	/**
	 * create panel depending on the current interpolation
	 * @return the panel constructed by components for all parameters of the current interpolation 
	 */
	private JPanel createParametersPanel() {
		if (interpolation == null) {
			return null;
		}
		
		InterpolationMethod method = interpolation.getMethod();

		parametersPanel = null;
		switch (method) {
		case AVERAGE:
			parametersPanel = new JPanel();
			parametersPanel.setLayout(new BoxLayout(parametersPanel, BoxLayout.PAGE_AXIS));
			
			//average partial coverage
			JCheckBox averagePartialCoverageCheckBox = new JCheckBox(LABEL_PARTIAL_COVERAGE);
			parameterComponents.put(LABEL_PARTIAL_COVERAGE, averagePartialCoverageCheckBox);
			parametersPanel.add(averagePartialCoverageCheckBox);
			
			// Weighted parameter
			JPanel averageWeightedPanel = new JPanel();
			averageWeightedPanel.add(new JLabel(LABEL_WEIGHTED));
			
			JComboBox<String> averageWeightedComboBox = new JComboBox<String>(new String[] {
					Weighted.DISTANCE_WEIGHTED.toString(), Weighted.ARITHMETIC_AVERAGE.toString() });
			averageWeightedPanel.add(averageWeightedComboBox);
			
			parameterComponents.put(LABEL_WEIGHTED, averageWeightedComboBox);
			parametersPanel.add(averageWeightedPanel);
			
			break;
		case N_NEAREST_NEIGHBOR:
			parametersPanel = new JPanel();
			parametersPanel.setLayout(new BoxLayout(parametersPanel, BoxLayout.PAGE_AXIS));
			
			// n parameter
			JPanel nPanel = new JPanel();
			nPanel.add(new JLabel(LABEL_N));
			JFormattedTextField nInputField = new JFormattedTextField();
			nInputField.setColumns(5);
			nInputField.setValue(new Integer(Default.INTERPOLATION_ATTR_N));
			nInputField.setHorizontalAlignment(JTextField.RIGHT);
			nPanel.add(nInputField);
			
			parametersPanel.add(nPanel);
			parameterComponents.put(LABEL_N, nInputField);
			
			// Weighted parameter
			JPanel weightedPanel = new JPanel();
			weightedPanel.add(new JLabel(LABEL_WEIGHTED));
			
			JComboBox<String> weightedComboBox = new JComboBox<String>(new String[] {
					Weighted.DISTANCE_WEIGHTED.toString(), Weighted.ARITHMETIC_AVERAGE.toString() });
			weightedPanel.add(weightedComboBox);
			parameterComponents.put(LABEL_WEIGHTED, weightedComboBox);
			parametersPanel.add(weightedPanel);
			break;
		case CONSERVATIVE:
			parametersPanel = new JPanel();
			parametersPanel.setLayout(new BoxLayout(parametersPanel, BoxLayout.PAGE_AXIS));
			
			//enforced conservation
			JCheckBox enforcedConsrvnCheckBox = new JCheckBox(LABEL_ENFORCED_CONSERVATION); 
			parameterComponents.put(LABEL_ENFORCED_CONSERVATION, enforcedConsrvnCheckBox);
			parametersPanel.add(enforcedConsrvnCheckBox);
			
			// partial coverage
			JCheckBox partialCoverageCheckBox = new JCheckBox(LABEL_PARTIAL_COVERAGE);
			parameterComponents.put(LABEL_PARTIAL_COVERAGE, partialCoverageCheckBox);
			parametersPanel.add(partialCoverageCheckBox);
			
			// normalisation
			JPanel normalisationPanel = new JPanel();
			normalisationPanel.add(new JLabel(LABEL_NORMALISATION));
			JComboBox<String> normalisationComboBox = new JComboBox<String>(new String[] {
					Normalisation.FRACAREA.toString(), Normalisation.DESTAREA.toString() });
			normalisationPanel.add(normalisationComboBox);
			parameterComponents.put(LABEL_NORMALISATION, normalisationComboBox);
			parametersPanel.add(normalisationPanel);
			break;
		case PATCH_RECOVERY:
			parametersPanel = new JPanel();
			parametersPanel.setLayout(new BoxLayout(parametersPanel, BoxLayout.PAGE_AXIS));
			
			//allow extrapolation
			JCheckBox allowExtrapolCheckBox = new JCheckBox(LABEL_ALLOW_EXTRAPOLATION);
			parameterComponents.put(LABEL_ALLOW_EXTRAPOLATION, allowExtrapolCheckBox);
			parametersPanel.add(allowExtrapolCheckBox);
			
			//extend Source Patch
			JPanel extendSourcePatchPanel = new JPanel();
			extendSourcePatchPanel.add(new JLabel(LABEL_EXTEND_SOURCE_PATCH));
			JFormattedTextField extendSourcePatchField = new JFormattedTextField();
			extendSourcePatchField.setColumns(5);
			extendSourcePatchField.setValue(new Integer(Default.INTERPOLATION_ATTR_EXTEND_SOURCE_PATCH));
			extendSourcePatchField.setHorizontalAlignment(JTextField.RIGHT);
			extendSourcePatchPanel.add(extendSourcePatchField);
			parameterComponents.put(LABEL_EXTEND_SOURCE_PATCH, extendSourcePatchField);
			parametersPanel.add(extendSourcePatchPanel);
			
			// polynomial order
			JPanel polynomialOrderPanel = new JPanel();
			polynomialOrderPanel.add(new JLabel(LABEL_POLYNOMIAL_ORDER));
			JFormattedTextField polynomialOrderField = new JFormattedTextField();
			polynomialOrderField.setColumns(5);
			polynomialOrderField.setValue(new Integer(Default.INTERPOLATION_ATTR_POLYNOMIAL_ORDER));
			polynomialOrderField.setHorizontalAlignment(JTextField.RIGHT);
			polynomialOrderPanel.add(polynomialOrderField);
			parameterComponents.put(LABEL_POLYNOMIAL_ORDER, polynomialOrderField);
			parametersPanel.add(polynomialOrderPanel);
			
			// gauss order
			JPanel gaussOrderPanel = new JPanel();
			gaussOrderPanel.add(new JLabel(LABEL_GAUSS_ORDER));
			JFormattedTextField gaussOrderField = new JFormattedTextField();
			gaussOrderField.setColumns(5);
			gaussOrderField.setValue(new Integer(Default.INTERPOLATION_ATTR_GAUSS_ORDER));
			gaussOrderField.setHorizontalAlignment(JTextField.RIGHT);
			gaussOrderPanel.add(gaussOrderField);
			parameterComponents.put(LABEL_GAUSS_ORDER, gaussOrderField);
			parametersPanel.add(gaussOrderPanel);
			break;
		case FIXED_VALUE:
			parametersPanel = new JPanel();
			// user value
			JLabel userValueLabel = new JLabel(LABEL_USER_VALUE);
			JFormattedTextField userValueInputField = new JFormattedTextField();
			userValueInputField.setColumns(10);
			userValueInputField.setValue(new Double(Default.INTERPOLATION_ATTR_USER_VALUE));
			userValueInputField.setHorizontalAlignment(JTextField.RIGHT);
			parametersPanel.add(userValueLabel);
			parametersPanel.add(userValueInputField);
			parameterComponents.put(LABEL_USER_VALUE, userValueInputField);
			break;
		case USER_FILE:
			parametersPanel = new JPanel();
			parametersPanel.setLayout(new BoxLayout(parametersPanel, BoxLayout.PAGE_AXIS));
			
			//file name
			FilenamePanel filenamePanel = new FilenamePanel();
			parametersPanel.add(filenamePanel);
			parameterComponents.put(LABEL_FILE, filenamePanel);
			
			JPanel yacLocationPanel = new JPanel();
			yacLocationPanel.add(new JLabel(LABEL_YAC_LOCATION));
			JComboBox<String> yacLocationComboBox = new JComboBox<String>(new String[] {
					YacLocation.CELL.toString(), YacLocation.EDGE.toString(), YacLocation.CORNER.toString()});
			yacLocationPanel.add(yacLocationComboBox);
			parameterComponents.put(LABEL_YAC_LOCATION, yacLocationComboBox);
			parametersPanel.add(yacLocationPanel);
			break;
		case SOURCE2TARGET_MAP:
			parametersPanel = new JPanel();
			parametersPanel.setLayout(new BoxLayout(parametersPanel, BoxLayout.PAGE_AXIS));			
			break;
		case SMOOTHED_PATCH_RECOVERY:
			parametersPanel = new JPanel();
			parametersPanel.setLayout(new BoxLayout(parametersPanel, BoxLayout.PAGE_AXIS));
			
			//allow extrapolation
			JCheckBox allowSmoothedExtrapolCheckBox = new JCheckBox(LABEL_ALLOW_EXTRAPOLATION);
			parameterComponents.put(LABEL_ALLOW_EXTRAPOLATION, allowSmoothedExtrapolCheckBox);
			parametersPanel.add(allowSmoothedExtrapolCheckBox);
			
			//extend Source Patch
			JPanel extendSmoothedSourcePatchPanel = new JPanel();
			extendSmoothedSourcePatchPanel.add(new JLabel(LABEL_EXTEND_SOURCE_PATCH));
			JFormattedTextField extendSmoothedSourcePatchField = new JFormattedTextField();
			extendSmoothedSourcePatchField.setColumns(5);
			extendSmoothedSourcePatchField.setValue(new Integer(Default.INTERPOLATION_ATTR_EXTEND_SOURCE_PATCH));
			extendSmoothedSourcePatchField.setHorizontalAlignment(JTextField.RIGHT);
			extendSmoothedSourcePatchPanel.add(extendSmoothedSourcePatchField);
			parameterComponents.put(LABEL_EXTEND_SOURCE_PATCH, extendSmoothedSourcePatchField);
			parametersPanel.add(extendSmoothedSourcePatchPanel);
			
			// polynomial order
			JPanel polynomialSmoothedOrderPanel = new JPanel();
			polynomialSmoothedOrderPanel.add(new JLabel(LABEL_POLYNOMIAL_ORDER));
			JFormattedTextField polynomialSmoothedOrderField = new JFormattedTextField();
			polynomialSmoothedOrderField.setColumns(5);
			polynomialSmoothedOrderField.setValue(new Integer(Default.INTERPOLATION_ATTR_POLYNOMIAL_ORDER));
			polynomialSmoothedOrderField.setHorizontalAlignment(JTextField.RIGHT);
			polynomialSmoothedOrderPanel.add(polynomialSmoothedOrderField);
			parameterComponents.put(LABEL_POLYNOMIAL_ORDER, polynomialSmoothedOrderField);
			parametersPanel.add(polynomialSmoothedOrderPanel);
			
			// gauss order
			JPanel gaussSmoothedOrderPanel = new JPanel();
			gaussSmoothedOrderPanel.add(new JLabel(LABEL_GAUSS_ORDER));
			JFormattedTextField gaussSmoothedOrderField = new JFormattedTextField();
			gaussSmoothedOrderField.setColumns(5);
			gaussSmoothedOrderField.setValue(new Integer(Default.INTERPOLATION_ATTR_GAUSS_ORDER));
			gaussSmoothedOrderField.setHorizontalAlignment(JTextField.RIGHT);
			gaussSmoothedOrderPanel.add(gaussSmoothedOrderField);
			parameterComponents.put(LABEL_GAUSS_ORDER, gaussSmoothedOrderField);
			parametersPanel.add(gaussSmoothedOrderPanel);
			
			//allow corner weight
			JCheckBox allowCornerWeightCheckBox = new JCheckBox(LABEL_ALLOW_CORNER_WEIGHT);
			parameterComponents.put(LABEL_ALLOW_CORNER_WEIGHT, allowCornerWeightCheckBox);
			parametersPanel.add(allowCornerWeightCheckBox);
			
			//corner weight
			JPanel cornerWeightPanel = new JPanel();
			cornerWeightPanel.add(new JLabel(LABEL_CORNER_WEIGHT));
			JFormattedTextField cornerWeightField = new JFormattedTextField();
			cornerWeightField.setColumns(5);
			cornerWeightField.setValue(new Integer(Default.INTERPOLATION_ATTR_CORNER_WEIGHT));
			cornerWeightField.setHorizontalAlignment(JTextField.RIGHT);
			cornerWeightField.setFormatterFactory(new JFormattedTextField.AbstractFormatterFactory() {

		        @Override
		        public AbstractFormatter getFormatter(JFormattedTextField tf) {
		        	//set number symbols using US symbols
		        	NumberFormat nf = NumberFormat.getNumberInstance(Locale.US);
		            DecimalFormat format = (DecimalFormat)nf;
		            //set maximun fraction digits 5, like 0.00001, for the reason that more digits can not completely show in the field
		            format.setMaximumFractionDigits(5);
		            //Behaves as for RoundingMode.UP if the discarded fraction is  0.5
		            format.setRoundingMode(RoundingMode.HALF_UP);
		            InternationalFormatter formatter = new InternationalFormatter(format);
		            //allow user to enter any inputs at first time no matter valid or not, check the validation when finish the edit of this field
		            formatter.setAllowsInvalid(true);
		            formatter.setMinimum(0.0);
		            formatter.setMaximum(1.0);
		            return formatter;         
		        }
		    });
			cornerWeightPanel.add(cornerWeightField);
			parameterComponents.put(LABEL_CORNER_WEIGHT, cornerWeightField);
			parametersPanel.add(cornerWeightPanel);
			if (allowCornerWeightCheckBox.isSelected()) {
				cornerWeightField.setEnabled(true);
			} else {
				cornerWeightField.setEnabled(false);
			}
			
			break;
		default:
			break;
		}

		return parametersPanel;
	}

	/**
 	 * set the parameter values according to the method
	 */
	private void showInterpolationParameters() {
		InterpolationMethod method = interpolation.getMethod();
		
		// according to the method, set the parameter values
		switch (method) {
		case AVERAGE:
			AverageInterpolation average = (AverageInterpolation) interpolation;
			JCheckBox averagePartialCoverageCheckBox = (JCheckBox) parameterComponents.get(LABEL_PARTIAL_COVERAGE);
			averagePartialCoverageCheckBox.setSelected(average.getPartialCoverage());
			
			@SuppressWarnings("unchecked")
			JComboBox<String> averageWeightedComboBox = (JComboBox<String>) parameterComponents.get(LABEL_WEIGHTED);
			averageWeightedComboBox.setSelectedItem(average.getWeighted().toString());
			break;
			
			
		case N_NEAREST_NEIGHBOR:
			NNearestNeighborInterpolation nnearestNeighbor = (NNearestNeighborInterpolation) interpolation;
			
			JFormattedTextField nInputField = (JFormattedTextField) parameterComponents.get(LABEL_N);
			nInputField.setText(String.valueOf(nnearestNeighbor.getN()));
			
			@SuppressWarnings("unchecked")
			JComboBox<String> weightedComboBox = (JComboBox<String>) parameterComponents.get(LABEL_WEIGHTED);
			weightedComboBox.setSelectedItem(nnearestNeighbor.getWeighted().toString());
			break;
		case CONSERVATIVE:
			ConservativeInterpolation conservative = (ConservativeInterpolation) interpolation;
			
			JCheckBox enforcedConsrvnCheckBox = (JCheckBox) parameterComponents.get(LABEL_ENFORCED_CONSERVATION);
			enforcedConsrvnCheckBox.setSelected(conservative.getEnforcedConservation()); 
			
			JCheckBox partialCoverageCheckBox = (JCheckBox) parameterComponents.get(LABEL_PARTIAL_COVERAGE);
			partialCoverageCheckBox.setSelected(conservative.getPartialCoverage());
			
			@SuppressWarnings("unchecked")
			JComboBox<String> normalisationComboBox = (JComboBox<String>) parameterComponents.get(LABEL_NORMALISATION);
			normalisationComboBox.setSelectedItem(conservative.getNormalisation().toString());
			break;
		case PATCH_RECOVERY:
			PatchRecoveryInterpolation patchRecovery = (PatchRecoveryInterpolation) interpolation;
			
			JCheckBox allowExtrapolationCheckBox = (JCheckBox) parameterComponents.get(LABEL_ALLOW_EXTRAPOLATION);
			allowExtrapolationCheckBox.setSelected(patchRecovery.isAllowExtrapolation());
			
			JFormattedTextField extendSourcePatchField = (JFormattedTextField) parameterComponents.get(LABEL_EXTEND_SOURCE_PATCH);
			extendSourcePatchField.setText(String.valueOf(patchRecovery.getExtendSourcePatch()));
			
			JFormattedTextField polynomialOrderField = (JFormattedTextField) parameterComponents.get(LABEL_POLYNOMIAL_ORDER);
			polynomialOrderField.setText(String.valueOf(patchRecovery.getPolynomialOrder()));
			
			JFormattedTextField gaussOrderField = (JFormattedTextField) parameterComponents.get(LABEL_GAUSS_ORDER);
			gaussOrderField.setText(String.valueOf(patchRecovery.getGaussOrder()));
			break;
		case FIXED_VALUE:
			FixedValueInterpolation fixedValue = (FixedValueInterpolation) interpolation;
			
			JFormattedTextField userValueInputField = (JFormattedTextField) parameterComponents.get(LABEL_USER_VALUE);
			userValueInputField.setText(String.valueOf(fixedValue.getUserValue()));
			break;
		case USER_FILE:
			UserFileInterpolation userFileInterpolation = (UserFileInterpolation) interpolation;
			
			FilenamePanel filenamePanel = (FilenamePanel) parameterComponents.get(LABEL_FILE);
			String filename = userFileInterpolation.getFilename();
			
			filenamePanel.setFilename(filename);
			
			@SuppressWarnings("unchecked")
			JComboBox<String> yacLocationComboBox = (JComboBox<String>) parameterComponents.get(LABEL_YAC_LOCATION);
			yacLocationComboBox.setSelectedItem(userFileInterpolation.getYacLocation().toString());
			break;
		case SOURCE2TARGET_MAP:
			Source2TargetMapInterpolation source2targetmap = (Source2TargetMapInterpolation) interpolation;
			break;
		case SMOOTHED_PATCH_RECOVERY:
			SmoothedPatchRecoveryInterpolation smoothedPatchRecovery = (SmoothedPatchRecoveryInterpolation) interpolation;
			
			JCheckBox allowSmoothedExtrapolationCheckBox = (JCheckBox) parameterComponents.get(LABEL_ALLOW_EXTRAPOLATION);
			allowSmoothedExtrapolationCheckBox.setSelected(smoothedPatchRecovery.isAllowExtrapolation());
			
			JFormattedTextField extendSmoothedSourcePatchField = (JFormattedTextField) parameterComponents.get(LABEL_EXTEND_SOURCE_PATCH);
 			extendSmoothedSourcePatchField.setText(String.valueOf(smoothedPatchRecovery.getExtendSourcePatch()));
			
			JFormattedTextField polynomialSmoothedOrderField = (JFormattedTextField) parameterComponents.get(LABEL_POLYNOMIAL_ORDER);
			polynomialSmoothedOrderField.setText(String.valueOf(smoothedPatchRecovery.getPolynomialOrder()));
			
			JFormattedTextField gaussSmoothedOrderField = (JFormattedTextField) parameterComponents.get(LABEL_GAUSS_ORDER);
			gaussSmoothedOrderField.setText(String.valueOf(smoothedPatchRecovery.getGaussOrder()));
			
			JCheckBox allowCornerWeightCheckBox = (JCheckBox) parameterComponents.get(LABEL_ALLOW_CORNER_WEIGHT);
			allowCornerWeightCheckBox.setSelected(smoothedPatchRecovery.isAllowCornerWeight());
			
			JFormattedTextField cornerWeightField = (JFormattedTextField) parameterComponents.get(LABEL_CORNER_WEIGHT);
			cornerWeightField.setText(String.valueOf(smoothedPatchRecovery.getCornerWeight()));
			if (allowCornerWeightCheckBox.isSelected()) {
				cornerWeightField.setEnabled(true);
			} else {
				cornerWeightField.setEnabled(false);
			}
			
			allowCornerWeightCheckBox.addActionListener(new ActionListener() {			
				@Override
				public void actionPerformed(ActionEvent arg0) {
					cornerWeightField.setEnabled(allowCornerWeightCheckBox.isSelected());
				}
			});
			break;
		default:
			break;
		}
	}
	
	/**
	* get the parameter values according to the method, at the end of each case stores informations in to the member object interpolation
    */
	private void collectInterpolationParameters() {
		if (interpolation == null) { //no interpolation is selected, just return
			return; 
		}
		
		InterpolationMethod method = interpolation.getMethod();
		
		// according to the method, get the parameter values
		switch (method) {
		case AVERAGE:
			JCheckBox averagePartialCoverageCheckBox = (JCheckBox) parameterComponents.get(LABEL_PARTIAL_COVERAGE);
			boolean averagePartialCoverage = (averagePartialCoverageCheckBox.isSelected());
			
			@SuppressWarnings("unchecked")
			JComboBox<String> averageWeightedComboBox = (JComboBox<String>) parameterComponents.get(LABEL_WEIGHTED);
			String averageWeightedValue = (String) averageWeightedComboBox.getSelectedItem();
			
			interpolation = new AverageInterpolation(averagePartialCoverage, Weighted.objectFromString(averageWeightedValue));
			break;
		case N_NEAREST_NEIGHBOR:
			JFormattedTextField nInputField = (JFormattedTextField) parameterComponents.get(LABEL_N);
			String nValue = nInputField.getText();
			
			@SuppressWarnings("unchecked")
			JComboBox<String> weightedComboBox = (JComboBox<String>) parameterComponents.get(LABEL_WEIGHTED);
			String weightedValue = (String) weightedComboBox.getSelectedItem();
			
			interpolation = new NNearestNeighborInterpolation(Integer.valueOf(nValue), 
					Weighted.objectFromString(weightedValue));
			break;
		case CONSERVATIVE:
			JCheckBox enforcedConsrvnCheckBox = (JCheckBox) parameterComponents.get(LABEL_ENFORCED_CONSERVATION);
			boolean enforcedConservation = enforcedConsrvnCheckBox.isSelected();
			
			JCheckBox partialCoverageCheckBox = (JCheckBox) parameterComponents.get(LABEL_PARTIAL_COVERAGE);
			boolean partialCoverage = partialCoverageCheckBox.isSelected();
			
			@SuppressWarnings("unchecked")
			JComboBox<String> normalisationComboBox = (JComboBox<String>) parameterComponents.get(LABEL_NORMALISATION);
			String normalisationValue = (String) normalisationComboBox.getSelectedItem();
			
			Normalisation normalisation = Normalisation.objectFromString(normalisationValue);
			interpolation = new ConservativeInterpolation(enforcedConservation, 
					partialCoverage, normalisation);
			break;
		case PATCH_RECOVERY:
			JCheckBox allowExtrapolationCheckBox = (JCheckBox) parameterComponents.get(LABEL_ALLOW_EXTRAPOLATION);
			boolean allowExtrapolation = allowExtrapolationCheckBox.isSelected();
			
			JFormattedTextField extendSourcePatchField = (JFormattedTextField) parameterComponents.get(LABEL_EXTEND_SOURCE_PATCH);
			String extendSourcePatchValue = extendSourcePatchField.getText();
			
			JFormattedTextField polynomialOrderField = (JFormattedTextField) parameterComponents.get(LABEL_POLYNOMIAL_ORDER);
			String polynomialOrderValue = polynomialOrderField.getText();
			
			JFormattedTextField gaussOrderField = (JFormattedTextField) parameterComponents.get(LABEL_GAUSS_ORDER);
			String gaussOrderValue = gaussOrderField.getText();
			
			interpolation = new PatchRecoveryInterpolation(allowExtrapolation, Integer.valueOf(extendSourcePatchValue), 
					Integer.valueOf(polynomialOrderValue), Integer.valueOf(gaussOrderValue));
			break;
		case FIXED_VALUE:
			JFormattedTextField userValueInputField = (JFormattedTextField) parameterComponents.get(LABEL_USER_VALUE);
			String userValue = userValueInputField.getText();
			interpolation = new FixedValueInterpolation(Double.valueOf(userValue));
			break;
		case USER_FILE:
			FilenamePanel filenamePanel = (FilenamePanel) parameterComponents.get(LABEL_FILE);
			String filePath = filenamePanel.getFilename();
			
			@SuppressWarnings("unchecked")
			JComboBox<String> yacLocationComboBox = (JComboBox<String>) parameterComponents.get(LABEL_YAC_LOCATION);
			String yacLocationValue = (String) yacLocationComboBox.getSelectedItem();
			
			YacLocation yacLocation = YacLocation.objectFromString(yacLocationValue);
			interpolation = new UserFileInterpolation(filePath, yacLocation);
			break;
		case SOURCE2TARGET_MAP:
//          no parameter 
			break;
		case SMOOTHED_PATCH_RECOVERY:
			JCheckBox allowSmoothedExtrapolationCheckBox = (JCheckBox) parameterComponents.get(LABEL_ALLOW_EXTRAPOLATION);
			boolean allowSmoothedExtrapolation = allowSmoothedExtrapolationCheckBox.isSelected();
			
			JFormattedTextField extendSmoothedSourcePatchField = (JFormattedTextField) parameterComponents.get(LABEL_EXTEND_SOURCE_PATCH);
			String extendSmoothedSourcePatchValue = extendSmoothedSourcePatchField.getText();
			
			JFormattedTextField polynomialSmoothedOrderField = (JFormattedTextField) parameterComponents.get(LABEL_POLYNOMIAL_ORDER);
			String polynomialSmoothedOrderValue = polynomialSmoothedOrderField.getText();
			
			JFormattedTextField gaussSmoothedOrderField = (JFormattedTextField) parameterComponents.get(LABEL_GAUSS_ORDER);
			String gaussSmoothedOrderValue = gaussSmoothedOrderField.getText();
			
			JCheckBox allowCornerWeightCheckBox = (JCheckBox) parameterComponents.get(LABEL_ALLOW_CORNER_WEIGHT);
			boolean allowCornerWeight = allowCornerWeightCheckBox.isSelected();
			
			JFormattedTextField cornerWeightField = (JFormattedTextField) parameterComponents.get(LABEL_CORNER_WEIGHT);

			String cornerWeightValue;
			if (allowCornerWeight) {
				cornerWeightValue = cornerWeightField.getText();
			} else {
				cornerWeightValue = Default.INTERPOLATION_ATTR_CORNER_WEIGHT;
			}
			
			interpolation = new SmoothedPatchRecoveryInterpolation(allowSmoothedExtrapolation, Integer.valueOf(extendSmoothedSourcePatchValue), 
					Integer.valueOf(polynomialSmoothedOrderValue), Integer.valueOf(gaussSmoothedOrderValue), allowCornerWeight ,Float.valueOf(cornerWeightValue));
			break;
		default:
			break;
		}
	}
}
