package de.mpg.mpimet.yacgui.ui.dialog.tabpanel;

import java.awt.Dimension;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import de.mpg.mpimet.yacgui.ui.dialog.tabpanel.InterpolationPanel.InterpolationUpdateListener;
import de.mpg.mpimet.yacgui.xml.TransientCouple;
import de.mpg.mpimet.yacgui.xml.interpolation.AverageInterpolation;
import de.mpg.mpimet.yacgui.xml.interpolation.Interpolation;
import de.mpg.mpimet.yacgui.xml.interpolation.InterpolationMethod;

/**
 * Definite the class of InterpolationsTab create components and update their status
 * @author Maxim Yastremsky
 * @update Junpei Xu 30/06/2016
 */
public class InterpolationsTab extends AbstractTab implements InterpolationUpdateListener {
	
	private static final long serialVersionUID = -7108123285574614990L;

	private static final String LABEL_ENFORCE_WRITE_WEIGHT_FILE = "Enforce write weight file";
	private static final String LABEL_USE_SOURCE_MASK = "Use source mask";
	private static final String LABEL_USE_TARGET_MASK = "Use target mask";

	public static final int DEFAULT_INTERPOLATIONS_NUMBER = 3;
	
	public static final int VIEW_PORT_WIDTH = 440;
	public static final int MAX_VIEWPORT_HEIGHT = 530;

	protected static final int MAX_INTERPOLATION_NUMBER = 15;

	private JPanel interpltnsContainerPanel;
	private List<InterpolationPanel> interpolationPanelsList = new ArrayList<InterpolationPanel>();
	
	private int interpNumber = 0;

	private JCheckBox enforceWriteWeightFileCheckBox;
	private FilenamePanel filenamePanel;

	private JCheckBox useSourceMaskCheckBox;
	private JCheckBox useTargetMaskCheckBox;

	private JButton addInterpolationButton;

	public InterpolationsTab(String dialogDescription, TransientCouple transientCouple) {
		super(dialogDescription, transientCouple);
		createLayout();
	}

	/**
	 * collect transient information on interpolationTab including enforce write weight file, use source mask, use target mask, interpolation 
	 */
	public void collectData() {
		transientCouple.setEnforceWriteWeightFile(enforceWriteWeightFileCheckBox.isSelected());
		transientCouple.setWeightFilename(enforceWriteWeightFileCheckBox.isSelected() ? filenamePanel.getFilename() : null);
		transientCouple.setUseSourceMask(useSourceMaskCheckBox.isSelected());
		transientCouple.setUseTargetMask(useTargetMaskCheckBox.isSelected());
		transientCouple.setInterpolations(getInterpolations());
	}

	@Override
	protected JComponent createCenterPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.PAGE_AXIS));
		
		//writing weights of interpolation stack
		JPanel writingWeightsPanel = new JPanel();
		writingWeightsPanel.setLayout(new BoxLayout(writingWeightsPanel, BoxLayout.PAGE_AXIS));
		
		enforceWriteWeightFileCheckBox = new JCheckBox(LABEL_ENFORCE_WRITE_WEIGHT_FILE);
		writingWeightsPanel.add(enforceWriteWeightFileCheckBox);
		
		filenamePanel = new FilenamePanel();
		writingWeightsPanel.add(filenamePanel);

		enforceWriteWeightFileCheckBox.addItemListener(new ItemListener() {
			
			@Override
			public void itemStateChanged(ItemEvent e) {
				if (enforceWriteWeightFileCheckBox.isSelected()) {
					filenamePanel.setEnabled(true);
				} else {
					filenamePanel.clean();
					filenamePanel.setEnabled(false);
				}
			}
		});
		panel.add(writingWeightsPanel);
		
		// label
		JLabel chooseLabel = new JLabel("Choose preferred interpolation method.");
		panel.add(chooseLabel);
		
		// use source mask parameter
		this.useSourceMaskCheckBox = new JCheckBox(LABEL_USE_SOURCE_MASK);
		panel.add(useSourceMaskCheckBox);
		
		// use target mask parameter
		this.useTargetMaskCheckBox = new JCheckBox(LABEL_USE_TARGET_MASK);
		panel.add(useTargetMaskCheckBox);
		
		// the interpolation options panel and its layout
		interpltnsContainerPanel = new JPanel();
		interpltnsContainerPanel.setLayout(new BoxLayout(interpltnsContainerPanel, BoxLayout.PAGE_AXIS));
		
		// add them to the intermediate panel
		panel.add(interpltnsContainerPanel);

		// button for adding more options
		addInterpolationButton = new JButton("Add interpolation");
		addInterpolationButton.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				
				addInterpolationPanel(new InterpolationPanel(InterpolationsTab.this, interpNumber++));
				onInterpolationUpdate();
				
				if (interpNumber >= MAX_INTERPOLATION_NUMBER) {
					addInterpolationButton.setEnabled(false);
				}
			}
		});
		panel.add(addInterpolationButton);
		
		loadParameters();
		
		// scrollable panel for interpolation parameters
		JScrollPane listScroller = new JScrollPane(panel);
		Dimension preferredSize = listScroller.getPreferredSize();
		int height = Math.min(preferredSize.height, MAX_VIEWPORT_HEIGHT);
		
		listScroller.getViewport().setPreferredSize(new Dimension(VIEW_PORT_WIDTH, height));
		listScroller.setBorder(BorderFactory.createEmptyBorder());
		return listScroller;
	}
	
	@Override
	/**
	 * load parameters enforce write weight file, use source mask, use target mask and interpolations for a transientCouple
	 * traverse all interpolations this transientCouple has
	 */
	public void loadParameters() {
		enforceWriteWeightFileCheckBox.setSelected(transientCouple.isEnforceWriteWeightFile());
		
		if (transientCouple.isEnforceWriteWeightFile()) {
			filenamePanel.setFilename(transientCouple.getWeightFilename());
		} else {
			filenamePanel.setEnabled(false);
		}
		
		useSourceMaskCheckBox.setSelected(transientCouple.getUseSourceMask());
		useTargetMaskCheckBox.setSelected(transientCouple.getUseTargetMask());

		interpltnsContainerPanel.removeAll();
		interpolationPanelsList.clear();
		interpNumber = 0;
		
		if (transientCouple.getInterpolations() != null && transientCouple.getInterpolations().size() > 0) {
			for (Interpolation interpolation : transientCouple.getInterpolations()) {
				addInterpolationPanel(new InterpolationPanel(this, interpNumber++, interpolation));
			}
		} 
		
		while (interpolationPanelsList.size() < DEFAULT_INTERPOLATIONS_NUMBER) {
			addInterpolationPanel(new InterpolationPanel(this, interpNumber++));
		}
	}
	
	/**
	*function of loading interpolation to panel and expand interpolationPanels list when it is more than 3
	*@param InterpolationPanel panel
	*/
	private void addInterpolationPanel(InterpolationPanel panel) {
		interpltnsContainerPanel.add(panel);
		interpolationPanelsList.add(panel);
	}

	@Override
	public void onInterpolationUpdate() {
		revalidate();
	}
	
	@Override
	/**
	 * function will be called when press clear button {@link InterpolationPanel}
	 * remove the index of this panel and update indexes of all next panels 
	 * @param panel the interpolation panel to be removed
	 */
	public void onInterpolationRemoved(InterpolationPanel panel) {
		if (interpolationPanelsList.size() > DEFAULT_INTERPOLATIONS_NUMBER) {
			//get the index of panel in the list
			int index = interpolationPanelsList.indexOf(panel);
			
			interpolationPanelsList.remove(panel);
			interpltnsContainerPanel.remove(panel);
			
			//update indexes of all next interpolation panels
			int i = index;
			for (; i < interpolationPanelsList.size(); i++) {
				interpolationPanelsList.get(i).setNewIndex(i);
			}
			interpNumber = i;
			
			onInterpolationUpdate();
		}
		
		if (interpNumber < MAX_INTERPOLATION_NUMBER) {
			addInterpolationButton.setEnabled(true);
		}
	}

	/**
	 * get interpolations from panels for collecting data
	 * @return interpolations the list of interpolations collected one by one from panels
	 */
	private List<Interpolation> getInterpolations() {
		List<Interpolation> interpolations = new ArrayList<Interpolation>();
		for (InterpolationPanel panel : interpolationPanelsList) {
			Interpolation interpolation = panel.getInterpolation();
			if (interpolation != null) {
				interpolations.add(interpolation);
			}
		}
		return interpolations;
	}
}
