package de.mpg.mpimet.yacgui.ui.dialog.tabpanel;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.NumberFormat;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import de.mpg.mpimet.yacgui.xml.DebugMode;
import de.mpg.mpimet.yacgui.xml.TransientCouple;

public class MoreTab extends AbstractTab {

	private static final long serialVersionUID = -4615817689584953223L;

	private static final String LABEL_DEBUG_MODE = "Debug mode: ";
	private static final String LABEL_ENFORCE_WRITE_RESTART = "Enforce write restart";
	private static final String LABEL_MASK_VALUE = "Mask value (optional): ";
	
	private static final String CHECKBOX_BEFORE_INTERPOLATION = " at source before interpolation";
	private static final String CHECKBOX_AFTER_INTERPOLATION = " at source after interpolation";
	private static final String CHECKBOX_AT_TARGET = " at target";
	
	private static final int DEFAULT_COLUMN_NUM = 7;

	private Double maskValue;
	private DebugMode debugMode;
	private boolean enforceWriteRestart;
	
	private JCheckBox atSourceBeforeInterpolationCheckBox;
	private JCheckBox atSourceAfterInterpolationCheckBox;
	private JCheckBox atTargetCheckBox;

	private JCheckBox enforceWriteRestartCheckBox;

	private JCheckBox maskValueCheckBox;
	private JFormattedTextField maskValueTextField;

	public MoreTab(String dialogDescription, TransientCouple transientCouple) {
		super(dialogDescription, transientCouple);
		createLayout();
	}

	public void collectData() {
		debugMode.setAtSourceBeforeInterpolation(atSourceBeforeInterpolationCheckBox.isSelected());
		debugMode.setAtSourceAfterInterpolation(atSourceAfterInterpolationCheckBox.isSelected());
		debugMode.setAtTarget(atTargetCheckBox.isSelected());
		
		transientCouple.setEnforceWriteRestart(enforceWriteRestartCheckBox.isSelected());
		
		if (maskValueCheckBox.isSelected()) {
			maskValue = ((Number) maskValueTextField.getValue()).doubleValue();
			transientCouple.setMaskValue(maskValue);
		} else {
			transientCouple.setMaskValue(null);
		}
	}

	@Override
	protected JPanel createCenterPanel() {
		JPanel moreOptionsPanel = new JPanel();
		moreOptionsPanel.setLayout(new BoxLayout(moreOptionsPanel, BoxLayout.PAGE_AXIS));
		
		// debug mode
		JPanel debugModePanel = new JPanel();
		debugModePanel.setLayout(new BoxLayout(debugModePanel, BoxLayout.PAGE_AXIS));
		JLabel debugModeLabel = new JLabel(LABEL_DEBUG_MODE);
		debugModeLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		debugModeLabel.setBorder(new EmptyBorder(5, 5, 5, 5));
		debugModePanel.add(debugModeLabel);
		
		// at source before interpolation
		atSourceBeforeInterpolationCheckBox = new JCheckBox(CHECKBOX_BEFORE_INTERPOLATION);
		atSourceBeforeInterpolationCheckBox.setAlignmentX(Component.CENTER_ALIGNMENT);
		debugModePanel.add(atSourceBeforeInterpolationCheckBox);
		
		// at source after interpolation
		atSourceAfterInterpolationCheckBox = new JCheckBox(CHECKBOX_AFTER_INTERPOLATION);
		atSourceAfterInterpolationCheckBox.setAlignmentX(Component.CENTER_ALIGNMENT);
		debugModePanel.add(atSourceAfterInterpolationCheckBox);
		
		// at target
		atTargetCheckBox = new JCheckBox(CHECKBOX_AT_TARGET);
		atTargetCheckBox.setAlignmentX(Component.CENTER_ALIGNMENT);
		debugModePanel.add(atTargetCheckBox);

		// enforce write-restart
		JPanel enforceWriteRestartPanel = new JPanel();
		enforceWriteRestartPanel.setLayout(new BoxLayout(enforceWriteRestartPanel, BoxLayout.PAGE_AXIS));
		enforceWriteRestartCheckBox = new JCheckBox(LABEL_ENFORCE_WRITE_RESTART);
		enforceWriteRestartCheckBox.setAlignmentX(Component.CENTER_ALIGNMENT);
		enforceWriteRestartPanel.add(enforceWriteRestartCheckBox);
		
		// mask value checkbox
		JPanel maskPanel = new JPanel();
		maskValueCheckBox = new JCheckBox(LABEL_MASK_VALUE);
		maskPanel.add(maskValueCheckBox);
		
		// mask value input
		NumberFormat format = NumberFormat.getNumberInstance();
		format.setMinimumFractionDigits(1);
		maskValueTextField = new JFormattedTextField(format);
		maskValueTextField.setColumns(DEFAULT_COLUMN_NUM);
		maskValueTextField.setHorizontalAlignment(JTextField.RIGHT);
		
		maskPanel.add(maskValueTextField);
		
		// mask value checkbox action listener
		maskValueCheckBox.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				maskValueTextField.setEnabled(maskValueCheckBox.isSelected());
			}
		});
		
		moreOptionsPanel.add(debugModePanel);
		moreOptionsPanel.add(Box.createVerticalGlue());
		moreOptionsPanel.add(enforceWriteRestartPanel);
		moreOptionsPanel.add(Box.createVerticalGlue());
		moreOptionsPanel.add(maskPanel);
		
		loadParameters();
		
		return moreOptionsPanel;
	}
	
	@Override
	public void loadParameters() {
		
		this.maskValue = transientCouple.getMaskValue();
		this.debugMode = transientCouple.getDebugMode();
		this.enforceWriteRestart = transientCouple.isEnforceWriteRestart();
		
		atSourceBeforeInterpolationCheckBox.setSelected(debugMode.isAtSourceBeforeInterpolation());
		atSourceAfterInterpolationCheckBox.setSelected(debugMode.isAtSourceAfterInterpolation());
		atTargetCheckBox.setSelected(debugMode.isAtTarget());
		enforceWriteRestartCheckBox.setSelected(enforceWriteRestart);
		
		maskValueCheckBox.setSelected(maskValue != null);
		
		if (maskValue != null) {
			maskValueTextField.setValue(new Double(maskValue));
			maskValueTextField.setEnabled(true);
		} else {
			maskValueTextField.setValue(new Double(0.0));
			maskValueTextField.setEnabled(false);
		}		
	}
}
