package de.mpg.mpimet.yacgui.ui.dialog.tabpanel;


import java.text.ParseException;


import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JFormattedTextField.AbstractFormatter;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JWindow;
import javax.swing.border.EmptyBorder;
import javax.swing.text.DefaultFormatter;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.DocumentFilter;
import javax.swing.text.InternationalFormatter;
import javax.swing.text.MaskFormatter;
import javax.swing.text.NumberFormatter;


import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import java.awt.GridLayout;
import java.awt.MouseInfo;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import de.mpg.mpimet.yacgui.xml.CouplingOperation;
//import de.mpg.mpimet.yacgui.xml.Timelag;
import de.mpg.mpimet.yacgui.xml.Timestep;
import de.mpg.mpimet.yacgui.xml.TimestepUnit;
import de.mpg.mpimet.yacgui.xml.TransientCouple;
import de.mpg.mpimet.yacgui.xml.interpolation.NNearestNeighborInterpolation.Weighted;

/**
 * Creates components on Timestep tab and provides functions responsible for collecting and loading data
 *
 * @author Maxim Yastremsky
 */

public class TimestepTab extends AbstractTab {

	private static final long serialVersionUID = 1287116936937629062L;
	
	private static final int DEFAULT_COLUMN_NUM = 10;

	private final static String TIMESTEP_UNIT_LABEL = "model timestep unit(s)";

	private TimestepUnit timestepUnit;
	private Timestep timestep;
	
	private JFormattedTextField sourceTimestepInputField;
	private JFormattedTextField targetTimestepInputField;
	private JFormattedTextField couplingPeriodInputField;
	
	private JComboBox<String> operationList;

	//private JFormattedTextField sourceTimeLagInputField;
	private JFormattedTextField targetTimeLagInputField;
	private JFormattedTextField sourceTimeLagInputField;
	private TimestepFormatter timestepFormatter;
	private TimestepFormatter timelagFormatter;

	public TimestepTab(String dialogDescription, TransientCouple transientCouple, TimestepUnit timestepUnit) {
		super(dialogDescription, transientCouple);
		
		this.timestepUnit = timestepUnit;
		createLayout();
	}
	
	/**
	 * get values from input field to compose timestep
	 */
	public void collectData() {
		Object sourceValue = sourceTimestepInputField.getValue();
		Object targetValue = targetTimestepInputField.getValue();
		Object couplingPeriodValue = couplingPeriodInputField.getValue();
		
		Object sourceTimeLagValue =  sourceTimeLagInputField.getValue();
		Object targetTimeLagValue =  targetTimeLagInputField.getValue();
		
		timestep.setSource(sourceValue.toString());
		timestep.setTarget(targetValue.toString());
		timestep.setCouplingPeriod(couplingPeriodValue.toString());

		timestep.setCouplingOperation((String) operationList.getSelectedItem());
		
		timestep.setSourceTimeLag(sourceTimeLagValue.toString());
		timestep.setTargetTimeLag(targetTimeLagValue.toString());
	}
	
	@Override
	/**
	 * create panel mainly to set format for input fields 
	 * depending on  {@link TimestepUnit}, if time step unit was set to be string or ISO format, 
	 * formatter is an instance of {@link TimestepFormatter}, otherwise formatter is an instance  
	 * of {@link NumberFormatter}
	 */
	protected JPanel createCenterPanel() {
		String timestepUnitString = null;
		AbstractFormatter formatter = null;

		//MaskFormatter maskformatter = null;
		
		switch (timestepUnit) {
		case ISO_FORMAT:
			
		case STRING:
//			DefaultFormatter defFormatter = new DefaultFormatter();
//			defFormatter.setOverwriteMode(false);
//			formatter = defFormatter;
			timestepFormatter = new TimestepFormatter();
			formatter = timestepFormatter;
			break;
		default:
			formatter = new NumberFormatter();
			((NumberFormatter) formatter).setMinimum(0);
			timestepUnitString = timestepUnit.toString() + "(s)";
			break;
		}
		
		DefaultFormatterFactory fact = new DefaultFormatterFactory(formatter);
		
		// the panel and its layout
		JPanel optionsPanel = new JPanel();
		optionsPanel.setLayout(new BoxLayout(optionsPanel, BoxLayout.PAGE_AXIS));
		
		// source timestep
		JPanel sourceTimestepPanel = new JPanel();
		sourceTimestepPanel.add(new JLabel("Source timestep: "));
		sourceTimestepInputField = new JFormattedTextField(fact);
		sourceTimestepInputField.setColumns(DEFAULT_COLUMN_NUM);
		sourceTimestepInputField.setHorizontalAlignment(JTextField.RIGHT);
		sourceTimestepPanel.add(sourceTimestepInputField);
		sourceTimestepPanel.add(new JLabel(timestepUnitString));

		// target timestep
		JPanel targetTimestepPanel = new JPanel();
		targetTimestepPanel.add(new JLabel("Target timestep: "));
		targetTimestepInputField = new JFormattedTextField(fact);
		targetTimestepInputField.setColumns(DEFAULT_COLUMN_NUM);
		targetTimestepInputField.setHorizontalAlignment(JTextField.RIGHT);
		targetTimestepPanel.add(targetTimestepInputField);
		targetTimestepPanel.add(new JLabel(timestepUnitString));

		// coupling period
		JPanel couplingPeriodPanel = new JPanel();
		couplingPeriodPanel.add(new JLabel("Coupling period: "));
		couplingPeriodInputField = new JFormattedTextField(fact);
		couplingPeriodInputField.setColumns(DEFAULT_COLUMN_NUM);
		couplingPeriodInputField.setHorizontalAlignment(JTextField.RIGHT);
		couplingPeriodPanel.add(couplingPeriodInputField);
		couplingPeriodPanel.add(new JLabel(timestepUnitString));
		
		// operation
		JPanel operationPanel = new JPanel();
		operationPanel.add(new JLabel("Operation: "));
		operationList = new JComboBox<String>(CouplingOperation.toStringArray());
		operationList.setBorder(new EmptyBorder(5, 5, 5, 5));
		operationPanel.add(operationList);

		// source lag
		JPanel sourceTimeLagPanel = new JPanel();
		sourceTimeLagPanel.add(new JLabel("Source Time Lag: "));
		sourceTimeLagInputField = new JFormattedTextField();
		timelagFormatter = new TimestepFormatter();
		//
		// Junpei Xu
		sourceTimeLagInputField.setFormatterFactory(new JFormattedTextField.AbstractFormatterFactory() {
			@Override
			public AbstractFormatter getFormatter(JFormattedTextField tf) {
				return timelagFormatter;
			}
		});
		sourceTimeLagInputField.setColumns(DEFAULT_COLUMN_NUM);
		sourceTimeLagInputField.setHorizontalAlignment(JTextField.RIGHT);
		
		/**
		 * pop up a window "non-negative number" when user set focus in the field
		 */
		sourceTimeLagInputField.addFocusListener(new FocusListener() {
			
			  JWindow jw =new JWindow();
			  
			  //count the times user click source and target time lag 
			  int isJwLabeladded = 0;
			  
		      public void focusGained(FocusEvent e) {
		        displayMessage("non-negative number", e);
		      }

		      public void focusLost(FocusEvent e) {
	               if(jw!=null)jw.setVisible(false);
		      }

		      void displayMessage(String prefix, FocusEvent e) {
		    	  
		    	   if(jw==null){
					  jw =new JWindow();
		    	   }
		    	   else{
	    			  jw.setLayout(new GridLayout(1,1));
					  jw.setLocation(sourceTimeLagInputField.getLocation().x + 500, sourceTimeLagInputField.getLocation().y + 455);
					  jw.setAlwaysOnTop(true);
					  jw.setSize(150, 30);
					  if (isJwLabeladded >= 1){
						  jw.setVisible(true);
					  }
					  else if(isJwLabeladded < 1 ){
		    			  JLabel jwLabel= new JLabel(prefix); 
						  jwLabel.setBackground(sourceTimeLagPanel.getBackground());
		    			  jw.add(jwLabel);
		    			  isJwLabeladded++;
		    			  jw.setVisible(true);
		    		  }
		    	   }
		      }

		    });
		sourceTimeLagPanel.add(sourceTimeLagInputField);
		sourceTimeLagPanel.add(new JLabel(TIMESTEP_UNIT_LABEL));
		
//		sourceTimeLagPanel.addMouseListener(new MouseListener(){
//      	  		   
//            JWindow jw=null;
//
//            @Override
//            public void mouseExited(MouseEvent e) {
//                    // TODO Auto-generated method stub
//                if(jw!=null)jw.setVisible(false);
//            }
//
//			@Override
//			public void mouseClicked(MouseEvent arg0) {
//				// TODO Auto-generated method stub
//				
//			}
//
//			@Override
//			public void mousePressed(MouseEvent e) {
//				// TODO Auto-generated method stub
//				
//			}
//
//			@Override
//			public void mouseReleased(MouseEvent e) {
//				// TODO Auto-generated method stub
//				
//			}
//			
//			@Override
//	        public void mouseEntered(MouseEvent e) {
//	                // TODO Auto-generated method stub
//				   if(jw==null){
//                       jw =new JWindow();
//                       // TODO Auto-generated method stub
//                       JLabel jwLabel= new JLabel("non-negative number"); 
//                       jwLabel.setBackground(sourceTimeLagPanel.getBackground());
//                       jw.setLayout(new GridLayout(1,1));
//                       jw.add(jwLabel);
//                   }
//				     
//				   jw.setLocation(MouseInfo.getPointerInfo().getLocation().x, MouseInfo.getPointerInfo().getLocation().y);
//				   jw.setAlwaysOnTop(true);
//				   jw.setSize(150, 30);
//				   jw.setVisible(true);
//	           }
//	           
//		});
		
		// target lag
		JPanel targetTimeLagPanel = new JPanel();
		targetTimeLagPanel.add(new JLabel("Target Time Lag: "));
		targetTimeLagInputField = new JFormattedTextField();
		
		//Junpei Xu
		targetTimeLagInputField.setFormatterFactory(new JFormattedTextField.AbstractFormatterFactory() {
			@Override
			public AbstractFormatter getFormatter(JFormattedTextField tf) {
				return timelagFormatter;
			}
		});
		targetTimeLagInputField.setColumns(DEFAULT_COLUMN_NUM);
		targetTimeLagInputField.setHorizontalAlignment(JTextField.RIGHT);
		/**
		 * pop up a window "non-negative number" when user set focus in the field
		 */
		targetTimeLagInputField.addFocusListener(new FocusListener() {
			
			  JWindow jw =new JWindow();
			  
			  //count the times user click source and target time lag 
			  int isJwLabeladded = 0;
			  
		      public void focusGained(FocusEvent e) {
		        displayMessage("non-negative number", e);
		      }

		      public void focusLost(FocusEvent e) {
	               if(jw!=null)jw.setVisible(false);
		      }

		      void displayMessage(String prefix, FocusEvent e) {
		    	  
		    	   if(jw==null){
					  jw =new JWindow();
		    	   }
		    	   else{
	    			  jw.setLayout(new GridLayout(1,1));
					  jw.setLocation(targetTimeLagInputField.getLocation().x + 500, targetTimeLagInputField.getLocation().y + 520);
					  jw.setAlwaysOnTop(true);
					  jw.setSize(150, 30);
					  if (isJwLabeladded >= 1){
						  jw.setVisible(true);
					  }
					  else if(isJwLabeladded < 1 ){
		    			  JLabel jwLabel= new JLabel(prefix); 
						  jwLabel.setBackground(sourceTimeLagPanel.getBackground());
		    			  jw.add(jwLabel);
		    			  isJwLabeladded++;
		    			  jw.setVisible(true);
		    		  }
		    	   }
		      }

		    });
		targetTimeLagPanel.add(targetTimeLagInputField);
		targetTimeLagPanel.add(new JLabel(TIMESTEP_UNIT_LABEL));
		
		optionsPanel.add(sourceTimestepPanel);
		optionsPanel.add(targetTimestepPanel);
		optionsPanel.add(couplingPeriodPanel);
		optionsPanel.add(operationPanel);
		optionsPanel.add(sourceTimeLagPanel);
		optionsPanel.add(targetTimeLagPanel);
		
		loadParameters();
		
		return optionsPanel;
	}
	
	@Override
	public void loadParameters() {
		this.timestep = transientCouple.getTimestep();
		
		switch (timestepUnit) {
		case ISO_FORMAT:
		case STRING:
			sourceTimestepInputField.setValue(timestep.getSource());
			targetTimestepInputField.setValue(timestep.getTarget());
			couplingPeriodInputField.setValue(timestep.getCouplingPeriod());
			//sourceTimeLagInputField.setValue(timestep.getSourceTimeLag());
			//targetTimeLagInputField.setValue(timestep.getTargetTimeLag());
			break;
		default:
			sourceTimestepInputField.setValue(new Integer(timestep.getSource()));
			targetTimestepInputField.setValue(new Integer(timestep.getTarget()));
			couplingPeriodInputField.setValue(new Integer(timestep.getCouplingPeriod()));
			//sourceTimeLagInputField.setValue(new Integer(timestep.getSourceTimeLag()));
			//targetTimeLagInputField.setValue(new Integer(timestep.getTargetTimeLag()));
			break;
		}

		operationList.setSelectedItem(timestep.getCouplingOperation().toString());

		//setTimeLagInputField(sourceTimeLagInputField);
		setSourceTimeLagInputField(sourceTimeLagInputField);
		setTargetTimeLagInputField(targetTimeLagInputField);

	}

	///////by Junpei Xu
	private void setSourceTimeLagInputField(JFormattedTextField timeLagField){
		
//		if(timestep.getSourceTimeLag().isTimelagTypeString())
			timeLagField.setValue(timestep.getSourceTimeLag());
//		else
//			timeLagField.setText(new Integer(timestep.getSourceTimeLag().getTimeLagValue()).toString());
	}
	
	private void setTargetTimeLagInputField(JFormattedTextField timeLagField){
		
//		if(timestep.getTargetTimeLag().isTimelagTypeString())
			timeLagField.setValue(timestep.getTargetTimeLag());
//		else
//			timeLagField.setText(new Integer(timestep.getTargetTimeLag().getTimeLagValue()).toString());
	}
}
