package de.mpg.mpimet.yacgui.utils;

import java.util.ArrayList;
import java.util.List;

import de.mpg.mpimet.yacgui.xml.Calendar;
import de.mpg.mpimet.yacgui.xml.CouplingOperation;
import de.mpg.mpimet.yacgui.xml.Dates;
import de.mpg.mpimet.yacgui.xml.Dom;
import de.mpg.mpimet.yacgui.xml.TimestepUnit;
import de.mpg.mpimet.yacgui.xml.interpolation.Interpolation;
import de.mpg.mpimet.yacgui.xml.interpolation.InterpolationFactory;
import de.mpg.mpimet.yacgui.xml.interpolation.InterpolationMethod;

public class Default {

	public static final String DATE_FORMAT = "Gyyyy-MM-dd'T'HH:mm:ss.SSS";
	public static final String CREATED_DATE_FORMAT = "dd-MM-yyyy HH:mm";
	
	/**
	 * Start date for coupling, should be written according {@link Default#DATE_FORMAT}
	 */
	public static final String START_DATE = "1800-01-01T00:00:00.000";
	/**
	 * End date for coupling, should be written according {@link Default#DATE_FORMAT}
	 */
	public static final String END_DATE = "2100-01-01T00:00:00.000";
	
	public static final Calendar CALENDAR = Calendar.PROLEPTIC_GREGORIAN;
	public static final Dates DATES = new Dates(START_DATE, END_DATE, CALENDAR);

	public static final TimestepUnit TIMESTEP_UNIT = TimestepUnit.SECOND;

	public static final String TIMESTEP_SOURCE = "10";
	public static final String TIMESTEP_TARGET = "10";
	public static final String COUPLING_PERIOD = "60";
	public static final CouplingOperation COUPLING_OPERATION = CouplingOperation.ACCUMULATE;
	public static final String SOURCE_TIMELAG = "0";
	public static final String TARGET_TIMELAG = "0";

	public static final Double MASK_VALUE = null;

	public static final boolean AT_SOURCE_BEFORE_INTERPOLATION = false;
	public static final boolean AT_SOURCE_AFTER_INTERPOLATION = false;
	public static final boolean AT_TARGET = false;

	public static final boolean ENFORCE_WRITE_RESTART = false;

	public static final String INTERPOLATION_ATTR_MASK_HANDLING = "none";
	public static final String INTERPOLATION_ATTR_N = "1";
	public static final String INTERPOLATION_ATTR_WEIGHTED = Dom.Value.DISTANCE_WEIGHTED;
	
	public static final String INTERPOLATION_ATTR_ENFORCED_CONSERVATION = "false";
	public static final String INTERPOLATION_ATTR_PARTIAL_COVERAGE = "false";
	public static final String INTERPOLATION_ATTR_NORMALISATION = Dom.Value.DESTAREA;
	
	public static final String INTERPOLATION_ATTR_ALLOW_EXTRAPOLATION = "false";
	public static final String INTERPOLATION_ATTR_EXTEND_SOURCE_PATCH = "1";
	public static final String INTERPOLATION_ATTR_POLYNOMIAL_ORDER = "1";
	public static final String INTERPOLATION_ATTR_GAUSS_ORDER = "1";
	public static final String INTERPOLATION_ATTR_ALLOW_CORNER_WEIGHT = "false";
	public static final String INTERPOLATION_ATTR_CORNER_WEIGHT = "1";

	
	public static final String INTERPOLATION_ATTR_USER_VALUE = "0.0";
	
	public static final String INTERPOLATION_ATTR_YAC_LOCATION = Dom.Value.CELL;

	/**
	 * Creates an implementation of {@link Interpolation} according to 
	 * given method, and uses default values as parameters.
	 * 
	 * @param method	String representation of {@link InterpolationMethod}
	 * @return Interpolation with default values as parameters.
	 */
	public static Interpolation defaultInterpolation(String method) {
		if (!InterpolationMethod.isValidName(method)) {
			throw new IllegalArgumentException("'" + method + "' is not a valid method");
		}
		InterpolationMethod interpolationMethod = InterpolationMethod.fromName(method);

		List<String[]> attributes = null;

		switch (interpolationMethod) {
		case AVERAGE:
			attributes = new ArrayList<String[]>();
			attributes.add(new String[] {Dom.Attribute.PARTIAL_COVERAGE, INTERPOLATION_ATTR_PARTIAL_COVERAGE});
			attributes.add(new String[] {Dom.Attribute.WEIGHTED, INTERPOLATION_ATTR_WEIGHTED});
			break;
		case N_NEAREST_NEIGHBOR:
			attributes = new ArrayList<String[]>();
			attributes.add(new String[] {Dom.Attribute.N, INTERPOLATION_ATTR_N});
			attributes.add(new String[] {Dom.Attribute.WEIGHTED, INTERPOLATION_ATTR_WEIGHTED});
			break;
		case CONSERVATIVE:
			attributes = new ArrayList<String[]>();
			attributes.add(new String[] {Dom.Attribute.ENFORCED_CONSERVATION, INTERPOLATION_ATTR_ENFORCED_CONSERVATION});
			attributes.add(new String[] {Dom.Attribute.NORMALISATION, INTERPOLATION_ATTR_NORMALISATION});
			attributes.add(new String[] {Dom.Attribute.PARTIAL_COVERAGE, INTERPOLATION_ATTR_PARTIAL_COVERAGE});
			break;
		case PATCH_RECOVERY:
			attributes = new ArrayList<String[]>();
			attributes.add(new String[] {Dom.Attribute.ALLOW_EXTRAPOLATION, INTERPOLATION_ATTR_ALLOW_EXTRAPOLATION});
			attributes.add(new String[] {Dom.Attribute.EXTEND_SOURCE_PATCH, INTERPOLATION_ATTR_EXTEND_SOURCE_PATCH});
			attributes.add(new String[] {Dom.Attribute.GAUSS_ORDER, INTERPOLATION_ATTR_GAUSS_ORDER});
			attributes.add(new String[] {Dom.Attribute.POLYNOMIAL_ORDER, INTERPOLATION_ATTR_POLYNOMIAL_ORDER});
			break;
		case FIXED_VALUE:
			attributes = new ArrayList<String[]>();
			attributes.add(new String[] { Dom.Attribute.USER_VALUE, INTERPOLATION_ATTR_USER_VALUE });
			break;
		case USER_FILE:
			attributes = new ArrayList<String[]>();
			attributes.add(new String[] { Dom.Attribute.FILENAME, null });
			attributes.add(new String[] { Dom.Attribute.YAC_LOCATION, INTERPOLATION_ATTR_YAC_LOCATION });
			break;
		case SOURCE2TARGET_MAP:
			attributes = new ArrayList<String[]>();
			break;
		case SMOOTHED_PATCH_RECOVERY:
			attributes = new ArrayList<String[]>();
			attributes.add(new String[] {Dom.Attribute.ALLOW_CORNER_WEIGHT, INTERPOLATION_ATTR_ALLOW_CORNER_WEIGHT});
			attributes.add(new String[] {Dom.Attribute.ALLOW_EXTRAPOLATION, INTERPOLATION_ATTR_ALLOW_EXTRAPOLATION});
			attributes.add(new String[] {Dom.Attribute.CORNER_WEIGHT, INTERPOLATION_ATTR_CORNER_WEIGHT});
			attributes.add(new String[] {Dom.Attribute.EXTEND_SOURCE_PATCH, INTERPOLATION_ATTR_EXTEND_SOURCE_PATCH});
			attributes.add(new String[] {Dom.Attribute.GAUSS_ORDER, INTERPOLATION_ATTR_GAUSS_ORDER});
			attributes.add(new String[] {Dom.Attribute.POLYNOMIAL_ORDER, INTERPOLATION_ATTR_POLYNOMIAL_ORDER});
			break;
		default:
			throw new IllegalArgumentException("'" + method + "' is not a valid method");
		}

		return InterpolationFactory.createInterpolation(method, attributes);
	}

}
