package de.mpg.mpimet.yacgui.utils;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;

/**
 * Helper logger class for feeling like a master of logs. :)
 * 
 * @author Maxim Yastremsky
 */
public class Logger {
	
	private static final String DEFAULT_LOG_FILENAME = "CouplingGui_log.txt";
	public static final String DATE_FORMAT = "yyyy-MM-dd_HH:mm:ss";
	
	public enum LogLevel {
		DEBUG,
		INFO,
		WARN,
		ERROR,
		OFF;
	}
	
	public enum LogOutput {
		SYSTEM_OUT,
		FILE;
	}
	
	public static LogLevel currentLevel = LogLevel.DEBUG;
	public static LogOutput currentOutput = LogOutput.SYSTEM_OUT;
	
	private static BufferedWriter writer;
	private static File logFile;
	
	public static boolean logTime = true;
	public static boolean logLogLevel = true;
	
	public static void logDebug(String logInfo) {
		if (currentLevel.ordinal() <= LogLevel.DEBUG.ordinal()) {
			log(LogLevel.DEBUG, logInfo);
		}
	}
	
	private static void log(LogLevel level, String logInfo) {
		log(level, logInfo, null);
	}
	
	private static void log(LogLevel level, String logInfo, Exception exp) {
		
		if (logInfo == null || logInfo.equals("")) {
			return;
		}
		
		StringBuilder builder = new StringBuilder();
		if (logTime) {
			builder.append(getTimeLog() + "; ");
		}
		
		if (logLogLevel) {
			builder.append(level.toString() + ": ");
		}
		
		builder.append(logInfo);
		
		switch (currentOutput) {
		case SYSTEM_OUT:
			System.out.println(builder.toString());
			
			if (exp != null) {
				exp.printStackTrace(System.out);
			}
			break;
		case FILE:
			try {
				if (logFile == null || !logFile.exists()) { 
					logFile = new File(DEFAULT_LOG_FILENAME);
					
					if (!logFile.exists()) {
						System.out.println("creating controller log file");
					}
					writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(logFile, true), "utf-8"));
				}
				
				writer.write(builder.toString());
				
				if (exp != null) {
					writer.write(Arrays.toString(exp.getStackTrace()));
				}
				writer.newLine();
				writer.flush();
			} catch (Exception e) {
				e.printStackTrace();
			}
			break;
		default:
			break;
		}
		
	}

	public static String getTimeLog() {
		return new SimpleDateFormat(DATE_FORMAT).format(new Date());
	}

	public static void logInfo(String logInfo) {
		if (currentLevel.ordinal() <= LogLevel.INFO.ordinal()) {
			log(LogLevel.INFO, logInfo);
		}
	}
	
	public static void logWarn(String logInfo) {
		if (currentLevel.ordinal() <= LogLevel.WARN.ordinal()) {
			log(LogLevel.WARN, logInfo);
		}
	}
	
	public static void logError(String logInfo) {
		if (currentLevel.ordinal() <= LogLevel.ERROR.ordinal()) {
			log(LogLevel.ERROR, logInfo);
		}
	}
	
	public static void logError(String logInfo, Exception exc) {
		if (currentLevel.ordinal() <= LogLevel.ERROR.ordinal()) {
			log(LogLevel.ERROR, logInfo, exc);
		}
	}
}
