/**
 * 
 */
package de.mpg.mpimet.yacgui.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.ImageIcon;

import de.mpg.mpimet.yacgui.ui.HintPanel;
import de.mpg.mpimet.yacgui.xml.Component;
import de.mpg.mpimet.yacgui.xml.Couple;
import de.mpg.mpimet.yacgui.xml.Grid;
import de.mpg.mpimet.yacgui.xml.Transient;
import de.mpg.mpimet.yacgui.xml.TransientGridRef;

/**
 * Utils class.
 * 
 * @author Maxim Yastremsky
 */
public class Utils {

	/**
	 * Returns string id (form of id1-id2) for possible couple between components. Keeps track
	 * of ascending order of components' ids in the final string.
	 * 
	 * @param comp1Id
	 * @param comp2Id
	 * 
	 * @return String id of from 
	 */
	public static String getCoupleId(int comp1Id, int comp2Id) {
		if (comp1Id < comp2Id) {
			return String.valueOf(comp1Id) + "-" + String.valueOf(comp2Id);
		} else {
			return String.valueOf(comp2Id) + "-" + String.valueOf(comp1Id);
		}
	}
	
	/**
	 * Parses the couple's Id string in order to get the second component Id.
	 * @see Utils#getCoupleId(int, int)
	 * 
	 * @param coupleId couple Id to parse
	 * @param firstComponentId component id to compare
	 * 
	 * @return int id of the second component
	 */
	public static int parseSecondComponentId(String coupleId, int firstComponentId) {
		int hyphenPos = coupleId.indexOf("-");
		
		int lowestId = Integer.parseInt(coupleId.substring(0, hyphenPos));
		int biggestId = Integer.parseInt(coupleId.substring(hyphenPos + 1));
		
		return lowestId != firstComponentId ? lowestId :  biggestId;
	}

	/**
	 * Gets the key String value that will be the first, alphabetically sorted.
	 * 
	 * @param couples
	 *            unsorted map with String keys
	 * @return first key String, alphabetically sorted
	 */
	public static String getAlphabeticallyFirst(Map<String, Couple> couples) {
		List<String> keys = new ArrayList<String>(couples.keySet());
		Collections.sort(keys);
		return keys.get(0);
	}

	/**
	 * Gets a integer key value for a component at position in sorted list.
	 * 
	 * @param components
	 *            unsorted map with Integer keys
	 * @param position
	 *            index of the element that should be returned
	 * @return key int at position, numerically sorted
	 */
	public static int getKeyAtSortedPosition(Map<Integer, Component> components, int position) {
		List<Integer> keys = new ArrayList<Integer>(components.keySet());
		Collections.sort(keys);
		return keys.get(position);
	}

	/**
	 * Returns a new sorted (by transientGridRef Id) copy of passed TransientGridRefs.
	 * 
	 * @param transientGridRefs
	 * @return new list of given TransientGridRef, sorted by their transientRef
	 */
	public static List<TransientGridRef> sortByTransientGridRefId(List<TransientGridRef> transientGridRefs) {
		List<TransientGridRef> result = new ArrayList<TransientGridRef>(transientGridRefs);
		Collections.sort(result);
		return result;
	}
	
	public static List<Component> sortByComponentId(List<Component> components) {
		List<Component> result = new ArrayList<Component>(components);
		Collections.sort(result);
		return result;
	}
	
	/**
	 * Used for tests
	 * 
	 * @param transients
	 * @return
	 */
	public static List<String> getTransientStandardNames(Map<Integer, Transient> transients) {
		List<String> transientStandardNames = new ArrayList<String>();
		for (Transient t : transients.values()) {
			transientStandardNames.add(t.getTransientStandardName());
		}
		return transientStandardNames;
	}
	
	/**
	 * Used for tests
	 * 
	 * @param grids
	 * @return
	 */
	public static List<String> getAliasNames(Map<Integer, Grid> grids) {
		List<String> aliasNames = new ArrayList<String>();
		for (Grid g : grids.values()) {
			aliasNames.add(g.getAliasName());
		}
		return aliasNames;
	}
	
	
	/**
	 * Used for tests
	 * 
	 * @param transients
	 * @param transientStandardNames
	 * @return
	 */
	public static boolean transientsContainAll(Map<Integer, Transient> transients, 
			Collection<String> transientStandardNames) {
		Set<String> names = new HashSet<String>(transientStandardNames);
		for (Transient t : transients.values()) {
			names.remove(t.getTransientStandardName());
		}
		return names.isEmpty();
	}
	
	
	/** Used for tests
	 * 
	 * @param grids
	 * @param aliasNames
	 * @return
	 */
	public static boolean gridsContainAll(Map<Integer, Grid> grids, Collection<String> aliasNames) {
		Set<String> names = new HashSet<String>(aliasNames);
		for (Grid g : grids.values()) {
			names.remove(g.getAliasName());
		}
		return names.isEmpty();
	}
	
    /**
     * @param path
     * @return ImageIcon, or null if the path was invalid.
     */
    public static ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = HintPanel.class.getResource(path);
        if (imgURL != null) {
            return new ImageIcon(imgURL);
        }
		System.err.println("Couldn't find file: " + path);
		return null;
    }
}