package de.mpg.mpimet.yacgui.utils;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * ErrorHandler implementation for XmlDomHandler.
 * 
 * Methods error(), fatalError(), warning() shouldn't normally be called by user.
 * 
 * Additional functionality is used for storing all errors messages and returning them later
 * 
 * @author Maxim Yastremsky
 */
public class XmlDomErrorHandler implements ErrorHandler {
	
	private static StringBuilder buffer = new StringBuilder();
	
	@Override
	public void error(SAXParseException exception) throws SAXException {
		storeMessage(exception.getMessage());
//		Logger.logInfo(exception.getMessage());
	}

	private String storeMessage(String message) {
		buffer.append(message).append("\n");
		return message;
	}

	@Override
	public void fatalError(SAXParseException exception) throws SAXException {
		storeMessage(exception.getMessage());
//		Logger.logInfo(exception.getMessage());
	}

	@Override
	public void warning(SAXParseException exception) throws SAXException {
		storeMessage(exception.getMessage());
//		Logger.logInfo(exception.getMessage());
	}
	
	public String getStoredErrorMessages() {
		String mes = buffer.toString();
		resetBuffer();
		return mes;
	}

	private void resetBuffer() {
		buffer.setLength(0); //deleting whole buffer's content
	}
} 