/**
 * 
 */
package de.mpg.mpimet.yacgui.xml;

import de.mpg.mpimet.yacgui.utils.Default;

/**
 * Class representing &lt;debug_mode&gt; element.
 * 
 * @author Teresa Holfeld
 */
public class DebugMode {

	private boolean atSourceBeforeInterpolation;
	private boolean atSourceAfterInterpolation;
	private boolean atTarget;

	/**
	 * @param atSourceBeforeInterpolation
	 * @param atSourceAfterInterpolation
	 * @param atTarget
	 */
	public DebugMode(boolean atSourceBeforeInterpolation,
			boolean atSourceAfterInterpolation, boolean atTarget) {
		this.atSourceBeforeInterpolation = atSourceBeforeInterpolation;
		this.atSourceAfterInterpolation = atSourceAfterInterpolation;
		this.atTarget = atTarget;
	}
	
	/**
	 * Returns default DebugMode with the following values:<br>
	 * atSourceBeforeInterpolation: <b>false</b><br>
	 * atSourceAfterInterpolation: <b>false</b><br>
	 * atTarget: <b>false</b><br>
	 * 
	 * @return DebugMode with default values.
	 */
	public static DebugMode getDefault() {
		return new DebugMode(Default.AT_SOURCE_BEFORE_INTERPOLATION, 
				Default.AT_SOURCE_AFTER_INTERPOLATION, Default.AT_TARGET);
	}

	/**
	 * @return the atSourceBeforeInterpolation
	 */
	public boolean isAtSourceBeforeInterpolation() {
		return atSourceBeforeInterpolation;
	}

	/**
	 * @return the atSourceAfterInterpolation
	 */
	public boolean isAtSourceAfterInterpolation() {
		return atSourceAfterInterpolation;
	}

	/**
	 * @return the atTarget
	 */
	public boolean isAtTarget() {
		return atTarget;
	}
	
	public void setAtSourceBeforeInterpolation(
			boolean atSourceBeforeInterpolation) {
		this.atSourceBeforeInterpolation = atSourceBeforeInterpolation;
	}
	
	public void setAtSourceAfterInterpolation(
			boolean atSourceAfterInterpolation) {
		this.atSourceAfterInterpolation = atSourceAfterInterpolation;
	}
	
	public void setAtTarget(boolean atTarget) {
		this.atTarget = atTarget;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (atSourceAfterInterpolation ? 1231 : 1237);
		result = prime * result + (atSourceBeforeInterpolation ? 1231 : 1237);
		result = prime * result + (atTarget ? 1231 : 1237);
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof DebugMode)) {
			return false;
		}
		DebugMode other = (DebugMode) obj;
		if (atSourceAfterInterpolation != other.atSourceAfterInterpolation) {
			return false;
		}
		if (atSourceBeforeInterpolation != other.atSourceBeforeInterpolation) {
			return false;
		}
		if (atTarget != other.atTarget) {
			return false;
		}
		return true;
	}
	
	@Override
	public String toString() {
		return "[beforeIntrp=" + atSourceBeforeInterpolation + ", afterInterp="
				+ atSourceAfterInterpolation + ", atTarget=" + atTarget + "]";
	}
	
}
