package de.mpg.mpimet.yacgui.xml;

/**
 * Java representation of {@code <transient_grid_ref>} xml element.
 * 
 * @author Maxim Yastremsky, Teresa Holfeld
 */
public class TransientGridRef implements Comparable<TransientGridRef> {
	
	/** ID of this TransientGridRef */
	private int id;
	
	/** ID of references Transient */
	private int transientRef;
	
	/** ID of references Grid */
	private int gridRef;
	
	/** Collection size */
	private int collectionSize;
	
	/**
	 * Creates a TransientGridRef object with id, transientRef and usersGrid.
	 * 
	 * @param id			ID of this TransientGridRef
	 * @param transientRef	ID of referenced Transient 
	 * @param gridRef		ID of referenced Grid
	 */
	public TransientGridRef(int id, int transientRef, int gridRef, int collectionSize) {
		this.id = id;
		this.transientRef = transientRef;
		this.gridRef = gridRef;
		this.collectionSize = collectionSize;
	}
	
	/**
	 * @return id: ID of this TransientGridRef.
	 */
	public int getId() {
		return id;
	}

	/**
	 * @return transientRef: ID of referenced Transient.
	 */
	public int getTransientRef() {
		return transientRef;
	}
	
	public void setTransientRef(int transientRef) {
		this.transientRef = transientRef;
	}

	/**
	 * @return gridRef: ID of referenced Grid.
	 */
	public int getGridRef() {
		return gridRef;
	}
	
	public void setGridRef(int gridRef) {
		this.gridRef = gridRef;
	}
	
	/**
	 * @return collectionSize
	 */
	public int getCollectionSize() {
		return collectionSize;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + gridRef;
		result = prime * result + id;
		result = prime * result + transientRef;
		result = prime * result + collectionSize;
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof TransientGridRef)) {
			return false;
		}
		TransientGridRef other = (TransientGridRef) obj;
		if (gridRef != other.gridRef) {
			return false;
		}
		if (id != other.id) {
			return false;
		}
		if (transientRef != other.transientRef) {
			return false;
		}
		if (collectionSize != other.collectionSize) {
			return false;
		}
		return true;
	}
	
	@Override
	public int compareTo(TransientGridRef o) {
		return id - o.id;
	}
}
