package de.mpg.mpimet.yacgui.xml.interpolation;

import de.mpg.mpimet.yacgui.xml.Dom;
import de.mpg.mpimet.yacgui.xml.interpolation.NNearestNeighborInterpolation.Weighted;

/**
 * Definite interpolation method average
 * 
 * @author Maxim Yastremsky
 */
public class AverageInterpolation extends Interpolation {

	private boolean partialCoverage;
	private Weighted weighted;
	
	public AverageInterpolation(boolean partialCoverage, Weighted weighted) {
		attributesNumber = 2;
		method = InterpolationMethod.AVERAGE;
		this.partialCoverage = partialCoverage;
		this.weighted = weighted;
	}
	
	/**
	 * 
	 * @return true if partial coverage is active
	 */
	public boolean getPartialCoverage() {
		return partialCoverage;
	}
	
	/**
	 * 
	 * @return object of {@link Weighted} indicating method of weighted
	 */
	public Weighted getWeighted() {
		return weighted;
	}
	
	/**
	 * get attribute name by the order of index
	 * @param index the order of attributes on the panel
	 * @return  the name of attribute
	 * 				if 0 return PARTIAL_COVERAGE, if 1 return WEIGHTED
	 */
	public String getAttributeName(int index) {
		if (index == 0) {
			return Dom.Attribute.PARTIAL_COVERAGE;
		} else if (index == 1) {
			return Dom.Attribute.WEIGHTED;
		}
		return null;
	}
	
	/**
	 * get attribute value by the order of index
	 * @param index the order of attributes on the panel
	 * @return the value of attribute as String
	 * 				if 0 return partialCoverage, if 1 return weighted
	 */
	public String getAttributeValue(int index) {
		if (index == 0) {
			return String.valueOf(partialCoverage);
		} else if (index == 1) {
			return weighted.toString();
		}
		return null;
	}
}
