package de.mpg.mpimet.yacgui.xml.interpolation;

import de.mpg.mpimet.yacgui.xml.Dom;

/**
 * Definite interpolation method conservative
 * 
 * @author Maxim Yastremsky
 */
public class ConservativeInterpolation extends Interpolation {
	
	/**
	 * the method of normalisation
	 * {@code FRACAREA, DESTAREA}
	 */
	public enum Normalisation {
		FRACAREA,
		DESTAREA;
		
		public static boolean isValidString(String s) {
			return (s.equals(Dom.Value.FRACAREA) || 
					s.equals(Dom.Value.DESTAREA));
		}
		public static Normalisation objectFromString(String s) {
			if (s.equals(Dom.Value.FRACAREA)) {
				return FRACAREA;
			} else if (s.equals(Dom.Value.DESTAREA)) {
				return DESTAREA;
			}
			return null; 
		}
		
		@Override
		public String toString() {
			if (this == FRACAREA) {
				return Dom.Value.FRACAREA;
			} else if (this == DESTAREA) {
				return Dom.Value.DESTAREA;
			}
			return null;
		}
	}
	
	private boolean enforcedConservation;
	private boolean partialCoverage;
	private Normalisation normalisation;
	
	public ConservativeInterpolation(boolean enforcedConservation, boolean partialCoverage,
			Normalisation normalisation) {
		
		attributesNumber = 3;
		method = InterpolationMethod.CONSERVATIVE;
		this.enforcedConservation = enforcedConservation;
		this.partialCoverage = partialCoverage;
		this.normalisation = normalisation;
	}
	
	/**
	 * 
	 * @return true if enforced conservation is active
	 */
	public boolean getEnforcedConservation() {
		return enforcedConservation;
	}
	
	/**
	 * 
	 * @return true if partial coverage is active
	 */
	public boolean getPartialCoverage() {
		return partialCoverage;
	}
	
	/**
	 * 
	 * @return object of {@link Normalisation} indicating the method of normalisation
	 */
	public Normalisation getNormalisation() {
		return normalisation;
	}
	
	/**
	 * get attribute name by the order of index
	 * @param index the order of attributes on the panel
	 * @return  the name of attribute
	 * 				if 0 return ENFORCED_CONSERVATION, if 1 return PARTIAL_COVERAGE, if 2 return NORMALISATION
	 */
	public String getAttributeName(int index) {
		if (index == 0) {
			return Dom.Attribute.ENFORCED_CONSERVATION;
		} else if (index == 1) {
			return Dom.Attribute.PARTIAL_COVERAGE;
		} else if (index == 2) {
			return Dom.Attribute.NORMALISATION;
		}
		return null;
	}
	
	/**
	 * get attribute value by the order of index
	 * @param index the order of attributes on the panel
	 * @return the value of attribute as String
	 * 				if 0 return enforcedConservation, if 1 return partialCoverage, if 2 return normalisation
	 */
	public String getAttributeValue(int index) {
		if (index == 0) {
			return String.valueOf(enforcedConservation);
		} else if (index == 1) {
			return String.valueOf(partialCoverage);
		} else if (index == 2) {
			return normalisation.toString();
		}
		return null;
	}

}
