package de.mpg.mpimet.yacgui.xml.interpolation;

import de.mpg.mpimet.yacgui.xml.Dom;

/**
 * Definite interpolation method n nearest neighbor
 * 
 * @author Maxim Yastremsky
 */
public class NNearestNeighborInterpolation extends Interpolation {
	
	/**
	 * the method of weighted
	 * {@code DISTANCE_WEIGHTED, ARITHMETIC_AVERAGE}
	 */
	public enum Weighted {
		DISTANCE_WEIGHTED,
		ARITHMETIC_AVERAGE;
		
		public static boolean isValidString(String s) {
			return (s.equals(Dom.Value.DISTANCE_WEIGHTED) || 
					s.equals(Dom.Value.ARITHMETIC_AVERAGE));
		}
		public static Weighted objectFromString(String s) {
			if (s.equals(Dom.Value.DISTANCE_WEIGHTED)) {
				return DISTANCE_WEIGHTED;
			} else if (s.equals(Dom.Value.ARITHMETIC_AVERAGE)) {
				return ARITHMETIC_AVERAGE;
			}
			return null; 
		}
		
		@Override
		public String toString() {
			if (this == DISTANCE_WEIGHTED) {
				return Dom.Value.DISTANCE_WEIGHTED;
			} else if (this == ARITHMETIC_AVERAGE) {
				return Dom.Value.ARITHMETIC_AVERAGE;
			}
			return null;
		}
	}

	private int n;
	private Weighted weighted;
	
	public NNearestNeighborInterpolation(int n, Weighted weighted) throws IllegalArgumentException {
		if (n < 1) {
			throw new IllegalArgumentException("n must be bigger than 0");
		}
		
		attributesNumber = 2;
		method = InterpolationMethod.N_NEAREST_NEIGHBOR;
		this.n = n;
		this.weighted = weighted;
	}
	
	/**
	 * 
	 * @return number of source point values
	 */
	public int getN() {
		return n;
	}
	
	/**
	 * 
	 * @return object of {@link Weighted} indicating method of weighted
	 */
	public Weighted getWeighted() {
		return weighted;
	}
	
	/**
	 * get attribute name by the order of index
	 * @param index the order of attributes on the panel
	 * @return  the name of attribute
	 * 				if 0 return N, if 1 return WEIGHTED
	 */
	public String getAttributeName(int index) {
		if (index == 0) {
			return Dom.Attribute.N;
		} else if (index == 1) {
			return Dom.Attribute.WEIGHTED;
		}
		return null;
	}
	
	/**
	 * get attribute value by the order of index
	 * @param index the order of attributes on the panel
	 * @return the value of attribute as String
	 *				if 0 return n, if 1 return weighted
	 */
	public String getAttributeValue(int index) {
		if (index == 0) {
			return String.valueOf(n);
		} else if (index == 1) {
			return weighted.toString();
		}
		return null;
	}
}
