package de.mpg.mpimet.yacgui.xml.interpolation;

import de.mpg.mpimet.yacgui.xml.Dom;

/**
 * Definite interpolation method patch recovery
 * 
 * @author Maxim Yastremsky
 */
public class PatchRecoveryInterpolation extends Interpolation {

	private boolean allowExtrapolation;
	private int extendSourcePatch;
	private int polynomialOrder;
	private int gaussOrder;
	
	public PatchRecoveryInterpolation(boolean allowExtrapolation, int extendSourcePatch,
			int polynomialOrder, int gaussOrder) throws IllegalArgumentException {
		
		if (extendSourcePatch < 1) {
			throw new IllegalArgumentException("ExtendSourcePatch must be bigger than 0");
		}
		if (polynomialOrder < 1) {
			throw new IllegalArgumentException("PolynomialOrder must be bigger than 0");
		}
		if (gaussOrder < 1) {
			throw new IllegalArgumentException("GaussOrder must be bigger than 0");
		}
		
		attributesNumber = 4;
		method = InterpolationMethod.PATCH_RECOVERY;
		this.allowExtrapolation = allowExtrapolation;
		this.extendSourcePatch = extendSourcePatch;
		this.polynomialOrder = polynomialOrder;
		this.gaussOrder = gaussOrder;
	}
	
	/**
	 * 
	 * @return true if extrapolation is allowed in patch recovery  
	 */
	public boolean isAllowExtrapolation() {
		return allowExtrapolation;
	}

	/**
	 * 
	 * @return integer value of extend source patch
	 */
	public int getExtendSourcePatch() {
		return extendSourcePatch;
	}
	
	/**
	 * 
	 * @return integer value of polynomial order
	 */
	public int getPolynomialOrder() {
		return polynomialOrder;
	}
	

	/**
	 * 
	 * @return integer value of gauss order
	 */
	public int getGaussOrder() {
		return gaussOrder;
	}
	
	/**
	 * get attribute name by the order of index
	 * @param index the order of attributes on the panel
	 * @return  the name of attribute
	 * 				if index = 0 return ALLOW_EXTRAPOLATION, if 1 return EXTEND_SOURCE_PATCH, if 2 return POLYNOMIAL_ORDER, 
	 * if 3 return GAUSS_ORDER
	 */
	public String getAttributeName(int index) {
		if (index == 0) {
			return Dom.Attribute.ALLOW_EXTRAPOLATION;
		} else if (index == 1) {
			return Dom.Attribute.EXTEND_SOURCE_PATCH;
		} else if (index == 2) {
			return Dom.Attribute.POLYNOMIAL_ORDER;
		} else if (index == 3) {
			return Dom.Attribute.GAUSS_ORDER;
		}
		return null;
	}
	
	/**
	 * get attribute value by the order of index
	 * @param index the order of attributes on the panel
	 * @return the value of attribute as String
	 *				if 0 return allowExtrapolation, if 1 return extendSourcePatch, if 2 return polynomialOrder, 
	 * if 3 return gaussOrder
	 */
	public String getAttributeValue(int index) {
		if (index == 0) {
			return String.valueOf(allowExtrapolation);
		} else if (index == 1) {
			return String.valueOf(extendSourcePatch);
		} else if (index == 2) {
			return String.valueOf(polynomialOrder);
		} else if (index == 3) {
			return String.valueOf(gaussOrder);
		}
		return null;
	}
}
