package de.mpg.mpimet.yacgui.xml.interpolation;

import de.mpg.mpimet.yacgui.xml.Dom;

/**
 * Definite interpolation method user file
 * 
 * @author Maxim Yastremsky
 */
public class UserFileInterpolation extends Interpolation {
	
	/**
	 * set the location of the data
	 * cell, edge, corner
	 */
	public enum YacLocation {
		CELL,
		EDGE,
		CORNER;
		
		public static boolean isValidString(String s) {
			return (s.equals(Dom.Value.CELL) || 
					s.equals(Dom.Value.EDGE) ||
					s.equals(Dom.Value.CORNER));
		}
		public static YacLocation objectFromString(String s) {
			if (s.equals(Dom.Value.CELL)) {
				return CELL;
			} else if (s.equals(Dom.Value.EDGE)) {
				return EDGE;
			} else if (s.equals(Dom.Value.CORNER)) {
				return CORNER;
			}
			return null; 
		}
		
		@Override
		public String toString() {
			if (this == CELL) {
				return Dom.Value.CELL;
			} else if (this == EDGE) {
				return Dom.Value.EDGE;
			} else if (this == CORNER) {
				return Dom.Value.CORNER;
			}
			return null;
		}
	}
	
	private String filename;
	private YacLocation yacLocation;
	
	public UserFileInterpolation(String fileName, YacLocation yacLocation) {
		attributesNumber = 2;
		method = InterpolationMethod.USER_FILE;
		
		this.filename = fileName;
		this.yacLocation = yacLocation;
	}

	/**
	 * 
	 * @return file name
	 */
	public String getFilename() {
		return filename;
	}
	
	/**
	 * 
	 * @return object from {@link YacLocation} indicating the location of the data
	 */
	public YacLocation getYacLocation() {
		return yacLocation;
	}
	
	/**
	 * get attribute name by the order of index
	 * @param index the order of attributes on the panel
	 * @return  the name of attribute
	 * 				if index = 0 return FILENAME, if 1 return YAC_LOCATION
	 */
	public String getAttributeName(int index) {
		if (index == 0) {
			return Dom.Attribute.FILENAME;
		} else if (index == 1) {
			return Dom.Attribute.YAC_LOCATION;
		}
		return null;
	}
	
	/**
	 * get attribute value by the order of index
	 * @param index the order of attributes on the panel
	 * @return the value of attribute as String
	 * 				if 0 return filename, if 1 return yacLocation
	 */
	public String getAttributeValue(int index) {
		if (index == 0) {
			return filename;
		} else if (index == 1) {
			return yacLocation.toString();
		}
		return null;
	}

}
