#include <stdlib.h>
#include <stdio.h>
#include "grid_search.h"
#include "bucket_search.h"
#include "grid.h"
#include "grid_reg2d.h"

struct grid * generate_grid(double offset[2], double range[2][2],
                            unsigned * num_cells, unsigned * cyclic) {

   double *coords_x, *coords_y;

   coords_x = malloc((num_cells[0] + 1) * sizeof(*coords_x));
   coords_y = malloc((num_cells[1] + 1) * sizeof(*coords_y));

   unsigned i;
   double dx, dy;

   dx = (range[0][1] - range[0][0]) / (double) num_cells[0];
   dy = (range[1][1] - range[1][0]) / (double) num_cells[1];

   for (i = 0; i <= num_cells[0]; ++i) {

      coords_x[i] = range[0][0] + dx * (double)i + offset[0];
   }

   for (i = 0; i <= num_cells[1]; ++i) {

      coords_y[i] = range[1][0] + dy * (double)i + offset[0];
   }

   return yac_reg2d_grid_new(coords_x, coords_y, num_cells, cyclic);
}

int main (void) {

   struct grid * big_grid_a, * big_grid_b, * small_grid_a, * small_grid_b;

   unsigned num_cells_big[2] = {1024,1024};
   unsigned num_cells_small[2] = {128,128};
   unsigned cyclic[2] = {0,0};

   double offset_big_a[2] = {0,0};
   double offset_big_b[2] = {0.3,0.15};
   double offset_small_a[2] = {0,0};
   double offset_small_b[2] = {2.5,1.2};

   double range[2][2] = {{-180,180},{-80,80}};

   big_grid_a =   generate_grid(offset_big_a,   range, num_cells_big,   cyclic);
   big_grid_b =   generate_grid(offset_big_b,   range, num_cells_big,   cyclic);
   small_grid_a = generate_grid(offset_small_a, range, num_cells_small, cyclic);
   small_grid_b = generate_grid(offset_small_b, range, num_cells_small, cyclic);

   {
      struct grid_search * search;
      struct dep_list deps;

      search = yac_bucket_search_new(big_grid_a);

      yac_do_point_search_p(search, big_grid_b, &deps);

      yac_delete_grid_search(search);
   }

   return (EXIT_SUCCESS);
}
