/**
 * @file grid_unpack.c
 * @brief Generic rountine to initialise packing of grids.
 *
 * @copyright Copyright  (C)  2013 Moritz Hanke <hanke@dkrz.de>
 *                                 Rene Redler <rene.redler@mpimet.mpg.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler <rene.redler@mpimet.mpg.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 */

#include <stdlib.h>

#include "grid.h"
#include "grid_reg2d.h"
#include "grid_curve2d.h"
#include "grid_unstruct.h"
#include "utils.h"

struct grid * yac_unpack_grid(double * dble_buf, unsigned * dble_buf_data_size,
                              unsigned * uint_buf, unsigned * uint_buf_data_size) {

   if (uint_buf[0] == yac_hash("REG2D"))
      return yac_unpack_reg2d_grid(dble_buf, dble_buf_data_size, uint_buf, uint_buf_data_size);
   else if (uint_buf[0] == yac_hash("CURVE2D"))
      return yac_unpack_curve2d_grid(dble_buf, dble_buf_data_size, uint_buf, uint_buf_data_size);
   else if (uint_buf[0] == yac_hash("UNSTRUCT"))
      return yac_unpack_unstruct_grid(dble_buf, dble_buf_data_size, uint_buf, uint_buf_data_size);
   else {
      yac_internal_abort_message ( "ERROR: grid must be one of REG2D|CURVE2D|UNSTRUCT.", __FILE__, __LINE__ );
   }

   return NULL;
}
