/**
 * @file interpolation_method_file.h
 * @brief weight file based interpolation method
 *
 * point interpolation method that reads weights from a file in SCRIP format
 *
 * @copyright Copyright  (C)  2015 Moritz Hanke <hanke@dkrz.de>
 *                                 Rene Redler <rene.redler@mpimet.mpg.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler <rene.redler@mpimet.mpg.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 */

#ifndef INTERPOLATION_METHOD_FILE_H
#define INTERPOLATION_METHOD_FILE_H

#include "points.h"
#include "interpolation_method.h"

#define YAC_WEIGHT_FILE_VERSION_STRING "yac weight file 1.0"

/** \example test_interpolation_method_file.c
* This shows how the file interpolation works
*/
 
/**
 * constructor for a interpolation method of type interpolation_method_file
 * @param[in] search        global search object that is to be used for the
 *                          interpolation
 * @param[in] domain        %domain of the local process
 * @param[in] stack_name    name of the interpolation stack
 * @param[in] src_locations location for each pointset
 * @param[in] num_pointsets number of point sets of the field
 * @param[in] file_name     name of file that contains the link and weights
 *                          information
 * @param[in] io_proc_step  stride between ranks of io processes
 */
struct interpolation_method *
yac_new_interpolation_method_file(struct global_search * search,
                                  struct domain * domain,
                                  char const * stack_name,
                                  enum yac_location * src_locations,
                                  unsigned num_pointsets,
                                  char const * file_name,
                                  unsigned io_proc_step);

#endif // INTERPOLATION_METHOD_FILE_H
