/**
 * @file interpolation_enum.c
 *
 * @copyright Copyright  (C)  2013 Rene Redler <rene.redler@mpimet.mpg.de>
 *                                 Teresa Holfeld <teresa.holfeld@zmaw.de>
 *
 * @version 1.0
 * @author Rene Redler <rene.redler@mpimet.mpg.de>
 *         Teresa Holfeld <teresa.holfeld@zmaw.de>
 */
/*
 * Keywords:
 * Maintainer: Rene Redler <rene.redler@mpimet.mpg.de>
 *             Teresa Holfeld <teresa.holfeld@zmaw.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 */

#include <string.h>
#include "interpolation_enum.h"

static char average[]               = "average";
static char n_nearest_neighbor[]    = "n-nearest_neighbor";
static char conservative[]          = "conservative";
static char patch_recovery[]        = "patch_recovery";
static char source_to_target_map[]  = "source_to_target_map";
static char smooth_patch_recovery[] = "smooth_patch_recovery";
static char fixed_value[]           = "fixed_value";
static char user_file[]             = "user_file";
static char check[]                 = "check";

char * yac_interpolation_enum_to_string(const enum yac_interpolation_list interpolation) {
   switch (interpolation) {
   case (AVERAGE):
         return average;
   case (N_NEAREST_NEIGHBOR):
         return n_nearest_neighbor;
   case (CONSERVATIVE):
         return conservative;
   case (PATCH_RECOVERY):
         return patch_recovery;
   case (SOURCE_TO_TARGET_MAP):
         return source_to_target_map;
   case (SMOOTH_PATCH_RECOVERY):
         return smooth_patch_recovery;
   case (FIXED_VALUE):
         return fixed_value;
   case (USER_FILE):
         return user_file;
   case (CHECK):
         return check;
   default:
      return NULL;
   }
}

char * yac_interpolation_int_to_string(const int interpolation) {

   /* The case selection has to be consistent with what is implemented
    * in interpolation_enum.h . Unfortunately we have no way to check
    * for consistency.
    */

   switch (interpolation) {
   case (1):
         return average;
   case (2):
         return n_nearest_neighbor;
   case (3):
         return conservative;
   case (4):
         return patch_recovery;
   case (5):
         return smooth_patch_recovery;
   case (6):
         return source_to_target_map;
   case (7):
         return fixed_value;
   case (8):
         return user_file;
   case (9):
         return check;
   default:
      return NULL;
   }
}

enum yac_interpolation_list yac_interpolation_enum_from_string(const char * interpolation) {
   if ( interpolation == NULL ) return UNDEFINED;

   if (strcmp(interpolation, average) == 0) {
      return AVERAGE;
   } else if (strcmp(interpolation, n_nearest_neighbor) == 0) {
      return N_NEAREST_NEIGHBOR;
   } else if (strcmp(interpolation, conservative) == 0) {
      return CONSERVATIVE;
   } else if (strcmp(interpolation, patch_recovery) == 0) {
      return PATCH_RECOVERY;
   } else if (strcmp(interpolation, source_to_target_map) == 0) {
      return SOURCE_TO_TARGET_MAP;
   } else if (strcmp(interpolation, smooth_patch_recovery) == 0) {
      return SMOOTH_PATCH_RECOVERY;
   } else if (strcmp(interpolation, fixed_value) == 0) {
      return FIXED_VALUE;
   } else if (strcmp(interpolation, user_file) == 0) {
      return USER_FILE;
   } else if (strcmp(interpolation, check) == 0) {
      return CHECK;
   } else {
      return UNDEFINED;
   }
}

char * yac_verified_interpolation(const char * interpolation) {
   if (strcmp(interpolation, average) == 0) {
      return average;
   } else if (strcmp(interpolation, n_nearest_neighbor) == 0) {
      return n_nearest_neighbor;
   } else if (strcmp(interpolation, conservative) == 0) {
      return conservative;
   } else if (strcmp(interpolation, patch_recovery) == 0) {
      return patch_recovery;
   } else if (strcmp(interpolation, source_to_target_map) == 0) {
      return source_to_target_map;
   } else if (strcmp(interpolation, smooth_patch_recovery) == 0) {
      return smooth_patch_recovery;
   } else if (strcmp(interpolation, fixed_value) == 0) {
      return fixed_value;
   } else if (strcmp(interpolation, user_file) == 0) {
      return user_file;
   } else if (strcmp(interpolation, check) == 0) {
      return check;
   } else {
      return NULL;
   }
}
