/**
 * @file test_bisection.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include "utils.h"
#include "tests.h"
#include "search.h"

int main (void) {

   unsigned const axis_length = 511;
   unsigned const data_length = 8;
   unsigned const test_length = 100;
   unsigned const nbr_tests   = 10;

   double axis_data[axis_length];
   double search_data[data_length];
   double test_data[test_length];

   double frac;

   int * position, * found;

   unsigned i;

   position = malloc(MAX(data_length, test_length) * sizeof(*position));
   found = malloc(MAX(data_length, test_length) * sizeof(*found));

   // set the test data
   search_data[0] = -0.5;
   search_data[1] =  0.0;
   search_data[2] =  0.5;
   search_data[3] = 40;
   search_data[4] = 250.5;
   search_data[5] = (double)(axis_length-1) - 0.5;
   search_data[6] = (double)(axis_length-1);
   search_data[7] = (double)(axis_length-1) + 0.5;

#ifdef VERBOSE
   puts ("----- ascending axis data -----");
#endif
   
   // generate the axis data
   for (i = 0; i < axis_length; ++i) {
      axis_data[i] = (double)i;
   }

   yac_bisection_search(search_data, data_length, axis_data, axis_length, position,
                        found, 0);

#ifdef VERBOSE
   for (i = 0; i < data_length; ++i) {
      if (found[i])
         printf("%6.1f in box %3d : %6.1f - %6.1f %c\n", search_data[i], position[i], axis_data[position[i]], axis_data[position[i]+1], found[i]?'T':'F');
      else
         printf("%6.1f                              F\n", search_data[i]);
   }

   puts ("----- descending axis data -----");
#endif

   // generate the axis data
   for (i = 0; i < axis_length; ++i) {

      axis_data[i] = (double)(axis_length-1-i);
   }

   yac_bisection_search(search_data, data_length, axis_data, axis_length, position,
                        found, 0);

#ifdef VERBOSE
   for (i = 0; i < data_length; ++i) {
      if (found[i])
         printf("%6.1f in box %3d : %6.1f - %6.1f %c\n", search_data[i], position[i], axis_data[position[i]], axis_data[position[i]+1], found[i]?'T':'F');
      else
         printf("%6.1f                              F\n", search_data[i]);
   }
#endif

   // do some longer test for performance analysis

   frac = (double)(axis_length) / (double)(test_length);

   // re-generate the axis data
   for (i = 0; i < axis_length; ++i) {
      axis_data[i] = (double)i;
   }
   // generate the test data
   for (i=0; i < test_length;  ++i) {
     test_data[i] = (double)(i+1) * frac;
   }

   for (i = 0; i < nbr_tests; ++i) {
      yac_bisection_search(test_data, test_length, axis_data, axis_length, position,
                           found, 0);
   }

   int axis_data_int[axis_length];
   int search_data_int[data_length];

   // generate the axis data
   for (i = 0; i < axis_length; ++i) {
      axis_data_int[i] = axis_length - i;
   }
   // generate the test data
   for (i=0; i < data_length;  ++i) {
     search_data_int[i] = i * 100;
   }

#ifdef VERBOSE
   puts ("------- integer location -------");
   puts ("------ ascending axis data -----");
#endif

   yac_bisection_search_int (search_data_int, data_length, axis_data_int, axis_length, position, found);

#ifdef VERBOSE
   for (i = 0; i < data_length; ++i) {
      if (found[i])
         printf("%i on location %3d : %i  %c\n", search_data_int[i], position[i], axis_data_int[position[i]], found[i]?'T':'F');
      else
         printf("%i                       F\n", search_data_int[i]);
   }

   puts ("----- descending axis data -----");
#endif

   // generate the axis data
   for (i = 0; i < axis_length; ++i) {
      axis_data_int[i] = i;
   }

   yac_bisection_search_int (search_data_int, data_length, axis_data_int, axis_length, position, found);

#ifdef VERBOSE
   for (i = 0; i < data_length; ++i) {
      if (found[i])
         printf("%i on location %3d : %i  %c\n", search_data_int[i], position[i], axis_data_int[position[i]], found[i]?'T':'F');
      else
         printf("%i                       F\n", search_data_int[i]);
   }
#endif

   free(position);
   free(found);

   return TEST_EXIT_CODE;
}
