/**
 * @file test_communicator_local_fail.c
 *
 * @copyright Copyright  (C)  2015 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <unistd.h>

#include "tests.h"

#include "communicator.h"
#include "communicator_local.h"

int test_index = -1;

static void
parse_options(int *argc, char ***argv);

int main (int argc, char **argv) {

   parse_options(&argc, &argv);

   { // tests with a single local rank
      struct communicator * comm;
      struct communicator * child_comm;

      comm = yac_local_comm_new();
      child_comm = yac_named_local_comm_new(comm, "child");

      { // tests the size of the communicator
         if (yac_get_comm_size(comm) != 1)
            return 0;
      }

      switch(test_index) {

        case 0: { // tests sending of data (first send then recv)
                  // send more than to receive -> should fail

           unsigned send_data[4] = {0,1,2,3};

           yac_comm_bsend(send_data, 4, COMM_UINT, 0, 0, comm);

           yac_comm_recv_callback(1, COMM_UINT, 0, 0, comm, send_data,
                                  NULL, NULL, MSG_PRIO_HIGH);

           yac_comm_wait(comm, 0);

           break;
        }

        case 1: { // tests sending of data (first recv then send)
                  // send more than to receive -> should fail

           unsigned send_data[4] = {0,1,2,3};

           yac_comm_recv_callback(1, COMM_UINT, 0, 0, comm, send_data,
                                  NULL, NULL, MSG_PRIO_HIGH);

           yac_comm_bsend(send_data, 4, COMM_UINT, 0, 0, comm);

           yac_comm_wait(comm, 0);

           break;
        }
      }

      yac_free_communicator(child_comm);
      yac_free_communicator(comm);
   }

   yac_comm_finalize();

   return TEST_EXIT_CODE;
}

static void
parse_options(int *argc, char ***argv)
{
  int opt;
  while ((opt = getopt(*argc, *argv, "t:")) != -1) {
    switch (opt) {
    case 't':
      test_index = atoi(optarg);
    }
  }
}