#define VERBOSE
/**
 * @file test_couple_config_2.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <mpi.h>

#include "tests.h"

#include "yacpar.h"
#include "config_xml.h"
#include "communicator.h"
#include "communicator_mpi.h"
#include "grid_reg2d.h"
#include "domain.h"
#include "couple_config.h"
#include "utils.h"
#include "interpolation_method.h"
#include "interpolation_method_check.h"
#include "yac_interface.h"

void check_constructor(void * user_data);

void check_do_search(int const * global_ids, double const * x_coordinates,
                     double const * y_coordinates, unsigned num_points,
                     void * user_data);

int get_num_unique_configurations(unsigned tgt_config[3][4][2],
                                  unsigned src_config[3][4][2]);

int main (void) {

#ifdef USE_XML2

  yac_world_comm = yac_mpi_comm_new();
  int rank = yac_get_comm_rank(yac_world_comm);

  if (yac_get_comm_size(yac_world_comm) != 9) {

    PUT_ERR("ERROR: wrong number of processes\n");
    return TEST_EXIT_CODE;
  }

  char const * xml_config_filename = "couple_config_2_test_1.xml";
  char const * xml_config_file_format =
#include "test_couple_config_xml_string.h"
;

  unsigned num_components = 3;

  unsigned field_configuration[6][3][4][2];

  for (unsigned comp_rank = 0; comp_rank < 3; ++comp_rank) {
    // all fields have different configurations on all processes
    field_configuration[0][comp_rank][0][0] = 0;
    field_configuration[0][comp_rank][1][0] = 1;
    field_configuration[0][comp_rank][2][0] = 2;
    field_configuration[0][comp_rank][3][0] = 3;
    field_configuration[0][comp_rank][0][1] = 0;
    field_configuration[0][comp_rank][1][1] = 1;
    field_configuration[0][comp_rank][2][1] = 2;
    field_configuration[0][comp_rank][3][1] = 3;

    // all fields have identical configurations on all processes
    field_configuration[1][comp_rank][0][0] = 0;
    field_configuration[1][comp_rank][1][0] = 0;
    field_configuration[1][comp_rank][2][0] = 0;
    field_configuration[1][comp_rank][3][0] = 0;
    field_configuration[1][comp_rank][0][1] = 0;
    field_configuration[1][comp_rank][1][1] = 0;
    field_configuration[1][comp_rank][2][1] = 0;
    field_configuration[1][comp_rank][3][1] = 0;

    // there are only two different configurations on all processes
    field_configuration[2][comp_rank][0][0] = 0;
    field_configuration[2][comp_rank][1][0] = 0;
    field_configuration[2][comp_rank][2][0] = 1;
    field_configuration[2][comp_rank][3][0] = 1;
    field_configuration[2][comp_rank][0][1] = 0;
    field_configuration[2][comp_rank][1][1] = 0;
    field_configuration[2][comp_rank][2][1] = 1;
    field_configuration[2][comp_rank][3][1] = 1;

    // all fields have identical configurations on all processes except the
    // first one
    field_configuration[3][comp_rank][0][0] = 0;
    field_configuration[3][comp_rank][1][0] = 0;
    field_configuration[3][comp_rank][2][0] = 0;
    field_configuration[3][comp_rank][3][0] = 0;
    field_configuration[3][comp_rank][0][1] = (comp_rank == 0)?0:0;
    field_configuration[3][comp_rank][1][1] = (comp_rank == 0)?1:0;
    field_configuration[3][comp_rank][2][1] = (comp_rank == 0)?2:0;
    field_configuration[3][comp_rank][3][1] = (comp_rank == 0)?3:0;

    //
    field_configuration[4][comp_rank][0][0] = 0;
    field_configuration[4][comp_rank][1][0] = 0;
    field_configuration[4][comp_rank][2][0] = 2;
    field_configuration[4][comp_rank][3][0] = 3;
    field_configuration[4][comp_rank][0][1] = (comp_rank&1)?0:0;
    field_configuration[4][comp_rank][1][1] = (comp_rank&1)?1:0;
    field_configuration[4][comp_rank][2][1] = (comp_rank&1)?2:2;
    field_configuration[4][comp_rank][3][1] = (comp_rank&1)?2:3;

    //
    field_configuration[5][comp_rank][0][0] = 0;
    field_configuration[5][comp_rank][1][0] = 1;
    field_configuration[5][comp_rank][2][0] = 2;
    field_configuration[5][comp_rank][3][0] = 0;
    field_configuration[5][comp_rank][0][1] = 0;
    field_configuration[5][comp_rank][1][1] = 1;
    field_configuration[5][comp_rank][2][1] = 2;
    field_configuration[5][comp_rank][3][1] = 0;
  }

  char check_constructor_key[2][1024];
  char check_do_search_key[2][1024];

  sprintf(check_constructor_key[0], "check_constructor_%d_to_%d",
          (rank % num_components) + 1,(((rank % num_components) + 1) % 3) + 1);
  sprintf(check_constructor_key[1], "check_constructor_%d_to_%d",
          (rank % num_components) + 1,(((rank % num_components) + 2) % 3) + 1);

  sprintf(check_do_search_key[0], "check_do_search_%d_to_%d",
          (rank % num_components) + 1,(((rank % num_components) + 1) % 3) + 1);
  sprintf(check_do_search_key[1], "check_do_search_%d_to_%d",
          (rank % num_components) + 1,(((rank % num_components) + 2) % 3) + 1);

  int num_interpolation_method_stacks[2][2];

  for (unsigned i = 0; i < 2; ++i) {
    yac_interpolation_method_check_add_constructor_callback(
      check_constructor, &(num_interpolation_method_stacks[0][i]),
      check_constructor_key[i]);
    yac_interpolation_method_check_add_do_search_callback(
      check_do_search, &(num_interpolation_method_stacks[1][i]),
      check_do_search_key[i]);
  }

  for (unsigned tgt_config = 0; tgt_config < 6; ++tgt_config) {
    for (unsigned src_config = 0; src_config < 6; ++src_config) {

      for (unsigned i = 0; i < 2; ++i)
        for (unsigned j = 0; j < 2; ++j)
          num_interpolation_method_stacks[i][j] = 0;

      MPI_Barrier(MPI_COMM_WORLD);
      if (rank == 0) {

        // generate xml file
        FILE * xml_config_file = fopen(xml_config_filename, "w+");
        fprintf(xml_config_file, xml_config_file_format,
                field_configuration[tgt_config][0][0][0] + 1,
                field_configuration[tgt_config][0][1][0] + 1,
                field_configuration[tgt_config][0][2][0] + 1,
                field_configuration[tgt_config][0][3][0] + 1,
                field_configuration[src_config][0][0][0] + 1,
                field_configuration[src_config][0][1][0] + 1,
                field_configuration[src_config][0][2][0] + 1,
                field_configuration[src_config][0][3][0] + 1,
                field_configuration[tgt_config][1][0][0] + 1,
                field_configuration[tgt_config][1][1][0] + 1,
                field_configuration[tgt_config][1][2][0] + 1,
                field_configuration[tgt_config][1][3][0] + 1,
                field_configuration[src_config][1][0][0] + 1,
                field_configuration[src_config][1][1][0] + 1,
                field_configuration[src_config][1][2][0] + 1,
                field_configuration[src_config][1][3][0] + 1,
                field_configuration[tgt_config][2][0][0] + 1,
                field_configuration[tgt_config][2][1][0] + 1,
                field_configuration[tgt_config][2][2][0] + 1,
                field_configuration[tgt_config][2][3][0] + 1,
                field_configuration[src_config][2][0][0] + 1,
                field_configuration[src_config][2][1][0] + 1,
                field_configuration[src_config][2][2][0] + 1,
                field_configuration[src_config][2][3][0] + 1);
        fclose(xml_config_file);
      }

      MPI_Barrier(MPI_COMM_WORLD);

      yac_xml_init_config_xml(xml_config_filename, "coupling.xsd");
      yac_cdef_datetime ( NULL, NULL) ;

      char * component_names[3] = {"comp_1", "comp_2", "comp_3"};
      struct component * component;
      unsigned component_id;
      struct communicator * component_comm;
      int comp_rank;

      component = yac_component_new(component_names[rank % num_components]);
      component_id = yac_pointer_to_unique_id(component);
      component_comm = yac_component_get_communicator(component);
      comp_rank = yac_get_comm_rank(component_comm);

      struct grid * grid[4];
      unsigned num_cells[2] = {1,1};
      unsigned cyclic[2] = {0,0};
      double coordinates_x[2];
      double coordinates_y[4][2] = {{0,1},{1,2},{2,3},{3,4}};

      coordinates_x[0] = comp_rank;
      coordinates_x[1] = comp_rank + 1;

      for (unsigned i = 0; i < 4; ++i)
        grid[i] = yac_reg2d_grid_new(coordinates_x, coordinates_y[i], num_cells,
                                     cyclic);

      struct subdomain subdomain[2][4];
      int global_cell_ids = comp_rank;
      unsigned cell_is_on_rank = -1;
      int global_corner_ids[4] = {4 * comp_rank + 0, 4 * comp_rank + 1,
                                  4 * comp_rank + 2, 4 * comp_rank + 3};
      unsigned corner_is_on_rank[4] = {-1,-1,-1,-1};

      for (unsigned i = 0; i < 4; ++i) {
        yac_init_subdomain_unstruct(&subdomain[0][i],
                                    grid[field_configuration[tgt_config][comp_rank][i][0]],
                                    component_comm, global_corner_ids,
                                    corner_is_on_rank, &global_cell_ids,
                                    &cell_is_on_rank, NULL, NULL);
        yac_init_subdomain_unstruct(&subdomain[1][i],
                                    grid[field_configuration[src_config][comp_rank][i][0]],
                                    component_comm, global_corner_ids,
                                    corner_is_on_rank, &global_cell_ids,
                                    &cell_is_on_rank, NULL, NULL);
      }

      struct domain domain[2][4];
      struct subdomain * subdomains[2][4][1] = {{{subdomain[0]+0},
                                                 {subdomain[0]+1},
                                                 {subdomain[0]+2},
                                                 {subdomain[0]+3}},
                                                {{subdomain[1]+0},
                                                 {subdomain[1]+1},
                                                 {subdomain[1]+2},
                                                 {subdomain[1]+3}}};
      char domain_names[2][4][1024];

      for (unsigned i = 0; i < 4; ++i) {

        sprintf(domain_names[0][i], "grid_%d_in", i+1);
        yac_init_domain(&domain[0][i], component_comm, subdomains[0][i], 1,
                        domain_names[0][i]);

        sprintf(domain_names[1][i], "grid_%d_out", i+1);
        yac_init_domain(&domain[1][i], component_comm, subdomains[1][i], 1,
                        domain_names[1][i]);
      }

      for (unsigned i = 0; i < 4; ++i) {
        yac_enddef_domain(&domain[0][i]);
        yac_enddef_domain(&domain[1][i]);
      }

      struct points points[4];
      struct points * points_per_subdomain[4][1] = {{&points[0]}, {&points[1]},
                                                    {&points[2]}, {&points[3]}};
      struct points ** point_sets[4][1] = {{points_per_subdomain[0]},
                                           {points_per_subdomain[1]},
                                           {points_per_subdomain[2]},
                                           {points_per_subdomain[3]}};

      for (unsigned i = 0; i < 4; ++i)
        yac_init_points(&points[i], grid[i], CORNER, coordinates_x, coordinates_y[i]);

      char field_name_in[8][1024], field_name_out[8][1024];

      {
        unsigned count = 0;

        for (unsigned i = 0; i < 3; ++i) {

          if ((rank % num_components) == i) continue;

          for (unsigned j = 1; j <= 4; ++j) {
            sprintf(field_name_out[count], "transient_from_%d_to_%d_%d",
                    (rank % num_components)+1, i+1, j);
            sprintf(field_name_in[count], "transient_from_%d_to_%d_%d", i+1,
                    (rank % num_components)+1, j);
            ++count;
          }
        }
      }

      struct coupling_field * coupling_fields[2][8];

      for (unsigned i = 0; i < 8; ++i) {
        coupling_fields[0][i] =
          yac_coupling_field_new(field_name_in[i], component_id,
                                 // &domain[0][i&3],
                                 &domain[0][field_configuration[tgt_config][comp_rank][i&3][0]],
                                 point_sets[field_configuration[tgt_config][comp_rank][i&3][1]],
                                 NULL, 1);
        coupling_fields[1][i] =
          yac_coupling_field_new(field_name_out[i], component_id,
                                 // &domain[1][i&3],
                                 &domain[1][field_configuration[src_config][comp_rank][i&3][0]],
                                 point_sets[field_configuration[src_config][comp_rank][i&3][1]],
                                 NULL, 1);
      }

      yac_setup_couple_config(1, &component, 16, &(coupling_fields[0][0]));
      yac_enddef_couple_config();

      unsigned ref_num_unique_configs =
        get_num_unique_configurations(field_configuration[tgt_config],
                                      field_configuration[src_config]);

      for (unsigned i = 0; i < 2; ++i)
        for (unsigned j = 0; j < 2; ++j)
          if (num_interpolation_method_stacks[i][j] != ref_num_unique_configs)
            PUT_ERR("ERROR: field_interpolation_gen was not correctly reused\n");

      for (unsigned i = 0; i < 2; ++i)
        for (unsigned j = 0; j < 8; ++j)
          yac_coupling_field_delete(coupling_fields[i][j]);

      for (unsigned i = 0; i < 4; ++i)
        yac_free_points(&points[i]);

      for (unsigned i = 0; i < 4; ++i) {
        yac_free_domain(&domain[0][i]);
        yac_free_domain(&domain[1][i]);
      }

      for (unsigned i = 0; i < 4; ++i)
        yac_delete_grid(grid[i]);

      yac_component_delete(component);

      yac_free_couple_config();
      yac_xml_destroy_config_xml();

      yac_reset_interpolation_method();
      yac_components_new_first_call_reset();
    }
  }

  yac_free_communicator(yac_world_comm);

  yac_comm_finalize();

  if (rank == 0) unlink(xml_config_filename);

  return TEST_EXIT_CODE;

#else // USE_XML2

  return EXIT_SKIP_TEST;

#endif // USE_XML2
}

void check_constructor(void * user_data) {

  ++*(int*)user_data;

  return;
}

void check_do_search(int const * global_ids, double const * x_coordinates,
                     double const * y_coordinates, unsigned num_points,
                     void * user_data) {

  ++*(int*)user_data;

  return;
}

int get_num_unique_configurations(unsigned tgt_config[3][4][2],
                                  unsigned src_config[3][4][2]) {

  unsigned num_configs = 0;

  for (unsigned i = 0; i < 4; ++i) {

    unsigned k;

    for (k = i + 1; k < 4; ++k) {

      int differs = 0;

      for (unsigned j = 0; j < 3; ++j)
        differs |= (tgt_config[j][i][1] != tgt_config[j][k][1]) ||
                   (src_config[j][i][1] != src_config[j][k][1]);

      if (!differs) break;
    }

    num_configs += k == 4;
  }

  return num_configs;
}
