#define VERBOSE
/**
 * @file test_couple_config_3.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <mpi.h>

#include "tests.h"

#include "yacpar.h"
#include "config_xml.h"
#include "communicator.h"
#include "communicator_mpi.h"
#include "grid_reg2d.h"
#include "domain.h"
#include "couple_config.h"
#include "utils.h"
#include "interpolation_method.h"
#include "yac_interface.h"

char * str_logical[2] = {"true", "false"};

int main (void) {

#ifdef USE_XML2

  yac_world_comm = yac_mpi_comm_new();
  int rank = yac_get_comm_rank(yac_world_comm);

  if (yac_get_comm_size(yac_world_comm) != 9) {

    PUT_ERR("ERROR: wrong number of processes\n");
    return TEST_EXIT_CODE;
  }

  char const * xml_config_filename = "couple_config_3_test.xml";
  char const * xml_config_file_format =
#include "test_couple_config_3_xml_string.h"
;

  for (unsigned i = 0; i < 1 << 6; ++i) {

    MPI_Barrier(MPI_COMM_WORLD);
    if (rank == 0) {
      // generate xml file
      FILE * xml_config_file = fopen(xml_config_filename, "w+");
      fprintf(xml_config_file, xml_config_file_format,
              str_logical[!(i & (1 << 0))], str_logical[!(i & (1 << 1))],
              str_logical[!(i & (1 << 2))], str_logical[!(i & (1 << 3))],
              str_logical[!(i & (1 << 4))], str_logical[!(i & (1 << 5))]);
      fclose(xml_config_file);
    }
    MPI_Barrier(MPI_COMM_WORLD);

    yac_xml_init_config_xml(xml_config_filename, "coupling.xsd");
    yac_cdef_datetime ( NULL, NULL) ;

    char * component_names[3] = {"comp_1", "comp_2", "comp_3"};
    char const * local_component_names[2];
    struct component * components[2];
    unsigned component_ids[2];
    struct communicator * component_comms[2];
    int comp_ranks[2];
    int comp_idx[3][2] = {{0,1},{0,2},{1,2}};

    for (unsigned i = 0; i < 2; ++i)
      local_component_names[i] = component_names[comp_idx[rank/3][i]];

    yac_components_new(local_component_names, 2, components);

    for (unsigned i = 0; i < 2; ++i) {
      component_ids[i] = yac_pointer_to_unique_id(components[i]);
      component_comms[i] = yac_component_get_communicator(components[i]);
      comp_ranks[i] = yac_get_comm_rank(component_comms[i]);
    }

    unsigned num_cells[2] = {1,1};
    unsigned cyclic[2] = {0,0};
    double coordinates_x[2][2];
    double coordinates_y[2] = {0,1};
    struct grid * grid[2];
    struct subdomain subdomain[2];
    int global_cell_ids[2];
    unsigned cell_is_on_rank = -1;
    int global_corner_ids[2][4];
    unsigned corner_is_on_rank[4] = {-1,-1,-1,-1};

    for (unsigned i = 0; i < 2; ++i) {
      coordinates_x[i][0] = comp_ranks[i];
      coordinates_x[i][1] = comp_ranks[i] + 1;
      grid[i] =
        yac_reg2d_grid_new(coordinates_x[i], coordinates_y, num_cells, cyclic);
      global_corner_ids[i][0] = comp_ranks[i] + 0;
      global_corner_ids[i][1] = comp_ranks[i] + 1;
      global_corner_ids[i][2] = comp_ranks[i] + 8;
      global_corner_ids[i][3] = comp_ranks[i] + 7;
      global_cell_ids[i] = comp_ranks[i];
      yac_init_subdomain_unstruct(&subdomain[i], grid[i],
                                  component_comms[i], global_corner_ids[i],
                                  corner_is_on_rank, &global_cell_ids[i],
                                  &cell_is_on_rank, NULL, NULL);
    }

    struct domain domain[2];
    struct subdomain * subdomains[2] = {&subdomain[0], &subdomain[1]};
    char domain_names[2][1024];

    for (unsigned i = 0; i < 2; ++i) {
      sprintf(domain_names[i], "grid_%d", comp_idx[rank/3][i] + 1);
      yac_init_domain(&domain[i], component_comms[i], &subdomains[i], 1,
                      domain_names[i]);
    }

    for (unsigned i = 0; i < 2; ++i) {
      yac_enddef_domain(&domain[i]);
    }

    struct points points[2];
    struct points * points_per_subdomain[2][1] = {{&points[0]}, {&points[1]}};
    struct points ** point_sets[2][1] = {{points_per_subdomain[0]},
                                         {points_per_subdomain[1]}};

    for (unsigned i = 0; i < 2; ++i)
      yac_init_points(&points[i], grid[i], CORNER, coordinates_x[i], coordinates_y);

    char field_name[2][2][2][1024];

    for (unsigned i = 0; i < 2; ++i) {
      int curr_comp_idx = comp_idx[rank/3][i];
      for (unsigned j = 0; j < 2; ++j) {
        int remote_comp_idx = (3 + curr_comp_idx + j + 1) % 3;
        sprintf(field_name[i][j][0], "transient_from_%d_to_%d",
                curr_comp_idx + 1, remote_comp_idx + 1);
        sprintf(field_name[i][j][1], "transient_from_%d_to_%d",
                remote_comp_idx + 1, curr_comp_idx + 1);
      }
    }

    struct coupling_field * coupling_fields[2][2][2];

    for (unsigned i = 0; i < 2; ++i)
      for (unsigned j = 0; j < 2; ++j)
        for (unsigned k = 0; k < 2; ++k)
          coupling_fields[i][j][k] =
            yac_coupling_field_new(field_name[i][j][k], component_ids[i],
                                   &domain[i], point_sets[i], NULL, 1);

    yac_setup_couple_config(2, components, 8, &(coupling_fields[0][0][0]));
    yac_enddef_couple_config();

    // unsigned ref_num_unique_configs =
      // get_num_unique_configurations(field_configuration[tgt_config],
                                    // field_configuration[src_config]);

    // for (unsigned i = 0; i < 2; ++i)
      // for (unsigned j = 0; j < 2; ++j)
        // if (num_interpolation_method_stacks[i][j] != ref_num_unique_configs)
          // PUT_ERR("ERROR: field_interpolation_gen was not correctly reused\n");

    for (unsigned i = 0; i < 2; ++i)
      for (unsigned j = 0; j < 2; ++j)
        for (unsigned k = 0; k < 2; ++k)
          yac_coupling_field_delete(coupling_fields[i][j][k]);

    for (unsigned i = 0; i < 2; ++i) {
      yac_free_points(&points[i]);
      yac_free_domain(&domain[i]);
      yac_delete_grid(grid[i]);
      yac_component_delete(components[i]);
    }

    yac_free_couple_config();
    yac_xml_destroy_config_xml();

    yac_reset_interpolation_method();
    yac_components_new_first_call_reset();
  }

  yac_free_communicator(yac_world_comm);

  yac_comm_finalize();

  if (rank == 0) unlink(xml_config_filename);

  return TEST_EXIT_CODE;

#else // USE_XML2

  return EXIT_SKIP_TEST;

#endif // USE_XML2
}

void check_constructor(void * user_data) {

  ++*(int*)user_data;

  return;
}

void check_do_search(int const * global_ids, double const * x_coordinates,
                     double const * y_coordinates, unsigned num_points,
                     void * user_data) {

  ++*(int*)user_data;

  return;
}

int get_num_unique_configurations(unsigned tgt_config[3][4],
                                  unsigned src_config[3][4]) {

  unsigned num_configs = 0;

  for (unsigned i = 0; i < 4; ++i) {

    unsigned k;

    for (k = i + 1; k < 4; ++k) {

      int differs = 0;

      for (unsigned j = 0; j < 3; ++j)
        differs |= (tgt_config[j][i] != tgt_config[j][k]) ||
                   (src_config[j][i] != src_config[j][k]);

      if (!differs) break;
    }

    num_configs += k == 4;
  }

  return num_configs;
}
