/**
 * @file test_find_overlap.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include "tests.h"
#include "search.h"
#include "grid.h"
#include "grid_reg2d.h"

int main (void) {

   //---------------
   // setup
   //---------------

   struct grid * reg_tgt_grid;
   struct grid * reg_src_grid;

   double coordinates_x[2][3] = {{0,1,2}, {0.5,1.5,2.5}};
   double coordinates_y[2][3] = {{0,1,2}, {0.5,1.5,2.5}};
   unsigned num_cells[2] = {2,2};
   unsigned cyclic[2] = {0,0};

   struct dep_list inital_src_to_tgt;
   struct dep_list src_to_tgt;

   unsigned inital_num_deps_per_element[4] = {1,1,1,1};
   unsigned inital_dependencies[4] = {0,0,0,0};

   unsigned ref_num_overlap_per_cell[4] = {1,2,2,4};
   unsigned ref_overlaps[4][4] = {{0,-1,-1,-1},
                                  {0,1,-1,-1},
                                  {0,2,-1,-1},
                                  {0,1,2,3}};

   unsigned i, j, k;
   unsigned num_matching_overlaps;
   unsigned const * curr_overlaps;

   // set up grid data
   reg_src_grid = yac_reg2d_grid_new(coordinates_x[0], coordinates_y[0], num_cells, cyclic);
   reg_tgt_grid = yac_reg2d_grid_new(coordinates_x[1], coordinates_y[1], num_cells, cyclic);

   // set up inital_src_to_tgt
   yac_init_dep_list(&inital_src_to_tgt);
   yac_set_dependencies(&inital_src_to_tgt, 4, inital_num_deps_per_element,
                    inital_dependencies);

   //---------------
   // testing
   //---------------

   yac_find_overlapping_cells(reg_src_grid, reg_tgt_grid,
                              inital_src_to_tgt, &src_to_tgt);

   if (src_to_tgt.num_elements != 4)
      PUT_ERR("wrong number of elements in src_to_tgt\n")

   for (i = 0; i < 4; ++i)
      if (src_to_tgt.num_deps_per_element[i] != ref_num_overlap_per_cell[i])
         PUT_ERR("wrong number of overlaps per cell in src_to_tgt\n")

   for (i = 0; i < 4; ++i) {

      num_matching_overlaps = 0;
      curr_overlaps = yac_get_dependencies_of_element(src_to_tgt, i);

      for (j = 0; j < ref_num_overlap_per_cell[i]; ++j) {
         for (k = 0; k < ref_num_overlap_per_cell[i]; ++k) {

            if (curr_overlaps[j] == ref_overlaps[i][k]) ++num_matching_overlaps;
         }
      }
      if (num_matching_overlaps != ref_num_overlap_per_cell[i])
         PUT_ERR("wrong overlaps in src_to_tgt\n")
   }

   yac_free_dep_list(&src_to_tgt);
   yac_delete_grid(reg_src_grid);
   yac_delete_grid(reg_tgt_grid);

   return TEST_EXIT_CODE;
}
