/**
 * @file test_gcxgc.c
 *
 * @copyright Copyright  (C)  2014 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include "test_cxc.h"
#include "search.h"

void test_gcxgc(double lon_a, double lat_a, double lon_b, double lat_b,
                double lon_c, double lat_c, double lon_d, double lat_d,
                double lon_ref_p, double lat_ref_p,
                double lon_ref_q, double lat_ref_q, int ref_ret_val);
void test_gcxgc_rad(double lon_a, double lat_a, double lon_b, double lat_b,
                    double lon_c, double lat_c, double lon_d, double lat_d,
                    double lon_ref_p, double lat_ref_p,
                    double lon_ref_q, double lat_ref_q, int ref_ret_val);

int main (void) {

   unsigned const p_between_ab = 1 << 0;
   unsigned const q_between_ab = 1 << 1;
   unsigned const p_between_cd = 1 << 2;
   unsigned const q_between_cd = 1 << 3;
   unsigned const circles_are_identically = 1 << 4;

   // simple example

   test_gcxgc(-10,   0, // point a
               10,   0, // point b
                0,  10, // point c
                0, -10, // point d
                0,   0, // reference point p
              180,   0, // reference point q
              p_between_ab + p_between_cd); // reference return value

   // d at north pole

   test_gcxgc(-10,   0, // point a
               10,   0, // point b
                0, -20, // point c
                0,  90, // point d
                0,   0, // reference point p
              180,   0, // reference point q
              p_between_ab + p_between_cd); // reference return value

   // edge_a == edge_b

   test_gcxgc(-10,   0, // point a
               10,   0, // point b
              -10,   0, // point c
               10,   0, // point d
              -10,   0, // reference point p
               10,   0, // reference point q
              p_between_ab + p_between_cd + q_between_ab + q_between_cd +
              circles_are_identically); // reference return value

   // both circles on the same plane but no intersection

   test_gcxgc(-45,  45, // point a
                0,   0, // point b
              225,  45, // point c
              135, -45, // point d
              -45,  45, // reference point p
                0,   0, // reference point q
              p_between_ab + q_between_ab +
              circles_are_identically); // reference return value

   // normal case + some cyclic stuff

   test_gcxgc(  45,   3, // point a
                45, -10, // point b
               -45,   0, // point c
              -135, -45, // point d
                45,  45, // reference point p
              -135, -45, // reference point q
              q_between_cd); // reference return value

   // normal case + some heavier cyclic stuff

   test_gcxgc(  67.5,  40, // point a
                67.5, -13, // point b
                 405,   0, // point c
                 450,   0, // point d
              -112.5,   0, // reference point p
                67.5,   0, // reference point q
              q_between_ab + q_between_cd); // reference return value


   // normal case + some heavier cyclic stuff + shifted end of edge b

   test_gcxgc(  67.5,  40, // point a
                67.5, -13, // point b
                  45,   0, // point c
                 450,   0, // point d
              -112.5,   0, // reference point p
                67.5,   0, // reference point q
              q_between_ab + q_between_cd); // reference return value

  // one of the two edge has length zero and is not on the other plane

   test_gcxgc( 10,  10, // point a
               10,  10, // point b
              -10, -10, // point c
               10,  11, // point d
               -1,  -1, // reference point p
               -1,  -1, // reference point q
              -1); // reference return value

   test_gcxgc(-10, -10, // point a
               10,  11, // point b
               10,  10, // point c
               10,  10, // point d
               -1,  -1, // reference point p
               -1,  -1, // reference point q
              -1); // reference return value

  // one of the two edge has length zero, is not on the other edge, but on the other plane

   test_gcxgc( 10,   10, // point a
               10,   10, // point b
              -10,  -10, // point c
                0,    0, // point d
               10,   10, // reference point p
               190, -10, // reference point q
              p_between_ab); // reference return value

   test_gcxgc(-10,  -10, // point a
                0,    0, // point b
               10,   10, // point c
               10,   10, // point d
               10,   10, // reference point p
               190, -10, // reference point q
              p_between_cd); // reference return value

   test_gcxgc( 10,   10, // point a
               10,   10, // point b
              190,  -10, // point c
              170,   10, // point d
               10,   10, // reference point p
               190, -10, // reference point q
              p_between_ab + q_between_cd); // reference return value

   test_gcxgc(190,  -10, // point a
              170,   10, // point b
               10,   10, // point c
               10,   10, // point d
               10,   10, // reference point p
               190, -10, // reference point q
              p_between_cd + q_between_ab); // reference return value

  // one of the two edge has length zero and touches the other edge

   test_gcxgc( 10,  10, // point a
               10,  10, // point b
              -10, -10, // point c
               10,  10, // point d
               10,   10, // reference point p
               190, -10, // reference point q
              p_between_ab + p_between_cd); // reference return value

   test_gcxgc( 10,  10, // point a
               10,  10, // point b
              -10, -10, // point c
               10,  10, // point d
               10,   10, // reference point p
               190, -10, // reference point q
              p_between_ab + p_between_cd); // reference return value

  // one of the two edge has length zero and is on the other edge

   test_gcxgc(  0,   0, // point a
                0,   0, // point b
              -10, -10, // point c
               10,  10, // point d
                0,   0, // reference point p
              180,   0, // reference point q
              p_between_ab + p_between_cd); // reference return value

   test_gcxgc(-10, -10, // point a
               10,  10, // point b
                0,   0, // point c
                0,   0, // point d
                0,   0, // reference point p
              180,   0, // reference point q
              p_between_ab + p_between_cd); // reference return value

   // both edges have length zero and do not match

   test_gcxgc(-10, -10, // point a
              -10, -10, // point b
               10,  10, // point c
               10,  10, // point d
               -1,  -1, // reference point p
               -1,  -1, // reference point q
              -1); // reference return value

   test_gcxgc( 10,  10, // point a
               10,  10, // point b
              -10, -10, // point c
              -10, -10, // point d
               -1,  -1, // reference point p
               -1,  -1, // reference point q
              -1); // reference return value

   // both edges have length zero and do not match, but are directly opposite
   // of each other

   test_gcxgc(-10, -10, // point a
              -10, -10, // point b
              170,  10, // point c
              170,  10, // point d
              -10, -10, // reference point p
              170,  10, // reference point q
              p_between_ab + q_between_cd); // reference return value

   test_gcxgc(170,  10, // point a
              170,  10, // point b
              -10, -10, // point c
              -10, -10, // point d
              170,  10, // reference point p
              -10, -10, // reference point q
              p_between_ab + q_between_cd); // reference return value

   // both edges have length zero and match

   test_gcxgc(-10, -10, // point a
              -10, -10, // point b
              -10, -10, // point c
              -10, -10, // point d
              -10, -10, // reference point p
              170,  10, // reference point q
              p_between_ab + p_between_cd); // reference return value

   return TEST_EXIT_CODE;
}

void test_gcxgc_rad(double lon_a, double lat_a, double lon_b, double lat_b,
                    double lon_c, double lat_c, double lon_d, double lat_d,
                    double lon_ref_p, double lat_ref_p,
                    double lon_ref_q, double lat_ref_q, int ref_ret_val) {

   test_cxc_rad(lon_a, lat_a, lon_b, lat_b, lon_c, lat_c, lon_d, lat_d,
                GREAT_CIRCLE, GREAT_CIRCLE,
                lon_ref_p, lat_ref_p, lon_ref_q, lat_ref_q, ref_ret_val,
                yac_gcxgc, yac_gcxgc_vec);
}

void test_gcxgc(double lon_a, double lat_a, double lon_b, double lat_b,
                double lon_c, double lat_c, double lon_d, double lat_d,
                double lon_ref_p, double lat_ref_p,
                double lon_ref_q, double lat_ref_q, int ref_ret_val) {

   test_cxc(lon_a, lat_a, lon_b, lat_b, lon_c, lat_c, lon_d, lat_d,
            GREAT_CIRCLE, GREAT_CIRCLE,
            lon_ref_p, lat_ref_p, lon_ref_q, lat_ref_q, ref_ret_val,
            yac_gcxgc, yac_gcxgc_vec);
}
