#define VERBOSE

/**
 * @file test_interpolation_plan.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include "tests.h"
#include "interpolation_plan.h"

int main (void) {

   unsigned i;

   {
      struct interpolation_plan_element plan;

      unsigned num_points = 5;
      unsigned fixed_value = 1337.0;

      yac_init_interpolation_plan_element_fixed(&plan, num_points, fixed_value);

      struct interpolation_plan_element *plan_stack[] = {&plan, NULL};

      double output_data[] = {-1, -1, -1, -1, -1, -1};

      yac_execute_interpolation_plan_element(plan_stack, NULL, output_data);

      double ref_output_data[] = {fixed_value, fixed_value, fixed_value,
                                  fixed_value, fixed_value, -1};

      for (i = 0; i < sizeof(ref_output_data) / sizeof(ref_output_data[0]); ++i)
         if (ref_output_data[i] != output_data[i])
            PUT_ERR("error in output_data\n");

      yac_free_interpolation_plan_element(&plan);
   }

   {
      struct interpolation_plan_element plan_1, plan_2;

      unsigned num_points_1 = 5;
      unsigned num_src_per_tgt[5] = {1,2,3,2,1};
      unsigned src_indices[][3] = {{0,0,0}, {0,0,0},{0,0,1}, {0,0,0},{0,0,1},{0,0,2},
                                   {1,0,0},{1,0,1}, {1,0,0}};
      double weights[] = {1, 0.5,0.5, 0.25,0.25,0.25, 0.5,0.5, 1};
      unsigned num_remote_points = 3;

      yac_init_interpolation_plan_element_wpointsum(&plan_1, num_points_1,
                                                    (unsigned*)src_indices,
                                                    num_src_per_tgt, weights,
                                                    &num_remote_points, 1);

      unsigned num_points_2 = 3;
      unsigned fixed_value = 1337.0;

      yac_init_interpolation_plan_element_fixed(&plan_2, num_points_2, fixed_value);

      struct interpolation_plan_element *plan_stack[] = {&plan_1, &plan_2, NULL};

      double raw_input_data[] = {1,2,3, 4,5};
      double *subdomain_input_data[2][1] = {{raw_input_data+0}, {raw_input_data+3}};
      double **input_data[] = {subdomain_input_data[0], subdomain_input_data[1]};
      double output_data[] = {-1,-1,-1,-1,-1,-1,-1,-1,-1};

      yac_execute_interpolation_plan_element(plan_stack, input_data, output_data);

      double ref_output_data[] = {1, 1.5, 1.5, 4.5, 4, fixed_value, fixed_value, fixed_value, -1};

      for (i = 0; i < sizeof(ref_output_data) / sizeof(ref_output_data[0]); ++i)
         if (ref_output_data[i] != output_data[i])
            PUT_ERR("error in output_data\n");

      yac_free_interpolation_plan_element(&plan_1);
      yac_free_interpolation_plan_element(&plan_2);
   }

   {
      struct interpolation_plan_element plan_1, plan_2;

      unsigned num_points_1 = 5;
      unsigned src_indices[][3] = {{0,0,0}, {0,0,1}, {0,0,2}, {1,0,1}, {1,0,0}};
      unsigned num_remote_points = 3;

      yac_init_interpolation_plan_element_point(
         &plan_1, num_points_1, (unsigned*)src_indices, &num_remote_points, 1);

      unsigned num_points_2 = 3;
      unsigned fixed_value = 1337.0;

      yac_init_interpolation_plan_element_fixed(&plan_2, num_points_2, fixed_value);

      struct interpolation_plan_element *plan_stack[] = {&plan_1, &plan_2, NULL};

      double raw_input_data[] = {1,2,3, 4,5};
      double *subdomain_input_data[2][1] = {{raw_input_data+0}, {raw_input_data+3}};
      double **input_data[] = {subdomain_input_data[0], subdomain_input_data[1]};
      double output_data[] = {-1,-1,-1,-1,-1,-1,-1,-1,-1};

      yac_execute_interpolation_plan_element(plan_stack, input_data, output_data);

      double ref_output_data[] =
         {1, 2, 3, 5, 4, fixed_value, fixed_value, fixed_value, -1};

      for (i = 0; i < sizeof(ref_output_data) / sizeof(ref_output_data[0]); ++i)
         if (ref_output_data[i] != output_data[i])
            PUT_ERR("error in output_data\n");

      yac_free_interpolation_plan_element(&plan_1);
      yac_free_interpolation_plan_element(&plan_2);
   }

   {
#define NUM_PLANS 6
      struct interpolation_plan_element plan[NUM_PLANS];
      unsigned num_pointsets = 2;

      {
        unsigned num_points[2] = {5, 3};
        unsigned num_src_per_tgt[2][5] = {{1,2,3,2,1}, {3,3,3,-1,-1}};
        unsigned src_indices[2][9][3] =
          {{{0,0,0}, {0,0,0},{0,0,1}, {0,0,0},{0,0,1},{0,0,2},
            {1,0,0},{1,0,1}, {1,0,0}},
           {{0,0,0},{1,0,0},{2,0,0}, {0,1,0},{1,1,0},{2,1,0},
            {0,0,1},{0,1,1},{1,1,1}}};
        double weights[2][9] = {{1,2,3,4,5,6,7,8,9},
                                {10,11,12,13,14,15,16,17,18}};
        unsigned num_remote_points[2][2] = {{3,0},{2,2}};
        unsigned plan_idx[2] = {0, 3};

        for (unsigned i = 0; i < 2; ++i)
          yac_init_interpolation_plan_element_wpointsum(&plan[plan_idx[i]],
                                                        num_points[i],
                                                        (unsigned*)src_indices[i],
                                                        num_src_per_tgt[i],
                                                        weights[i],
                                                        num_remote_points[i],
                                                        num_pointsets);
      }

      { // test yac_get_link_data

        unsigned num_points[4] = {3, 4, 5, 6};
        double fixed_values[4] = {1.0, 2.0, 3.0, 2.0};
        unsigned plan_idx[4] = {1, 2, 4, 5};

        for (unsigned i = 0; i < 4; ++i)
          yac_init_interpolation_plan_element_fixed(&plan[plan_idx[i]],
                                                    num_points[i],
                                                    fixed_values[i]);
      }

      struct interpolation_plan_element *plan_stack[NUM_PLANS + 1];

      for (unsigned i = 0; i < NUM_PLANS; ++i) plan_stack[i] = &plan[i];
      plan_stack[NUM_PLANS] = NULL;

      {
        int raw_global_indices[3][2][5] =
          {{{1,2,3,4,5},{6,7}},{{8,9},{10,11}},{{12},{13}}};
        int *subdomain_global_indices[3][2];
        int **global_indices[3];

        for (unsigned i = 0; i < 3; ++i) {
          for (unsigned j = 0; j < 2; ++j)
            subdomain_global_indices[i][j] = raw_global_indices[i][j];
          global_indices[i] = subdomain_global_indices[i];
        }

        unsigned num_targets;
        unsigned * num_links_per_point_set;
        unsigned num_weights_per_link;
        unsigned * num_links_per_tgt;
        int * src_indices;
        unsigned * pointset_indices;
        double * weights;

        yac_get_link_data(plan_stack, global_indices, num_pointsets,
                          &num_targets, &num_links_per_point_set,
                          &num_weights_per_link, &num_links_per_tgt,
                          &src_indices, &pointset_indices, &weights);

        unsigned ref_num_targets = 26;
        unsigned ref_num_links_per_point_set[2] = {13, 5};
        unsigned ref_num_weights_per_link = 1;
        unsigned ref_num_links_per_tgt[26] = {1,2,3,2,1, 0,0,0, 0,0,0,0, 3,3,3,
                                              0,0,0,0,0, 0,0,0,0,0,0};
        int ref_src_indices[18] = {1,1,2,1,2,3,8,9,8, 4,8,12,6,10,13,5,7,11};
        unsigned ref_pointset_indices[18] = {0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,1,1};
        double ref_weights[18] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18};

        if (num_targets != ref_num_targets)
          PUT_ERR("wrong number of targets\n");

        if (num_weights_per_link != ref_num_weights_per_link)
          PUT_ERR("wrong number of weights per link\n");

        for (unsigned i = 0; i < num_pointsets; ++i)
          if (num_links_per_point_set[i] != ref_num_links_per_point_set[i])
            PUT_ERR("wrong number of links per point set\n");

        for (unsigned i = 0, offset = 0; i < num_targets; ++i) {
          if (num_links_per_tgt[i] != ref_num_links_per_tgt[i])
            PUT_ERR("wrong number of links for a target\n");

          for (unsigned j = 0; j < ref_num_links_per_tgt[i]; ++j, ++offset) {
            if (src_indices[offset] != ref_src_indices[offset])
              PUT_ERR("wrong source index\n");
            if (pointset_indices[offset] != ref_pointset_indices[offset])
              PUT_ERR("wrong pointset index\n");
            if (weights[offset] != ref_weights[offset])
              PUT_ERR("wrong weight\n");
          }

        }

        free(num_links_per_point_set);
        free(num_links_per_tgt);
        free(src_indices);
        free(pointset_indices);
        free(weights);
      }

      { // test yac_get_fixed_data

        int global_ids[] = {-1,-1,-1,-1,-1, 3,2,1, 12,18,10,7, -1,-1,-1,
                            8,9,5,11,6, 13,14,15,17,4,16};
        int * global_tgt_indices;
        unsigned num_tgt_indices;
        unsigned * num_tgt_per_fixed_value;
        double * fixed_values;
        unsigned num_fixed_values;

        yac_get_fixed_data(plan_stack, global_ids, &global_tgt_indices,
                           &num_tgt_indices, &num_tgt_per_fixed_value,
                           &fixed_values, &num_fixed_values);

        unsigned ref_global_tgt_indices[18] = {1,2,3,
                                               4,7,10,12,13,14,15,16,17,18,
                                               5,6,8,9,11};
        unsigned ref_num_tgt_indices = 18;
        unsigned ref_num_tgt_per_fixed_value[3] = {3, 10, 5};
        double ref_fixed_values[3] = {1.0, 2.0, 3.0};
        unsigned ref_num_fixed_values = 3;

        if (num_tgt_indices != ref_num_tgt_indices)
          PUT_ERR("wrong number of target indices\n");

        for (unsigned i = 0; i < num_tgt_indices; ++i)
          if (global_tgt_indices[i] != ref_global_tgt_indices[i])
            PUT_ERR("wrong global target index\n");

        if (num_fixed_values != ref_num_fixed_values)
          PUT_ERR("wrong number of fixed values\n");

        for (unsigned i = 0; i < num_fixed_values; ++i) {
          if (num_tgt_per_fixed_value[i] != ref_num_tgt_per_fixed_value[i])
            PUT_ERR("wrong number of targets per fixed value\n");
          if (fixed_values[i] != ref_fixed_values[i])
            PUT_ERR("wrong fixed value\n");
        }

        free(global_tgt_indices);
        free(num_tgt_per_fixed_value);
        free(fixed_values);
      }

      for (unsigned i = 0; i < NUM_PLANS; ++i)
        yac_free_interpolation_plan_element(plan_stack[i]);
#undef NUM_PLANS
   }

   { // check interpolation_plan_collection that contains no interpolation plans
      struct interpolation_plan_element plan[7];
      unsigned num_pointsets = 2;

      {
        unsigned num_points[2] = {5, 3};
        unsigned num_src_per_tgt[2][5] = {{1,2,3,2,1}, {3,3,3,-1,-1}};
        unsigned src_indices[2][9][3] =
          {{{0,0,0}, {0,0,0},{0,0,1}, {0,0,0},{0,0,1},{0,0,2},
            {1,0,0},{1,0,1}, {1,0,0}},
           {{0,0,0},{1,0,0},{2,0,0}, {0,1,0},{1,1,0},{2,1,0},
            {0,0,1},{0,1,1},{1,1,1}}};
        double weights[2][9] = {{1,2,3,4,5,6,7,8,9},
                                {10,11,12,13,14,15,16,17,18}};
        unsigned num_remote_points[2][2] = {{3,0},{2,2}};
        unsigned plan_idx[2] = {0, 3};

        for (unsigned i = 0; i < 2; ++i)
          yac_init_interpolation_plan_element_wpointsum(&plan[plan_idx[i]],
                                                        num_points[i],
                                                        (unsigned*)src_indices[i],
                                                        num_src_per_tgt[i],
                                                        weights[i],
                                                        num_remote_points[i],
                                                        num_pointsets);
      }

      {

        unsigned num_points[4] = {3, 4, 5, 6};
        double fixed_values[4] = {1.0, 2.0, 3.0, 2.0};
        unsigned plan_idx[4] = {1, 2, 4, 5};

        for (unsigned i = 0; i < 4; ++i)
          yac_init_interpolation_plan_element_fixed(&plan[plan_idx[i]],
                                                    num_points[i],
                                                    fixed_values[i]);
      }

      yac_init_interpolation_plan_element_collection(&plan[6], 0, NULL);

      struct interpolation_plan_element *plan_stack[7 + 1] =
      {&plan[0], &plan[1], &plan[2], &plan[3], &plan[6], &plan[4], &plan[5], NULL};

      {
        int raw_global_indices[3][2][5] =
          {{{1,2,3,4,5},{6,7}},{{8,9},{10,11}},{{12},{13}}};
        int *subdomain_global_indices[3][2];
        int **global_indices[3];

        for (unsigned i = 0; i < 3; ++i) {
          for (unsigned j = 0; j < 2; ++j)
            subdomain_global_indices[i][j] = raw_global_indices[i][j];
          global_indices[i] = subdomain_global_indices[i];
        }

        unsigned num_targets;
        unsigned * num_links_per_point_set;
        unsigned num_weights_per_link;
        unsigned * num_links_per_tgt;
        int * src_indices;
        unsigned * pointset_indices;
        double * weights;

        yac_get_link_data(plan_stack, global_indices, num_pointsets,
                          &num_targets, &num_links_per_point_set,
                          &num_weights_per_link, &num_links_per_tgt,
                          &src_indices, &pointset_indices, &weights);

        unsigned ref_num_targets = 26;
        unsigned ref_num_links_per_point_set[2] = {13, 5};
        unsigned ref_num_weights_per_link = 1;
        unsigned ref_num_links_per_tgt[26] = {1,2,3,2,1, 0,0,0, 0,0,0,0, 3,3,3,
                                              0,0,0,0,0, 0,0,0,0,0,0};
        int ref_src_indices[18] = {1,1,2,1,2,3,8,9,8, 4,8,12,6,10,13,5,7,11};
        unsigned ref_pointset_indices[18] = {0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,1,1};
        double ref_weights[18] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18};

        if (num_targets != ref_num_targets)
          PUT_ERR("wrong number of targets\n");

        if (num_weights_per_link != ref_num_weights_per_link)
          PUT_ERR("wrong number of weights per link\n");

        for (unsigned i = 0; i < num_pointsets; ++i)
          if (num_links_per_point_set[i] != ref_num_links_per_point_set[i])
            PUT_ERR("wrong number of links per point set\n");

        for (unsigned i = 0, offset = 0; i < num_targets; ++i) {
          if (num_links_per_tgt[i] != ref_num_links_per_tgt[i])
            PUT_ERR("wrong number of links for a target\n");

          for (unsigned j = 0; j < ref_num_links_per_tgt[i]; ++j, ++offset) {
            if (src_indices[offset] != ref_src_indices[offset])
              PUT_ERR("wrong source index\n");
            if (pointset_indices[offset] != ref_pointset_indices[offset])
              PUT_ERR("wrong pointset index\n");
            if (weights[offset] != ref_weights[offset])
              PUT_ERR("wrong weight\n");
          }

        }

        free(num_links_per_point_set);
        free(num_links_per_tgt);
        free(src_indices);
        free(pointset_indices);
        free(weights);
      }

      { // test yac_get_fixed_data

        int global_ids[] = {-1,-1,-1,-1,-1, 3,2,1, 12,18,10,7, -1,-1,-1,
                            8,9,5,11,6, 13,14,15,17,4,16};
        int * global_tgt_indices;
        unsigned num_tgt_indices;
        unsigned * num_tgt_per_fixed_value;
        double * fixed_values;
        unsigned num_fixed_values;

        yac_get_fixed_data(plan_stack, global_ids, &global_tgt_indices,
                           &num_tgt_indices, &num_tgt_per_fixed_value,
                           &fixed_values, &num_fixed_values);

        unsigned ref_global_tgt_indices[18] = {1,2,3,
                                               4,7,10,12,13,14,15,16,17,18,
                                               5,6,8,9,11};
        unsigned ref_num_tgt_indices = 18;
        unsigned ref_num_tgt_per_fixed_value[3] = {3, 10, 5};
        double ref_fixed_values[3] = {1.0, 2.0, 3.0};
        unsigned ref_num_fixed_values = 3;

        if (num_tgt_indices != ref_num_tgt_indices)
          PUT_ERR("wrong number of target indices\n");

        for (unsigned i = 0; i < num_tgt_indices; ++i)
          if (global_tgt_indices[i] != ref_global_tgt_indices[i])
            PUT_ERR("wrong global target index\n");

        if (num_fixed_values != ref_num_fixed_values)
          PUT_ERR("wrong number of fixed values\n");

        for (unsigned i = 0; i < num_fixed_values; ++i) {
          if (num_tgt_per_fixed_value[i] != ref_num_tgt_per_fixed_value[i])
            PUT_ERR("wrong number of targets per fixed value\n");
          if (fixed_values[i] != ref_fixed_values[i])
            PUT_ERR("wrong fixed value\n");
        }

        free(global_tgt_indices);
        free(num_tgt_per_fixed_value);
        free(fixed_values);
      }

      for (unsigned i = 0; plan_stack[i] != NULL; ++i)
        yac_free_interpolation_plan_element(plan_stack[i]);
   }

   { // check interpolation_plan_collection that contains only fixed
     // interpolations
      struct interpolation_plan_element plan[7];
      unsigned num_pointsets = 2;

      {
        unsigned num_points[2] = {5, 3};
        unsigned num_src_per_tgt[2][5] = {{1,2,3,2,1}, {3,3,3,-1,-1}};
        unsigned src_indices[2][9][3] =
          {{{0,0,0}, {0,0,0},{0,0,1}, {0,0,0},{0,0,1},{0,0,2},
            {1,0,0},{1,0,1}, {1,0,0}},
           {{0,0,0},{1,0,0},{2,0,0}, {0,1,0},{1,1,0},{2,1,0},
            {0,0,1},{0,1,1},{1,1,1}}};
        double weights[2][9] = {{1,2,3,4,5,6,7,8,9},
                                {10,11,12,13,14,15,16,17,18}};
        unsigned num_remote_points[2][2] = {{3,0},{2,2}};
        unsigned plan_idx[2] = {0, 3};

        for (unsigned i = 0; i < 2; ++i)
          yac_init_interpolation_plan_element_wpointsum(&plan[plan_idx[i]],
                                                        num_points[i],
                                                        (unsigned*)src_indices[i],
                                                        num_src_per_tgt[i],
                                                        weights[i],
                                                        num_remote_points[i],
                                                        num_pointsets);
      }

      {

        unsigned num_points[4] = {3, 4, 5, 6};
        double fixed_values[4] = {1.0, 2.0, 3.0, 2.0};
        unsigned plan_idx[4] = {1, 2, 4, 5};

        for (unsigned i = 0; i < 4; ++i)
          yac_init_interpolation_plan_element_fixed(&plan[plan_idx[i]],
                                                    num_points[i],
                                                    fixed_values[i]);
      }

      {
        struct interpolation_plan_element * plans[2] =
          {&plan[1],&plan[2]};

        yac_init_interpolation_plan_element_collection(&plan[6], 2, plans);
      }

      struct interpolation_plan_element *plan_stack[5 + 1] =
      {&plan[0], &plan[6], &plan[3], &plan[4], &plan[5], NULL};

      {
        int raw_global_indices[3][2][5] =
          {{{1,2,3,4,5},{6,7}},{{8,9},{10,11}},{{12},{13}}};
        int *subdomain_global_indices[3][2];
        int **global_indices[3];

        for (unsigned i = 0; i < 3; ++i) {
          for (unsigned j = 0; j < 2; ++j)
            subdomain_global_indices[i][j] = raw_global_indices[i][j];
          global_indices[i] = subdomain_global_indices[i];
        }

        unsigned num_targets;
        unsigned * num_links_per_point_set;
        unsigned num_weights_per_link;
        unsigned * num_links_per_tgt;
        int * src_indices;
        unsigned * pointset_indices;
        double * weights;

        yac_get_link_data(plan_stack, global_indices, num_pointsets,
                          &num_targets, &num_links_per_point_set,
                          &num_weights_per_link, &num_links_per_tgt,
                          &src_indices, &pointset_indices, &weights);

        unsigned ref_num_targets = 26;
        unsigned ref_num_links_per_point_set[2] = {13, 5};
        unsigned ref_num_weights_per_link = 1;
        unsigned ref_num_links_per_tgt[26] = {1,2,3,2,1, 0,0,0, 0,0,0,0, 3,3,3,
                                              0,0,0,0,0, 0,0,0,0,0,0};
        int ref_src_indices[18] = {1,1,2,1,2,3,8,9,8, 4,8,12,6,10,13,5,7,11};
        unsigned ref_pointset_indices[18] = {0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,1,1};
        double ref_weights[18] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18};

        if (num_targets != ref_num_targets)
          PUT_ERR("wrong number of targets\n");

        if (num_weights_per_link != ref_num_weights_per_link)
          PUT_ERR("wrong number of weights per link\n");

        for (unsigned i = 0; i < num_pointsets; ++i)
          if (num_links_per_point_set[i] != ref_num_links_per_point_set[i])
            PUT_ERR("wrong number of links per point set\n");

        for (unsigned i = 0, offset = 0; i < num_targets; ++i) {
          if (num_links_per_tgt[i] != ref_num_links_per_tgt[i])
            PUT_ERR("wrong number of links for a target\n");

          for (unsigned j = 0; j < ref_num_links_per_tgt[i]; ++j, ++offset) {
            if (src_indices[offset] != ref_src_indices[offset])
              PUT_ERR("wrong source index\n");
            if (pointset_indices[offset] != ref_pointset_indices[offset])
              PUT_ERR("wrong pointset index\n");
            if (weights[offset] != ref_weights[offset])
              PUT_ERR("wrong weight\n");
          }

        }

        free(num_links_per_point_set);
        free(num_links_per_tgt);
        free(src_indices);
        free(pointset_indices);
        free(weights);
      }

      { // test yac_get_fixed_data

        int global_ids[] = {-1,-1,-1,-1,-1, 3,2,1, 12,18,10,7, -1,-1,-1,
                            8,9,5,11,6, 13,14,15,17,4,16};
        int * global_tgt_indices;
        unsigned num_tgt_indices;
        unsigned * num_tgt_per_fixed_value;
        double * fixed_values;
        unsigned num_fixed_values;

        yac_get_fixed_data(plan_stack, global_ids, &global_tgt_indices,
                           &num_tgt_indices, &num_tgt_per_fixed_value,
                           &fixed_values, &num_fixed_values);

        unsigned ref_global_tgt_indices[18] = {1,2,3,
                                               4,7,10,12,13,14,15,16,17,18,
                                               5,6,8,9,11};
        unsigned ref_num_tgt_indices = 18;
        unsigned ref_num_tgt_per_fixed_value[3] = {3, 10, 5};
        double ref_fixed_values[3] = {1.0, 2.0, 3.0};
        unsigned ref_num_fixed_values = 3;

        if (num_tgt_indices != ref_num_tgt_indices)
          PUT_ERR("wrong number of target indices\n");

        for (unsigned i = 0; i < num_tgt_indices; ++i)
          if (global_tgt_indices[i] != ref_global_tgt_indices[i])
            PUT_ERR("wrong global target index\n");

        if (num_fixed_values != ref_num_fixed_values)
          PUT_ERR("wrong number of fixed values\n");

        for (unsigned i = 0; i < num_fixed_values; ++i) {
          if (num_tgt_per_fixed_value[i] != ref_num_tgt_per_fixed_value[i])
            PUT_ERR("wrong number of targets per fixed value\n");
          if (fixed_values[i] != ref_fixed_values[i])
            PUT_ERR("wrong fixed value\n");
        }

        free(global_tgt_indices);
        free(num_tgt_per_fixed_value);
        free(fixed_values);
      }

      for (unsigned i = 0; plan_stack[i] != NULL; ++i)
        yac_free_interpolation_plan_element(plan_stack[i]);
   }

   { // check interpolation_plan_collection that contains only fixed
     // interpolations
      struct interpolation_plan_element plan[7];
      unsigned num_pointsets = 2;

      {
        unsigned num_points[2] = {5, 3};
        unsigned num_src_per_tgt[2][5] = {{1,2,3,2,1}, {3,3,3,-1,-1}};
        unsigned src_indices[2][9][3] =
          {{{0,0,0}, {0,0,0},{0,0,1}, {0,0,0},{0,0,1},{0,0,2},
            {1,0,0},{1,0,1}, {1,0,0}},
           {{0,0,0},{1,0,0},{2,0,0}, {0,1,0},{1,1,0},{2,1,0},
            {0,0,1},{0,1,1},{1,1,1}}};
        double weights[2][9] = {{1,2,3,4,5,6,7,8,9},
                                {10,11,12,13,14,15,16,17,18}};
        unsigned num_remote_points[2][2] = {{3,0},{2,2}};
        unsigned plan_idx[2] = {0, 3};

        for (unsigned i = 0; i < 2; ++i)
          yac_init_interpolation_plan_element_wpointsum(&plan[plan_idx[i]],
                                                        num_points[i],
                                                        (unsigned*)src_indices[i],
                                                        num_src_per_tgt[i],
                                                        weights[i],
                                                        num_remote_points[i],
                                                        num_pointsets);
      }

      {

        unsigned num_points[4] = {3, 4, 5, 6};
        double fixed_values[4] = {1.0, 2.0, 3.0, 2.0};
        unsigned plan_idx[4] = {1, 2, 4, 5};

        for (unsigned i = 0; i < 4; ++i)
          yac_init_interpolation_plan_element_fixed(&plan[plan_idx[i]],
                                                    num_points[i],
                                                    fixed_values[i]);
      }

      {
        struct interpolation_plan_element * plans[2] =
          {&plan[1],&plan[2]};

        yac_init_interpolation_plan_element_collection(&plan[6], 2, plans);
      }

      struct interpolation_plan_element *plan_stack[5 + 1] =
      {&plan[0], &plan[6], &plan[3], &plan[4], &plan[5], NULL};

      {
        int raw_global_indices[3][2][5] =
          {{{1,2,3,4,5},{6,7}},{{8,9},{10,11}},{{12},{13}}};
        int *subdomain_global_indices[3][2];
        int **global_indices[3];

        for (unsigned i = 0; i < 3; ++i) {
          for (unsigned j = 0; j < 2; ++j)
            subdomain_global_indices[i][j] = raw_global_indices[i][j];
          global_indices[i] = subdomain_global_indices[i];
        }

        unsigned num_targets;
        unsigned * num_links_per_point_set;
        unsigned num_weights_per_link;
        unsigned * num_links_per_tgt;
        int * src_indices;
        unsigned * pointset_indices;
        double * weights;

        yac_get_link_data(plan_stack, global_indices, num_pointsets,
                          &num_targets, &num_links_per_point_set,
                          &num_weights_per_link, &num_links_per_tgt,
                          &src_indices, &pointset_indices, &weights);

        unsigned ref_num_targets = 26;
        unsigned ref_num_links_per_point_set[2] = {13, 5};
        unsigned ref_num_weights_per_link = 1;
        unsigned ref_num_links_per_tgt[26] = {1,2,3,2,1, 0,0,0, 0,0,0,0, 3,3,3,
                                              0,0,0,0,0, 0,0,0,0,0,0};
        int ref_src_indices[18] = {1,1,2,1,2,3,8,9,8, 4,8,12,6,10,13,5,7,11};
        unsigned ref_pointset_indices[18] = {0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,1,1};
        double ref_weights[18] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18};

        if (num_targets != ref_num_targets)
          PUT_ERR("wrong number of targets\n");

        if (num_weights_per_link != ref_num_weights_per_link)
          PUT_ERR("wrong number of weights per link\n");

        for (unsigned i = 0; i < num_pointsets; ++i)
          if (num_links_per_point_set[i] != ref_num_links_per_point_set[i])
            PUT_ERR("wrong number of links per point set\n");

        for (unsigned i = 0, offset = 0; i < num_targets; ++i) {
          if (num_links_per_tgt[i] != ref_num_links_per_tgt[i])
            PUT_ERR("wrong number of links for a target\n");

          for (unsigned j = 0; j < ref_num_links_per_tgt[i]; ++j, ++offset) {
            if (src_indices[offset] != ref_src_indices[offset])
              PUT_ERR("wrong source index\n");
            if (pointset_indices[offset] != ref_pointset_indices[offset])
              PUT_ERR("wrong pointset index\n");
            if (weights[offset] != ref_weights[offset])
              PUT_ERR("wrong weight\n");
          }

        }

        free(num_links_per_point_set);
        free(num_links_per_tgt);
        free(src_indices);
        free(pointset_indices);
        free(weights);
      }

      { // test yac_get_fixed_data

        int global_ids[] = {-1,-1,-1,-1,-1, 3,2,1, 12,18,10,7, -1,-1,-1,
                            8,9,5,11,6, 13,14,15,17,4,16};
        int * global_tgt_indices;
        unsigned num_tgt_indices;
        unsigned * num_tgt_per_fixed_value;
        double * fixed_values;
        unsigned num_fixed_values;

        yac_get_fixed_data(plan_stack, global_ids, &global_tgt_indices,
                           &num_tgt_indices, &num_tgt_per_fixed_value,
                           &fixed_values, &num_fixed_values);

        unsigned ref_global_tgt_indices[18] = {1,2,3,
                                               4,7,10,12,13,14,15,16,17,18,
                                               5,6,8,9,11};
        unsigned ref_num_tgt_indices = 18;
        unsigned ref_num_tgt_per_fixed_value[3] = {3, 10, 5};
        double ref_fixed_values[3] = {1.0, 2.0, 3.0};
        unsigned ref_num_fixed_values = 3;

        if (num_tgt_indices != ref_num_tgt_indices)
          PUT_ERR("wrong number of target indices\n");

        for (unsigned i = 0; i < num_tgt_indices; ++i)
          if (global_tgt_indices[i] != ref_global_tgt_indices[i])
            PUT_ERR("wrong global target index\n");

        if (num_fixed_values != ref_num_fixed_values)
          PUT_ERR("wrong number of fixed values\n");

        for (unsigned i = 0; i < num_fixed_values; ++i) {
          if (num_tgt_per_fixed_value[i] != ref_num_tgt_per_fixed_value[i])
            PUT_ERR("wrong number of targets per fixed value\n");
          if (fixed_values[i] != ref_fixed_values[i])
            PUT_ERR("wrong fixed value\n");
        }

        free(global_tgt_indices);
        free(num_tgt_per_fixed_value);
        free(fixed_values);
      }

      for (unsigned i = 0; plan_stack[i] != NULL; ++i)
        yac_free_interpolation_plan_element(plan_stack[i]);
   }

   { // check interpolation_plan_collection that contains only wpointsum
     // interpolations
      struct interpolation_plan_element plan[7];
      unsigned num_pointsets = 2;

      {
        unsigned num_points[2] = {5, 3};
        unsigned num_src_per_tgt[2][5] = {{1,2,3,2,1}, {3,3,3,-1,-1}};
        unsigned src_indices[2][9][3] =
          {{{0,0,0}, {0,0,0},{0,0,1}, {0,0,0},{0,0,1},{0,0,2},
            {1,0,0},{1,0,1}, {1,0,0}},
           {{0,0,0},{1,0,0},{2,0,0}, {0,1,0},{1,1,0},{2,1,0},
            {0,0,1},{0,1,1},{1,1,1}}};
        double weights[2][9] = {{1,2,3,4,5,6,7,8,9},
                                {10,11,12,13,14,15,16,17,18}};
        unsigned num_remote_points[2][2] = {{3,0},{2,2}};
        unsigned plan_idx[2] = {2, 3};

        for (unsigned i = 0; i < 2; ++i)
          yac_init_interpolation_plan_element_wpointsum(&plan[plan_idx[i]],
                                                        num_points[i],
                                                        (unsigned*)src_indices[i],
                                                        num_src_per_tgt[i],
                                                        weights[i],
                                                        num_remote_points[i],
                                                        num_pointsets);
      }

      {

        unsigned num_points[4] = {3, 4, 5, 6};
        double fixed_values[4] = {1.0, 2.0, 3.0, 2.0};
        unsigned plan_idx[4] = {0, 1, 4, 5};

        for (unsigned i = 0; i < 4; ++i)
          yac_init_interpolation_plan_element_fixed(&plan[plan_idx[i]],
                                                    num_points[i],
                                                    fixed_values[i]);
      }

      {
        struct interpolation_plan_element * plans[2] =
          {&plan[2],&plan[3]};

        yac_init_interpolation_plan_element_collection(&plan[6], 2, plans);
      }

      struct interpolation_plan_element *plan_stack[5 + 1] =
      {&plan[0], &plan[1], &plan[6], &plan[4], &plan[5], NULL};

      {
        int raw_global_indices[3][2][5] =
          {{{1,2,3,4,5},{6,7}},{{8,9},{10,11}},{{12},{13}}};
        int *subdomain_global_indices[3][2];
        int **global_indices[3];

        for (unsigned i = 0; i < 3; ++i) {
          for (unsigned j = 0; j < 2; ++j)
            subdomain_global_indices[i][j] = raw_global_indices[i][j];
          global_indices[i] = subdomain_global_indices[i];
        }

        unsigned num_targets;
        unsigned * num_links_per_point_set;
        unsigned num_weights_per_link;
        unsigned * num_links_per_tgt;
        int * src_indices;
        unsigned * pointset_indices;
        double * weights;

        yac_get_link_data(plan_stack, global_indices, num_pointsets,
                          &num_targets, &num_links_per_point_set,
                          &num_weights_per_link, &num_links_per_tgt,
                          &src_indices, &pointset_indices, &weights);

        unsigned ref_num_targets = 26;
        unsigned ref_num_links_per_point_set[2] = {13, 5};
        unsigned ref_num_weights_per_link = 1;
        unsigned ref_num_links_per_tgt[26] = {0,0,0, 0,0,0,0, 1,2,3,2,1, 3,3,3,
                                              0,0,0,0,0, 0,0,0,0,0,0};
        int ref_src_indices[18] = {1,1,2,1,2,3,8,9,8, 4,8,12,6,10,13,5,7,11};
        unsigned ref_pointset_indices[18] = {0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,1,1};
        double ref_weights[18] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18};

        if (num_targets != ref_num_targets)
          PUT_ERR("wrong number of targets\n");

        if (num_weights_per_link != ref_num_weights_per_link)
          PUT_ERR("wrong number of weights per link\n");

        for (unsigned i = 0; i < num_pointsets; ++i)
          if (num_links_per_point_set[i] != ref_num_links_per_point_set[i])
            PUT_ERR("wrong number of links per point set\n");

        for (unsigned i = 0, offset = 0; i < num_targets; ++i) {
          if (num_links_per_tgt[i] != ref_num_links_per_tgt[i])
            PUT_ERR("wrong number of links for a target\n");

          for (unsigned j = 0; j < ref_num_links_per_tgt[i]; ++j, ++offset) {
            if (src_indices[offset] != ref_src_indices[offset])
              PUT_ERR("wrong source index\n");
            if (pointset_indices[offset] != ref_pointset_indices[offset])
              PUT_ERR("wrong pointset index\n");
            if (weights[offset] != ref_weights[offset])
              PUT_ERR("wrong weight\n");
          }

        }

        free(num_links_per_point_set);
        free(num_links_per_tgt);
        free(src_indices);
        free(pointset_indices);
        free(weights);
      }

      { // test yac_get_fixed_data

        int global_ids[] = {3,2,1, 12,18,10,7, -1,-1,-1,-1,-1, -1,-1,-1,
                            8,9,5,11,6, 13,14,15,17,4,16};
        int * global_tgt_indices;
        unsigned num_tgt_indices;
        unsigned * num_tgt_per_fixed_value;
        double * fixed_values;
        unsigned num_fixed_values;

        yac_get_fixed_data(plan_stack, global_ids, &global_tgt_indices,
                           &num_tgt_indices, &num_tgt_per_fixed_value,
                           &fixed_values, &num_fixed_values);

        unsigned ref_global_tgt_indices[18] = {1,2,3,
                                               4,7,10,12,13,14,15,16,17,18,
                                               5,6,8,9,11};
        unsigned ref_num_tgt_indices = 18;
        unsigned ref_num_tgt_per_fixed_value[3] = {3, 10, 5};
        double ref_fixed_values[3] = {1.0, 2.0, 3.0};
        unsigned ref_num_fixed_values = 3;

        if (num_tgt_indices != ref_num_tgt_indices)
          PUT_ERR("wrong number of target indices\n");

        for (unsigned i = 0; i < num_tgt_indices; ++i)
          if (global_tgt_indices[i] != ref_global_tgt_indices[i])
            PUT_ERR("wrong global target index\n");

        if (num_fixed_values != ref_num_fixed_values)
          PUT_ERR("wrong number of fixed values\n");

        for (unsigned i = 0; i < num_fixed_values; ++i) {
          if (num_tgt_per_fixed_value[i] != ref_num_tgt_per_fixed_value[i])
            PUT_ERR("wrong number of targets per fixed value\n");
          if (fixed_values[i] != ref_fixed_values[i])
            PUT_ERR("wrong fixed value\n");
        }

        free(global_tgt_indices);
        free(num_tgt_per_fixed_value);
        free(fixed_values);
      }

      for (unsigned i = 0; plan_stack[i] != NULL; ++i)
        yac_free_interpolation_plan_element(plan_stack[i]);
   }

   return TEST_EXIT_CODE;
}
