/**
 * @file test_gcxgc.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include "test_cxc.h"
#include "search.h"

void test_latcxlatc(double lon_a, double lat_a, double lon_b, double lat_b,
                    double lon_c, double lat_c, double lon_d, double lat_d,
                    double lon_ref_p, double lat_ref_p,
                    double lon_ref_q, double lat_ref_q, int ref_ret_val);

int main (void) {

   unsigned const p_between_ab = 1 << 0;
   unsigned const q_between_ab = 1 << 1;
   unsigned const p_between_cd = 1 << 2;
   unsigned const q_between_cd = 1 << 3;
   unsigned const circles_are_identically = 1 << 4;

   // two circles of latitude on the same plane that intersect

   for (int i = 0; i < 2; ++i) {

      double lon_coords[2][2] = {{-10, 5}, {-5, 10}};
      test_latcxlatc(lon_coords[i][0],   30, // point a
                     lon_coords[i][1],   30, // point b
                     lon_coords[i^1][0], 30, // point c
                     lon_coords[i^1][1], 30, // point d
                     -5,                 30, // reference point p
                      5,                 30, // reference point q
                     p_between_ab + p_between_cd + q_between_ab + q_between_cd +
                     circles_are_identically); // reference return value
   }

   for (int i = 0; i < 2; ++i) {

      double lon_coords[2][2] = {{20, 25}, {30, 10}};
      test_latcxlatc(lon_coords[i][0],   -15, // point a
                     lon_coords[i][1],   -15, // point b
                     lon_coords[i^1][0], -15, // point c
                     lon_coords[i^1][1], -15, // point d
                     20,                 -15, // reference point p
                     25,                 -15, // reference point q
                     p_between_ab + p_between_cd + q_between_ab + q_between_cd +
                     circles_are_identically); // reference return value
   }

   // two circles of latitude on the same plane that do not intersect

   for (int i = 0; i < 2; ++i) {

      double lon_coords[2][2] = {{-80, -70}, {20, 30}};
      test_latcxlatc(lon_coords[i][0],   10, // point a
                     lon_coords[i][1],   10, // point b
                     lon_coords[i^1][0], 10, // point c
                     lon_coords[i^1][1], 10, // point d
                     lon_coords[i][0],   10, // reference point p
                     lon_coords[i][1],   10, // reference point q
                     p_between_ab + q_between_ab +
                     circles_are_identically); // reference return value
   }

   // two circles of latitude on different planes that do not intersect

   for (int i = 0; i < 2; ++i) {

      double lat_coords[2] = {0, 5};
      test_latcxlatc(20, lat_coords[i],   // point a
                     30, lat_coords[i],   // point b
                     20, lat_coords[i^1], // point c
                     30, lat_coords[i^1], // point d
                     -1, -1,              // reference point p
                     -1, -1,              // reference point q
                     -1); // reference return value

      test_latcxlatc(-80, lat_coords[i],   // point a
                     -70, lat_coords[i],   // point b
                      20, lat_coords[i^1], // point c
                      30, lat_coords[i^1], // point d
                      -1, -1,              // reference point p
                      -1, -1,              // reference point q
                     -1); // reference return value
   }

   // two circles of latitude on the pole

   for (int i = 0; i < 2; ++i) {

      double lat_coords[2] = {90, -90};
      test_latcxlatc(  0, lat_coords[i], // point a
                      10, lat_coords[i], // point b
                     180, lat_coords[i], // point c
                     190, lat_coords[i], // point d
                       0, lat_coords[i],  // reference point p
                       0, lat_coords[i],  // reference point q
                     p_between_ab + p_between_cd +
                     circles_are_identically); // reference return value
   }

   for (int i = 0; i < 2; ++i) {

      double lat_coords[2] = {90, -90};
      test_latcxlatc( 0, lat_coords[i], // point a
                     10, lat_coords[i], // point b
                      5, lat_coords[i], // point c
                     15, lat_coords[i], // point d
                      0, lat_coords[i],  // reference point p
                      0, lat_coords[i],  // reference point q
                     p_between_ab + p_between_cd +
                     circles_are_identically); // reference return value
   }

   // two circles of latitude on different planes with length zero

   for (int i = 0; i < 2; ++i) {

      double lat_coords[2] = {-10, 10};
      test_latcxlatc(45    , lat_coords[i],   // point a
                     45    , lat_coords[i],   // point b
                     90    , lat_coords[i^1], // point c
                     90    , lat_coords[i^1], // point d
                     -1    , -1,              // reference point p
                     -1    , -1,              // reference point q
                     -1); // reference return value
   }

   // two circles of latitude on the same plane that do not intersect
   // with length zero

   test_latcxlatc(45    , 10, // point a
                  45    , 10, // point b
                  90    , 10, // point c
                  90    , 10, // point d
                  45    , 10, // reference point p
                  45+180, 10, // reference point q
                  p_between_ab + circles_are_identically); // reference return value

   // two circles of latitude on the same plane that do intersect
   // with length zero

   test_latcxlatc(90, 10, // point a
                  90, 10, // point b
                  90, 10, // point c
                  90, 10, // point d
                  90, 10, // reference point p
                  90, 10, // reference point q
                  p_between_ab + p_between_cd +
                  circles_are_identically); // reference return value

   return TEST_EXIT_CODE;
}

void test_latcxlatc(double lon_a, double lat_a, double lon_b, double lat_b,
                    double lon_c, double lat_c, double lon_d, double lat_d,
                    double lon_ref_p, double lat_ref_p,
                    double lon_ref_q, double lat_ref_q, int ref_ret_val) {

   test_cxc(lon_a, lat_a, lon_b, lat_b, lon_c, lat_c, lon_d, lat_d,
            LAT_CIRCLE, LAT_CIRCLE,
            lon_ref_p, lat_ref_p, lon_ref_q, lat_ref_q, ref_ret_val,
            yac_latcxlatc, yac_latcxlatc_vec);
}
