/**
 * @file test_mask.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include "tests.h"
#include "grid.h"
#include "grid_reg2d.h"
#include "points.h"
#include "mask.h"

int main (void) {

   unsigned num_cells[2] = {2,2};
   unsigned cyclic[2] = {0,0};
   struct grid * grid[2];
   struct points points_corner, points_cell;

   { // set up data structures required for the testing the mask

      grid[0] = yac_reg2d_grid_new(NULL, NULL, num_cells, cyclic);
      yac_init_points(&points_corner, grid[0], CORNER, NULL, NULL);

      grid[1] = yac_reg2d_grid_new(NULL, NULL, num_cells, cyclic);
      yac_init_points(&points_cell, grid[1], CELL, NULL, NULL);
   }

   {
      struct mask mask;

      unsigned mask_values[] = {0,1,0,1};

      yac_init_mask(&mask, mask_values, &points_cell);

      yac_free_mask(&mask);
   }

   {
      struct mask mask;

      unsigned mask_values[] = {0,0,0,1,1,1,0,0,0};

      yac_init_mask(&mask, mask_values, &points_corner);

      yac_free_mask(&mask);
   }

   // clean up
   yac_free_points(&points_cell);
   yac_free_points(&points_corner);
   yac_delete_grid(grid[0]);
   yac_delete_grid(grid[1]);

   return TEST_EXIT_CODE;
}
