/**
 * @file test_points.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include "tests.h"
#include "grid.h"
#include "grid_reg2d.h"
#include "points.h"

int main (void) {

   unsigned i;

   { // testing points that are defined on the corners of the base grid
      //---------------
      // setup
      //---------------
      double grid_coord_x[3] = {0,1,2};
      double grid_coord_y[3] = {0,1,2};
      double points_coord_x[3] = {0.1,1.1,2.1};
      double points_coord_y[3] = {0.1,1.1,2.1};
      unsigned num_cells[2] = {2,2};
      unsigned cyclic[2] = {0,0};
      struct grid * reg_grid;
      struct points points;
      struct grid * point_grid;
      unsigned coordinate_array_sizes[2];
      unsigned ref_coordinate_array_sizes[2] = {3, 3};

      reg_grid = yac_reg2d_grid_new(grid_coord_x, grid_coord_y, num_cells, cyclic);

      //---------------
      // testing
      //---------------
      yac_init_points(&points, reg_grid, CORNER, points_coord_x, points_coord_y);

      point_grid = yac_get_point_grid(&points);

      if (yac_get_num_grid_corners(point_grid) != 9) PUT_ERR("error in get_num_grid_corners\n")
      if (yac_get_num_grid_cells(point_grid) != 4) PUT_ERR("error in get_num_grid_cells\n")
      if (yac_get_num_grid_edges(point_grid) != 12) PUT_ERR("error in get_num_grid_edges\n")

      yac_get_coordinate_array_sizes(reg_grid, CORNER, coordinate_array_sizes);
      if (coordinate_array_sizes[0] != ref_coordinate_array_sizes[0] ||
          coordinate_array_sizes[1] != ref_coordinate_array_sizes[1])
         PUT_ERR("error in get_coordinate_array_sizes\n")
      if (yac_get_data_size(points) != 9)
         PUT_ERR("error in get_data_size\n")

      double ref_coordinates[9][2] = {{0.1,0.1},{1.1,0.1},{2.1,0.1},
                                      {0.1,1.1},{1.1,1.1},{2.1,1.1},
                                      {0.1,2.1},{1.1,2.1},{2.1,2.1}};
      double coordinates[2];
      for (i = 0; i < 9; ++i) {
         yac_get_point_coordinates(&points, i, coordinates);
         if (coordinates[0] != ref_coordinates[i][0] ||
             coordinates[1] != ref_coordinates[i][1])
            PUT_ERR("error in get_point_coordinates");
      }

      yac_free_points(&points);
      yac_delete_grid(reg_grid);
   }

   { // testing points that are defined in the cells of the base grid
      //---------------
      // setup
      //---------------
      double grid_coord_x[3] = {0,1,2};
      double grid_coord_y[3] = {0,1,2};
      double points_coord_x[2] = {0.5,1.5};
      double points_coord_y[2] = {0.5,1.5};
      unsigned num_cells[2] = {2,2};
      unsigned cyclic[2] = {0,0};
      struct grid * reg_grid;
      struct points points;
      struct grid * point_grid;
      unsigned coordinate_array_sizes[2];
      unsigned ref_coordinate_array_sizes[2] = {2, 2};

      reg_grid = yac_reg2d_grid_new(grid_coord_x, grid_coord_y, num_cells, cyclic);

      //---------------
      // testing
      //---------------
      yac_init_points(&points, reg_grid, CELL, points_coord_x, points_coord_y);

      point_grid = yac_get_point_grid(&points);

      if (yac_get_num_grid_corners(point_grid) != 4) PUT_ERR("error in get_num_grid_corners\n")
      if (yac_get_num_grid_cells(point_grid) != 1) PUT_ERR("error in get_num_grid_cells\n")
      if (yac_get_num_grid_edges(point_grid) != 4) PUT_ERR("error in get_num_grid_edges\n")

      yac_get_coordinate_array_sizes(reg_grid, CELL, coordinate_array_sizes);
      if (coordinate_array_sizes[0] != ref_coordinate_array_sizes[0] ||
          coordinate_array_sizes[1] != ref_coordinate_array_sizes[1])
         PUT_ERR("error in get_coordinate_array_sizes\n")
      if (yac_get_data_size(points) != 4)
         PUT_ERR("error in get_data_size\n")

      double ref_coordinates[4][2] = {{0.5,0.5},{1.5,0.5},
                                      {0.5,1.5},{1.5,1.5}};
      double coordinates[2];
      for (i = 0; i < 4; ++i) {
         yac_get_point_coordinates(&points, i, coordinates);
         if (coordinates[0] != ref_coordinates[i][0] ||
             coordinates[1] != ref_coordinates[i][1])
            PUT_ERR("error in get_point_coordinates");
      }

      yac_free_points(&points);
      yac_delete_grid(reg_grid);
   }

   { // testing points that are defined on the edges of the base grid
      //---------------
      // setup
      //---------------
      double grid_coord_x[3] = {0,1,2};
      double grid_coord_y[3] = {0,1,2};
      double points_coord_x[12] = {0.5,0.0,1.5,1.0,2.0, 0.5,0.0,1.5,1.0,2.0, 0.5,1.5};
      double points_coord_y[12] = {0.0,0.5,0.0,0.5,0.5, 1.0,1.5,1.0,1.5,1.5, 2.0,2.0};
      unsigned num_cells[2] = {2,2};
      unsigned cyclic[2] = {0,0};
      struct grid * reg_grid;
      struct points points;
      unsigned coordinate_array_sizes[2];
      unsigned ref_coordinate_array_sizes[2] = {12, 12};

      reg_grid = yac_reg2d_grid_new(grid_coord_x, grid_coord_y, num_cells, cyclic);

      //---------------
      // testing
      //---------------
      yac_init_points(&points, reg_grid, EDGE, points_coord_x, points_coord_y);

      yac_get_coordinate_array_sizes(reg_grid, EDGE, coordinate_array_sizes);
      if (coordinate_array_sizes[0] != ref_coordinate_array_sizes[0] ||
          coordinate_array_sizes[1] != ref_coordinate_array_sizes[1])
         PUT_ERR("error in get_coordinate_array_sizes\n")
      if (yac_get_data_size(points) != 12)
         PUT_ERR("error in get_data_size\n")

      double coordinates[2];
      for (i = 0; i < 4; ++i) {
         yac_get_point_coordinates(&points, i, coordinates);
         if (coordinates[0] != points_coord_x[i] ||
             coordinates[1] != points_coord_y[i])
            PUT_ERR("error in get_point_coordinates");
      }

      yac_free_points(&points);
      yac_delete_grid(reg_grid);
   }

   return TEST_EXIT_CODE;
}
