/**
 * @file test_sphere_part.c
 *
 * @copyright Copyright  (C)  2014 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include <stdlib.h>
#include <stdio.h>

#include "tests.h"
#include "grid_search.h"
#include "read_icon_grid.h"
#include "grid.h"
#include "sphere_part.h"
#include "grid_reg2d.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

int file_exists(const char * filename);

#ifdef HAVE_C_NETCDF

int main(void) {

  {
      if (!file_exists("iconR2B00-grid.nc"))
         return EXIT_SKIP_TEST;

      struct grid * icon_grid = read_icon_grid("iconR2B00-grid.nc");

      struct grid_search * search = yac_sphere_part_search_new(icon_grid);

      yac_delete_grid_search(search);

      yac_delete_grid(icon_grid);
   }

   {
      struct grid * reg_grid;
      unsigned num_cells[2] = {20,20};
      unsigned cyclic[2] = {1,0};
      double coordinates_x[20];
      double coordinates_y[21];

      for (unsigned i = 0; i < 20; ++i)
         coordinates_x[i] = (0.0 + (double)i * 18.0) * rad;

      for (unsigned i = 0; i < 21; ++i)
         coordinates_y[i] = (-90.0 + (double)i * 9.0) * rad;

      reg_grid = yac_reg2d_grid_new(coordinates_x, coordinates_y,
                                    num_cells, cyclic);

      struct grid_search * search = yac_sphere_part_search_new(reg_grid);

      struct bounding_circle extent;
      yac_get_grid_bounding_circle(reg_grid, &extent);

      struct dep_list result_list;
      yac_do_bnd_circle_search(search, &extent, 1, &result_list);

      if (result_list.num_elements != 1)
        PUT_ERR("wrong number of elements in result list");

      if (result_list.num_deps_per_element[0] != 20*20)
        PUT_ERR("wrong number of dependencies in result list");

      yac_delete_grid_search(search);

      yac_delete_grid(reg_grid);
   }

   return TEST_EXIT_CODE;
}

int file_exists(const char * filename) {

   FILE * file = fopen(filename, "r");

   if (file != NULL) {
      fclose(file);
      return 1;
   }
   return 0;
}

#else

int main(void) {
   return EXIT_SKIP_TEST;
}
#endif
