!!
!! @file test_quicksort.c
!!
!! @copyright Copyright  (C)  2013 DKRZ, MPI-M
!!
!! @version 1.0
!!
!! @author Moritz Hanke <hanke@dkrz.de>
!!         Rene Redler  <rene.redler@mpimet.mpg.de>
!!
!!
!
!  Keywords:
! 
!  Maintainer: Moritz Hanke <hanke@dkrz.de>
!              Rene Redler  <rene.redler@mpimet.mpg.de>
! 
!  URL: https://doc.redmine.dkrz.de/YAC/html/index.html
! 
!  This file is part of YAC.
! 
!  YAC is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
! 
!  YAC is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
! 
!  You should have received a copy of the GNU General Public License
!  along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
!

module utest

   implicit none

   private

   public :: start_test, stop_test, start_timer, stop_timer, &
             test_v_r, test_r, exit_tests

   integer :: utest_testcount, utest_errorcount
   real    :: start, end

contains

   subroutine start_test (name)
      character(len=*), intent(in) :: name
      utest_errorcount = 0
      utest_testcount = 0
#ifdef VERBOSE
      if (.true.) then
#else
      if (.false.) then
#endif
         print *, "testing ", name, ":"
      endif
   end subroutine start_test

   subroutine stop_test ()
#ifdef VERBOSE
      if (.true.) then
#else
      if (.false.) then
#endif
         if (utest_errorcount == 0) then
            print *, "all",utest_testcount, "test(s) succeeded"
         else
            write(0,*) utest_errorcount, " out of ", &
                       utest_testcount, " test(s) failed"
         endif
      endif
   end subroutine stop_test

   subroutine start_timer (name)
      character(len=*), intent(in) :: name
      call cpu_time(start)
#ifdef VERBOSE
      if (.true.) then
#else
      if (.false.) then
#endif
         print *, "start timing ", name
      endif
   end subroutine start_timer

   subroutine stop_timer ()
      call cpu_time(end)
#ifdef VERBOSE
      if (.true.) then
#else
      if (.false.) then
#endif
         print '(" elapsed time:",f6.3," seconds.")',end-start
      endif
   end subroutine stop_timer

   subroutine test_v_r(arg, file, line)

      logical, intent(in) :: arg
      integer, intent(in) :: line
      character(len=*), intent(in) :: file

      utest_testcount = utest_testcount + 1
      if (arg) then
#ifdef VERBOSE
         if (.true.) then
#else
         if (.false.) then
#endif
            print *, "- test", utest_testcount, "succeeded"
         endif
         return
      else
         utest_errorcount = utest_errorcount + 1
#ifdef VERBOSE
         if (.true.) then
#else
         if (.false.) then
#endif
            write(0,*) "- test", utest_testcount, "failed (",file," :", line, ")"
         endif
      endif

   end subroutine test_v_r

   subroutine test_r(arg, file, line)

      logical, intent(in) :: arg
      integer, intent(in) :: line
      character(len=*), intent(in) :: file

      utest_testcount = utest_testcount + 1
      if (arg) then
#ifdef VERBOSE
         if (.true.) then
#else
         if (.false.) then
#endif
            print *, "- test", utest_testcount, "succeeded"
         endif
         return
      else
         utest_errorcount = utest_errorcount + 1
#ifdef VERBOSE
         if (.true.) then
#else
         if (.false.) then
#endif
            write(0,*) "- test", utest_testcount, "failed (",file," :", line, ")"
         endif
      endif

   end subroutine test_r

   subroutine exit_tests()

      if (utest_errorcount > 0) then
         stop 1
      endif
   end subroutine exit_tests

end module utest
